// *---------------------------------------------------------------------------*
//  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
//  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.
//
//  Copyright (C)2011-2014 Nintendo  All rights reserved.
//
//  These coded instructions, statements, and computer programs contain
//  proprietary information of Nintendo and/or its licensors and are 
//  protected by national and international copyright laws.
//  They may not be disclosed to third parties or copied or duplicated
//  in any form, in whole or in part, without the prior written consent
//  of Nintendo.
//  The content herein is highly confidential and should be handled
//  accordingly.
// *---------------------------------------------------------------------------*

//
// DDLBasic.ddl
//

// The following #include is required for host migration
#use <NetZ/src/Core/HostMigration/HostMigrationExt.ddl>

title "DDLBasic";

// General data. Must be sent via a reliable channel, and the changes to duplicas must be reflected by executing Update after the values are updated.
// 
dataset Position
{
    float x;
    float y;
};

// Configure transmissions on an untrusted channel with an extrapolation filter
dataset ExtrapolatedPosition
{
	float x;
	float y;
} extrapolation_filter, unreliable;

// In addition to the ExtrapolatedPosition, apply interpolation from the extrapolation filter to the duplication master
dataset ExtrapolatedLoopbackPosition
{
	float x;
	float y;
} extrapolation_filter, unreliable, loopback;

// Other information
// Record the station ID
dataset AvatarInfo
{
    int32 stationID;
};

// Definition of the avatar class
// This normally has a position, values updated by the extrapolation filter, and other parameters
doclass Avatar
{
    AvatarInfo m_dsInfo;
    Position m_dsPosition;
	ExtrapolatedPosition m_dsExtrapolatedPosition;
	ExtrapolatedLoopbackPosition m_dsExtrapolatedLoopbackPosition;
};

// Define this to be constant
dataset Boundaries
{
    float minX;
    float maxX;
    float minY;
    float maxY;
} constant;

// Definition of a single world object
doclass World
{
    Boundaries m_dsBoundaries;
};

// Define World as a well-known object
// This is created by the session master, and it is guaranteed that a copy will be created for each station prior to when the session begins.
// 
wellknown World g_hWorld;

