﻿/*--------------------------------------------------------------------------
  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <nex_P2p.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/UpdatableTimeProvider.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>

#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>

#include "../common/StatisticValue.h"

#include <OnlineCore/src/Transport/UDP/Packet.h>

#include "../common/NexUtil.h"

#include "DeadReckoningDDF.h"

#include "../common/NetZManager.h"
#include "../common/ErrorHandling.h"
#include "../common/Version.h"
#include "../common/HeapManager.h"

#include "SceneSelectNetZSession.h"
#include "SceneError.h"
#include "ExtrapolatedLoopbackPosition.h"
#include "ExtrapolatedPosition.h"

#ifdef CTR_LOCAL_COMMUNICATION
#error
#endif

#if defined(CTR_TARGET)
#include <nn.h>
#include <nn/Result.h>
#include <nn/hid.h>
#endif

//n1769
#include "../common/DemoFramework.h"

//SZ_FMT is defined as "%s" or L"%ls" depending on the compile options.


//n2619
namespace
{


//=============================================================================
// A class doing only AcquireInstance() and ReleaseInstance().
//=============================================================================
class GlobalVariablesManager
{
public:
    GlobalVariablesManager()
    {
        nn::nex::GlobalVariables::AcquireInstance();
    }
    ~GlobalVariablesManager()
    {
        nn::nex::GlobalVariables::ReleaseInstance();
    }
};

#if !defined(CAFE_TARGET)
#pragma push
#endif /* !CAFE_TARGET */
#pragma diag_suppress 111

qInt GenericMain(qInt argc, qChar8** argv)
{
    (void)argc;
    (void)argv;

    // For `AcquireInstance` and `ReleaseInstance` in `GlobalVariables`, this happens in the constructor and destructor.
    // 
    GlobalVariablesManager oGVManager;

    ErrorHandling::Initialize();

    nn::nex::NetZ::SetDOClassInitializationCallback(&nn::nex::InitDOClasses);

    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);

    nn::nex::TraceLog::GetInstance()->SetFlag(
        TRACE_ALWAYS
#if 0
        //        |TRACE_INIT
        //        | TRACE_UPDATE
        //      | TRACE_PHBDR
        //        | TRACE_OPERATION
        |TRACE_FAULT
        //        |TRACE_DUPSPACE
        //        | TRACE_LOADBALANCING
        //        | TRACE_TEARDOWN
        //        | TRACE_SESSIONDISCOVERY
        //        |TRACE_CONNECTION
        //        |TRACE_NAT
        //        | TRACE_SCHEDULER
        //        | TRACE_TIME

        |D_TRACE_THREAD
        //      | D_TRACE_CONSTRUCT
        //      | D_TRACE_TRANSPORT
        //      | D_TRACE_TRANSPORT_END_POINT
        //      | D_TRACE_DO_MESSAGE_BUNDLING
        //      | D_TRACE_TRANSPORT_SEND_MULTI
        //      |D_TRACE_TRANSPORT_BUNDLING
        //      | D_TRACE_TRANSPORT_RELIABLE
        |D_TRACE_TRANSPORT_ROUTING
        //      | D_TRACE_TRANSPORT_PACKET

#if defined(CTR_TARGET)
        //        |TRACE_UDS
        //        |D_TRACE_TRANSPORT_UDS
#endif
#endif
    );

    nn::nex::UtilTrace::GetInstance()->SetFlag(
        SAMPLE_TRACE_ALWAYS
#if 1
        |SAMPLE_TRACE_ERROR
        |SAMPLE_TRACE_BUTTON
        //      |SAMPLE_TRACE_UDS
        |SAMPLE_TRACE_TRANSPORT

        //      |SAMPLE_TRACE_UPDATE
        |SAMPLE_TRACE_ROUTE

        |SAMPLE_TRACE_SCENE
        |SAMPLE_TRACE_AVATAR
        |SAMPLE_TRACE_WORLD
        //      |SAMPLE_TRACE_POSITION
#endif
    );

    //n1589
#if 0
    TestUnsignedInt64(D_TRACE_DEBUG);
    TestUnsignedInt64(D_TRACE_ALL);
    TestUnsignedInt32(D_TRACE_DEBUG);
    TestUnsignedInt32(D_TRACE_ALL);

    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("SHIFT_TEST1:") << hex << SHIFT_TEST1);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("SHIFT_TEST2:") << hex << SHIFT_TEST2);
#endif

    nn::nex::ConsoleIO::Banner(_T("NEX DeadReckoning sample")); // Print banner

    // Create Framework object.
    DemoFramework::CreateInstance();
    DemoFramework::GetInstance()->SetFramePerSecond(FRAME_PER_SECOND);

    NetZManager::CreateInstance();

    SceneBase *pScene = SceneSelectNetZSession::GetInstance();
    pScene->Initialize();

    ExtrapolatedPosition::GetErrorToleranceFunction()->SetQuadraticError(
        QREAL(0.1),
        1,
        50
    );
    ExtrapolatedLoopbackPosition::GetErrorToleranceFunction()->SetQuadraticError(
        QREAL(0.1),
        1,
        50
    );

    while (1) // Main loop
    {
        //      Platform::Sleep(100);

        DemoFramework::GetInstance()->BeginFrame();

        //n1589: For checking termination process
        if(
            DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_L) &&
            DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_R) &&
            DemoFramework::GetInstance()->IsTrigger(DEMO_BUTTON_X))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("GenericMain:BUTTON_L & BUTTON_R & BUTTON_X"));
            break;
        }

        SceneBase *pNext = NULL;
        if((pScene!=SceneError::GetInstance()) && ErrorHandling::IsErrorDetected())
        {
            // When an error is detected for the first time, prepare to transition to SceneError.
            pNext = SceneError::GetInstance();
        }
        else
        {
            pNext = pScene->Update();
        }
        NEX_DEMO_ASSERT(pNext != NULL);

        pScene->Render();

        // State Transitions
        if(pNext != pScene)
        {
            _UTIL_TRACE(
                SAMPLE_TRACE_SCENE,
                (
                    _T("Changing Scene %d to %d"),
                    pScene->GetSceneId(),
                    pNext->GetSceneId()
                )
            );

            pScene->Finalize();
            pNext->SetParentScene(pScene);
            pScene = pNext;
            pScene->Initialize();
        }

        DemoFramework::GetInstance()->EndFrame();
    }

    NetZManager::GetInstance()->TerminateNetZ();
    NetZManager::GetInstance()->DestroyNetZ();
    NetZManager::DestroyInstance();

    // Destroy Framework object.
    DemoFramework::DestroyInstance();

    ErrorHandling::Finalize();

    NEX_DEMO_LOG("Program end.");
    while(1)
    {
    }
    // (Unreachable exit)
    return EXIT_SUCCESS;
}

#if !defined(CAFE_TARGET)
#pragma pop
#endif /* !CAFE_TARGET */

}//anonymous-namespace

//-----------------------------------------------------------------------------
// Platform-Specific Main Function
//-----------------------------------------------------------------------------
MAIN_WRAPPER(GenericMain)
