﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/




#include "SceneBase.h"

#include <NetZ/src/Core/DOCore/Session.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"
#include "../common/NetZManager.h"
#include "../common/HeapManager.h"
#include "GlobalStatus.h"

namespace
{
}  // End of anonymous namespace


SceneBase::SceneBase(void)
{
    mSceneId = SCENE_ID_BASE;
    m_pParentScene = NULL;
}


SceneBase::~SceneBase(void)
{
}


void SceneBase::Initialize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneBase::Initialize"));

    g_gsStatus.m_uiUpdateCount = 0;
    g_gsStatus.m_uiProcessNexCount = 0;
    g_gsStatus.m_bMeasuring = false;
}


void SceneBase::Finalize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneBase::Finalize"));
}

SceneBase* SceneBase::Update(void)
{
    g_gsStatus.m_uiUpdateCount ++;

    if((g_gsStatus.m_uiUpdateCount % PROCESS_NEX_PERIOD) == 0)
    {
        ProcessNex();
    }

    return this;
}

//n1589
qBool SceneBase::CheckNexError()
{
#if ENABLE_CHECK_ERROR
    if(nn::nex::SystemError::IsError())
    {
        qUnsignedInt32 lastError = nn::nex::SystemError::GetLast();
        qUnsignedInt32 lastErrorExt = nn::nex::SystemError::GetLastExt();
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("SceneBase::CheckNexError:SystemError:Error:") << nn::nex::hex << lastError << _T(" ErrorExt:") << nn::nex::hex << lastErrorExt);

        qChar szError[256] = _T("");
        nn::nex::SystemError::GetErrorString(nn::nex::SystemError::GetLast(), szError, 256);
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("SceneBase::CheckNexError:ErrorString:") << szError);

        return true;
    }
    else
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("SceneBase::CheckNexError:no error"));
    }
#endif
    return false;
}

void SceneBase::SetParentScene(SceneBase* parent)
{
    m_pParentScene = parent;
}

void SceneBase::ProcessNex(void)
{
    if(NetZManager::GetInstance()->IsValid())
    {
        g_gsStatus.m_uiNbLocalStations = NetZManager::GetInstance()->GetNbLocalStations();
        g_gsStatus.m_uiSessionRole = nn::nex::Session::GetRole();

        nn::nex::StreamSettings& rSettings = nn::nex::Stream::GetSettings(nn::nex::Stream::DO);
        g_gsStatus.m_uiMaxSilenceTime = rSettings.GetMaxSilenceTime();
        g_gsStatus.m_uiKeepAliveTimeout = rSettings.GetKeepAliveTimeout();

        nn::nex::Network* pNetwork = nn::nex::Network::GetInstance();
        if(pNetwork != NULL)
        {
            nn::nex::SocketTransport* transport = (nn::nex::SocketTransport*)pNetwork->GetRootTransport();
            if(transport != NULL)
            {
                g_gsStatus.m_iTransportProcessCount = transport->GetTransportProcessCount();
            }
        }
    }
    else
    {
        g_gsStatus.m_uiNbLocalStations = 0;
        g_gsStatus.m_uiSessionRole = 0;
        g_gsStatus.m_uiMaxSilenceTime = 0;
        g_gsStatus.m_uiKeepAliveTimeout = 0;
        g_gsStatus.m_iTransportProcessCount = -1;
    }

    /*
    n1589: To shorten latency, it is good to perform dispatch processing immediately after game object send data is updated
    Since in user thread mode the receive callback is called with the dispatch process, it is good to perform dispatch processing immediately before the game object receive data is updated
    
    */
    (void)NetZManager::GetInstance()->DispatchNetZJob(DISPATCH_TIMEOUT);
    //  ProcessCommunicationReceive();
    //n1589: Because SenderSendTime is updated with Receive, ProcessCommunicationSend is after this
    //  ProcessCommunicationSend();
    //  (void)NetZManager::GetInstance()->DispatchNetZJob(DISPATCH_TIMEOUT);

    g_gsStatus.m_uiProcessNexCount ++;
}

void SceneBase::RenderInfo(const char *sceneName) const
{
    RenderSceneTitle(sceneName);
    RenderInfoTop();
    RenderInfoBottom();
}

void SceneBase::RenderSceneTitle(const char *name) const
{
    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SCENE_TITLE_X, DRAW_POSITION_SCENE_TITLE_Y, name);
}

void SceneBase::RenderInfoTop(void) const
{
    DemoFramework::GetInstance()->DrawText(
        DRAW_POSITION_VERSION_X,
        DRAW_POSITION_VERSION_Y,
        SAMPLE_VERSION_STRING
    );

    int y = DRAW_POSITION_SESSION_Y;
    switch(g_gsStatus.m_uiSessionRole)
    {
    case nn::nex::SESSION_ROLE_MASTER:
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "Master");
        break;
    case nn::nex::SESSION_ROLE_DUPLICA:
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "Client");
        break;
    default:
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "Undefined");
        break;
    }
    y += DRAW_FONT_HEIGHT;

    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "NbLocalStations:%d", g_gsStatus.m_uiNbLocalStations);
    y += DRAW_FONT_HEIGHT;
    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "MaxSilenceTime:%d", g_gsStatus.m_uiMaxSilenceTime / 1000);
    y += DRAW_FONT_HEIGHT;
    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_SESSION_X, y, "KeepAliveTimeout:%d", g_gsStatus.m_uiKeepAliveTimeout / 1000);
    y += DRAW_FONT_HEIGHT;
}

void SceneBase::RenderInfoBottom(void) const
{
    DemoFramework::GetInstance()->DrawText(
        DRAW_POSITION_BUNDLE_X,
        DRAW_POSITION_BUNDLE_Y,
        "bundle:%d d:%d p:%d %d d:%d",
        g_gsStatus.m_bEnableMessageBundle,
        g_gsStatus.m_uiMessageBundleFlushDelay,
        g_gsStatus.m_uiMessageBundleFlushCheckPeriod,
        g_gsStatus.m_bEnablePacketBundle,
        g_gsStatus.m_uiPacketBundleFlushDelay
    );

    DemoFramework::GetInstance()->DrawText(
        DRAW_POSITION_FRAME_X,
        DRAW_POSITION_FRAME_Y,
        "v%06d j%d u%04d n%04d t%04d",
        DemoFramework::GetInstance()->GetVsyncCounter(),
        g_gsStatus.m_uiJoinCount,
        g_gsStatus.m_uiUpdateCount,
        g_gsStatus.m_uiProcessNexCount,
        g_gsStatus.m_iTransportProcessCount
    );
}
