﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/



#include <nex_P2p.h>
#include <NetZ/src/Core/DOCore/Session.h>

#include "../common/DemoFramework.h"

#include "../common/NetZManager.h"

#include "SceneSelectNetZSession.h"
#include "SceneExitNetZSession.h"
#include "SceneSessionJoining.h"
#include "SceneSessionParticipant.h"
#include "CommonData.h"

namespace
{

}  // end of anonymous namespace



SceneSessionJoining::SceneSessionJoining(void)
{
    mSceneId = SCENE_ID_SESSION_JOINING;
}


SceneSessionJoining::~SceneSessionJoining(void)
{
}


SceneBase* SceneSessionJoining::GetInstance(void)
{
    static SceneSessionJoining s_instance;
    return &s_instance;
}



void SceneSessionJoining::Initialize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionJoining::Initialize"));
    SceneBase::Initialize();

    qBool ret = false;

    ret = m_context.Reset();
    if(!ret)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("ERROR: CallContext::Reset() failed.")
        );
    }

    nn::nex::SessionDescription session = CommonData::GetInstance()->GetSessionToJoin();
    const qChar *urlString = session.GetStationURL();
    nn::nex::StationURL stationUrl(urlString);
    ret = stationUrl.IsValid();
    if(!ret)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("ERROR: IsValid() failed.")
        );
    }

    ret = nn::nex::Session::JoinSession(&m_context, stationUrl);
    if(!ret)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("ERROR: JoinSession() failed.")
        );
    }
}

void SceneSessionJoining::Finalize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionJoining::Finalize"));
}


SceneBase* SceneSessionJoining::Update(void)
{
    (void)SceneBase::Update();
    //n1589
    if(SceneBase::CheckNexError())
    {
        return SceneExitNetZSession::GetInstance();
    }

    switch(m_context.GetState())
    {
    case nn::nex::CallContext::Available:
        return this;
    case nn::nex::CallContext::CallInProgress:
        return this;
    case nn::nex::CallContext::CallSuccess:
        return SceneSessionParticipant::GetInstance();
    case nn::nex::CallContext::CallFailure:
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("SceneSessionJoining::Update:CallFailure.")
        );
        return SceneExitNetZSession::GetInstance();
    case nn::nex::CallContext::CallCancelled:
        return SceneExitNetZSession::GetInstance();
    default:
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("SceneSessionJoining::Update: Unknown CallContext state.")
        );
        return SceneExitNetZSession::GetInstance();
    }
}


void SceneSessionJoining::Render(void) const
{
    RenderInfo("<SceneSessionJoining>");
}
