﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>

#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>

#include "../common/StatisticValue.h"

#include "../common/NetZManager.h"
#include "../common/ErrorHandling.h"
#include "../common/Version.h"
#include "../common/HeapManager.h"

#include "Avatar.h"
#include "SceneSelectNetZSession.h"
#include "SceneError.h"
#include "GlobalStatus.h"

#include "../common/NexUtil.h"

//n1589
#if defined(CTR_TARGET)
#include <nn.h>
#include <nn/Result.h>
#include <nn/hid.h>

//#include <Platform/Core/CachedLogDevice.h>
#endif

//n1769
#include "../common/DemoFramework.h"

// When using the direct send feature (= ddl file is not defined), the following functions must be defined
namespace nn
{
namespace nex
{
void InitDOClasses() {}
}
}

//SZ_FMT is defined as "%s" or L"%ls" depending on the compile options.


nn::nex::StatisticValue m_svTimeRender;

static SceneBase* s_pCurrentScene = NULL;
void OnStationEventUpdated(nn::nex::StationOperation* pOperation);

namespace
{


void printVersionInfo(void)
{
    NEX_DEMO_LOG("----- Version information begin -----\n");
    Version().Print();
    PrintSampleCommonDescription();
    NEX_DEMO_LOG("----- Version information end   -----\n");
}

//=============================================================================
// A class doing only AcquireInstance() and ReleaseInstance().
//=============================================================================
class GlobalVariablesManager
{
public:
    GlobalVariablesManager()
    {
        nn::nex::GlobalVariables::AcquireInstance();
    }
    ~GlobalVariablesManager()
    {
        nn::nex::GlobalVariables::ReleaseInstance();
    }
};

#if !defined(CAFE_TARGET)
#pragma push
#endif /* !CAFE_TARGET */
#pragma diag_suppress 111

qInt GenericMain(qInt argc, qChar8** argv)
{
    (void)argc;
    (void)argv;

    //n1769
    printVersionInfo();


    //n2619
    // For GlobalVariables, the AcquireInstance function and the ReleaseInstance function are done with a constructor and destructor, respectively.
    // 
    //nn::nex::GlobalVariables::AcquireInstance();
    GlobalVariablesManager oGVManager;

    ErrorHandling::Initialize();

    HeapManager::Initialize();

    //n2619
#if 0
    nn::nex::TraceLog::GetInstance()->SetLogDevice( new nn::nex::CachedLogDevice( s_LogBuffer, 2 * 1024 ) );
#endif

    //  nn::nex::EventLog::LogLevel logLeve = nn::nex::EventLog::Error;
    //  nn::nex::EventLog::LogLevel logLeve = nn::nex::EventLog::Info;
    nn::nex::EventLog::LogLevel logLeve = nn::nex::EventLog::Debug;
    NEX_DEMO_LOG("GenericMain:SetLogLevel:%d\n", logLeve);
    nn::nex::EventLog::GetInstance()->SetLogLevel(logLeve);

    nn::nex::TraceLog::GetInstance()->SetFlag(
        TRACE_ALWAYS
#if 0
        //        |TRACE_INIT
        |TRACE_TERMINATE
        |TRACE_FAULT
        |TRACE_TIME
        //        |TRACE_CONNECTION
        //        |TRACE_TEARDOWN
        //        |TRACE_UPDATE
        |TRACE_TRANSPORT
        //      |TRACE_PHBDR
        //        |TRACE_OPERATION
        //        |TRACE_DUPSPACE
        //        |TRACE_LOADBALANCING
        //        |TRACE_SESSIONDISCOVERY
        //        |TRACE_NAT
        //        |TRACE_SCHEDULER

        |D_TRACE_DO_MIGRATION
        //      |D_TRACE_THREAD
        //      | D_TRACE_CONSTRUCT
        //      | D_TRACE_TRANSPORT
        //      | D_TRACE_TRANSPORT_END_POINT
        //      |D_TRACE_DO_MESSAGE_BUNDLING
        //      |D_TRACE_TRANSPORT_SEND_MULTI
        //      |D_TRACE_TRANSPORT_BUNDLING
        //      |D_TRACE_TRANSPORT_RELIABLE
        //      |D_TRACE_TRANSPORT_ROUTING
        //      |D_TRACE_TRANSPORT_PACKET

#if defined(CTR_TARGET)
        //        |TRACE_UDS
        //        |D_TRACE_TRANSPORT_UDS
#endif
#endif
    );

    nn::nex::UtilTrace::GetInstance()->SetFlag(
        SAMPLE_TRACE_ALWAYS
#if 0
        |SAMPLE_TRACE_ERROR
        //      |SAMPLE_TRACE_BUTTON
        //      |SAMPLE_TRACE_UDS
        |SAMPLE_TRACE_TRANSPORT

        //      |SAMPLE_TRACE_UPDATE
        |SAMPLE_TRACE_ROUTE

        |SAMPLE_TRACE_SCENE
        |SAMPLE_TRACE_AVATAR
        |SAMPLE_TRACE_WORLD
        //      |SAMPLE_TRACE_POSITION
        |SAMPLE_TRACE_LATENCY
#endif
    );

#if defined(WATERMARK_ENABLED)
    NEX_DEMO_LOG("WATERMARK_ENABLED\n");
#else
    NEX_DEMO_LOG("not WATERMARK_ENABLED\n");
#endif

#ifdef ENABLE_SYSTEM_MULTIUNITHEAP_ALLOCATOR
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("ENABLE_SYSTEM_MULTIUNITHEAP_ALLOCATOR"));
#endif
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("UNRELIABLE_DATA_SIZE:") << UNRELIABLE_DATA_SIZE);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("UNRELIABLE_DATA_SIZE:") << UNRELIABLE_DATA_SIZE);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("AVATAR_UNRELIABLE_DATA_SIZE:") << AVATAR_UNRELIABLE_DATA_SIZE);
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("RELIABLE_DATA_SIZE:") << RELIABLE_DATA_SIZE);
    NEX_DEMO_ASSERT(UNRELIABLE_DATA_SIZE >= AVATAR_UNRELIABLE_DATA_SIZE);

    //nn::nex::ConsoleIO::Banner(_T("NEX DirectStream sample"),
    //nn::nex::ProductInfo::FindInstance(NETZ_PRODUCTID)); // Print banner

    // Create Framework object and set FPS.
    DemoFramework::CreateInstance();
    DemoFramework::GetInstance()->SetFramePerSecond(FRAME_PER_SECOND);

    //n1769
#if 0
    // (with key)
    const qByte key[] = {1, 2, 3, 4, 5, 6, 7, 8};
    Network::GetInstance()->SetP2PDataPacketDefaultSignatureKey(key, sizeof(key));
#else
    // Do nothing.  (no key)
#endif

    NetZManager::CreateInstance();

    s_pCurrentScene = SceneSelectNetZSession::GetInstance();
    s_pCurrentScene->Initialize();

    qUnsignedInt64 startTime;
    while (1) // Main loop
    {
        DemoFramework::GetInstance()->BeginFrame();
        s_pCurrentScene->BeginFrame();

        SceneBase *pNext = NULL;
        if((s_pCurrentScene!=SceneError::GetInstance()) && ErrorHandling::IsErrorDetected())
        {
            // When an error is detected for the first time, prepare to transition to SceneError.
            pNext = SceneError::GetInstance();
        }
        else
        {
            pNext = s_pCurrentScene->Update();
        }
        NEX_DEMO_ASSERT(pNext != NULL);

        //n1589
        s_pCurrentScene->EndGameProcess();

        startTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
        s_pCurrentScene->Render();

        // State Transitions
        if(pNext != s_pCurrentScene)
        {
            _UTIL_TRACE(
                SAMPLE_TRACE_SCENE,
                (
                    _T("Changing Scene %d to %d"),
                    s_pCurrentScene->GetSceneId(),
                    pNext->GetSceneId()
                )
            );

            s_pCurrentScene->Finalize();
            pNext->SetParentScene(s_pCurrentScene);
            s_pCurrentScene = pNext;
            s_pCurrentScene->Initialize();
        }

        m_svTimeRender.AddValue((qUnsignedInt64)nn::nex::SystemClock::GetTime() - startTime);

        DemoFramework::GetInstance()->EndFrame();
    }

    // Unreachable code. Comment out to avoid the compile warnings...
    NetZManager::GetInstance()->TerminateNetZ();
    NetZManager::GetInstance()->DestroyNetZ();

    NetZManager::DestroyInstance();

    // Destroy Framework object.
    DemoFramework::DestroyInstance();

    HeapManager::Finalize();

    ErrorHandling::Finalize();

    NEX_DEMO_LOG("Program end.");
    while(1)
    {
    }

    // (Unreachable exit)
    return EXIT_SUCCESS;
}

#if !defined(CAFE_TARGET)
#pragma pop
#endif /* !CAFE_TAGET */

}//anonynmous-namespace

void OnStationEventUpdated(nn::nex::StationOperation* pOperation)
{
    //n1589: Since the station event callback is called from the dispatch thread, duplicated object operation is possible
    nn::nex::DOHandle hStation = pOperation->GetTargetStation()->GetHandle();
    nn::nex::Station::Ref rStation(hStation);
    UTIL_TRACE(
        SAMPLE_TRACE_ALWAYS,
        _T("OnStationEventUpdated:Station:") << nn::nex::hex << hStation
        << _T(", State:") << rStation->GetState()
        << _T(", Type:") << pOperation->GetType()
        << _T(", LocalStation:") << nn::nex::Station::GetLocalStation()
        << _T(", SceneId:") << s_pCurrentScene->GetSceneId()
    );
}

//-----------------------------------------------------------------------------
// Platform-Specific Main Function
//-----------------------------------------------------------------------------
MAIN_WRAPPER(GenericMain)
