/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "SampleCommon.h"

#include "../common/NexUtil.h"

namespace
{
#ifndef RELEASE_VERSION  // Avoid warning.
const char* getNetworkTopologyString(enum nn::nex::NetworkTopology::_NetworkTopology mode)
{
    switch(mode)
    {
    case nn::nex::NetworkTopology::Peer2Peer:
        return "P2P";
    case nn::nex::NetworkTopology::ClientServer:
        return "C/S";
    case nn::nex::NetworkTopology::Hybrid:
        return "Hybrid";
    default:
        return "???";
    }
}
#endif  // end of RELEASE_VERSION
}  // End of anonymous namespace


//n1769
//It may be convenient to copy & paste into the email for verification report.
#ifdef RELEASE_VERSION  // Avoid warning.
void PrintSampleCommonDescription(void)
{
}
#else
void PrintSampleCommonDescription(void)
{
    const char *STR_CONNECTION = "SOC";

    const char *STR_THREAD_MODE =
#if   THREAD_MODE==THREAD_MODE_UNSAFE_USER_SINGLE_THREAD
        "US"
#elif THREAD_MODE==THREAD_MODE_UNSAFE_USER_MULTI_THREAD
        "UM"
#elif THREAD_MODE==THREAD_MODE_SAFE_USER_THREAD
        "SU"
#elif THREAD_MODE==THREAD_MODE_INTERNAL
        "IN"
#elif THREAD_MODE==THREAD_MODE_UNSAFE_TRANSPORT_BUFFER
        "TB"
#elif THREAD_MODE==THREAD_MODE_SAFE_TRANSPORT_BUFFER
        "SB"
#else
        "??"
#endif
        ;

    const char *STR_TOPOLOGY = getNetworkTopologyString(NETWORK_TOPOLOGY);

    // Build type. DUI? RUI? RUS? ...
    char buildType[4] = {};
#ifdef DEBUG_VERSION
    buildType[0] = 'D';
#elif defined(RELEASE_VERSION)
    buildType[0] = 'R';
#else
    buildType[0] = '?';
#endif

#ifdef _UNICODE
    buildType[1] = 'U';  // Unicode
#else
    buildType[1] = 'A';  // ANSI
#endif

#ifdef INTERNAL_VERSION
    buildType[2] = 'I';
#else
    buildType[2] = 'S';
#endif

    buildType[3] = '\0';  // Just in case.

    u32 masterFrame = 0;
    u32 clientFrame = 0;

    NEX_DEMO_LOG("%s,%s,%s,%s,Dispatch%d,MBundle%d,PBundle%d,U%d,MFrame%d,CFrame%d,LeaveSession\n",
                 buildType,
                 STR_CONNECTION,
                 STR_THREAD_MODE,
                 STR_TOPOLOGY,
                 DISPATCH_TIMEOUT,
                 ENABLE_MESSAGE_BUNDLING,
                 ENABLE_PACKET_BUNDLING,
                 UNRELIABLE_DATA_SIZE,
                 masterFrame,
                 clientFrame
                );
}
#endif  // end of RELEASE_VERSION
