/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <NetZ/src/Core/DOCore/Session.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"

#include "../common/NetZManager.h"
#include "SceneExitNetZSession.h"
#include "SceneSessionCreating.h"
#include "SceneSessionMaster.h"

namespace
{

}  // End of anonymous namespace



SceneSessionCreating::SceneSessionCreating(void)
    :m_subState(UNKNOWN)
{
    mSceneId = SCENE_ID_SESSION_CREATING;
}


SceneSessionCreating::~SceneSessionCreating(void)
{
}


SceneBase* SceneSessionCreating::GetInstance(void)
{
    static SceneSessionCreating s_instance;
    return &s_instance;
}


void SceneSessionCreating::Initialize(void)
{
    SceneBase::Initialize();

    m_subState = UNKNOWN;

    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionCreating::Initialize:CreateProc"));
    CreateProc(this);
}

void SceneSessionCreating::Finalize()
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionCreating::Finalize"));

    SceneBase::Finalize();
}


SceneBase* SceneSessionCreating::Update(void)
{
    (void)SceneBase::Update();

    if(m_subState==SUCCESS)
    {
        return SceneSessionMaster::GetInstance();
    }
    else if(m_subState==FAILURE)
    {
        UTIL_TRACE(SAMPLE_TRACE_ERROR, _T("ERROR: CreateSession() failed."));
        nn::nex::ConsoleIO::Error();

        return SceneExitNetZSession::GetInstance();
    }
    else
    {
        return this;
    }
}


void SceneSessionCreating::Render(void) const
{
    RenderInfo("<SceneSessionCreating>");

    u16 y = DRAW_POSITION_OPERATION_Y;
    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "Creating session...");
    y += DRAW_FONT_HEIGHT;
}


void SceneSessionCreating::CreateProc(SceneSessionCreating *p)
{
    //n1769: Exclusion to the process in the dispatch thread.
    nn::nex::ScopedCS slNexLock(NetZManager::GetInstance()->GetNexLock());

    qBool ret = nn::nex::Session::CreateSession(SAMPLE_TITLE);
    if(ret)
    {
        p->m_subState = SUCCESS;
    }
    else
    {
        p->m_subState = FAILURE;
    }
}
