/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef CTR_DEMO_FRAMEWORK_H_
#define CTR_DEMO_FRAMEWORK_H_

#include <nn.h>

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#include "../DemoColor.h"

namespace demo
{
#ifdef DEMO_LIB_AVAILABLE
class DrawFrameworkEx;
#else
class RenderSystemDrawing;
#endif
}


namespace CTR
{

const bit32 DEMO_BUTTON_A = nn::hid::BUTTON_A;
const bit32 DEMO_BUTTON_B = nn::hid::BUTTON_B;
const bit32 DEMO_BUTTON_X = nn::hid::BUTTON_X;
const bit32 DEMO_BUTTON_Y = nn::hid::BUTTON_Y;
const bit32 DEMO_BUTTON_L = nn::hid::BUTTON_L;
const bit32 DEMO_BUTTON_R = nn::hid::BUTTON_R;
const bit32 DEMO_BUTTON_UP = nn::hid::BUTTON_UP;
const bit32 DEMO_BUTTON_DOWN = nn::hid::BUTTON_DOWN;
const bit32 DEMO_BUTTON_LEFT = nn::hid::BUTTON_LEFT;
const bit32 DEMO_BUTTON_RIGHT = nn::hid::BUTTON_RIGHT;

// The PadStatus or PadReader constructor requests that the call of nn::hid::Initialize be complete.
// 
// Therefore, to anticipate this, prepare an object exclusively for initializing and releasing hid.
class Hid
{
public:
    Hid(void)
    {
        nn::Result result = nn::hid::Initialize();
        NN_UTIL_PANIC_IF_FAILED(result);
    }
    ~Hid(void)
    {
        nn::hid::Finalize();
    }
};


/* Please see man pages for details 

 */
class DemoFramework
{
public:
    /* Please see man pages for details 
    
     */
    static void CreateInstance(void);


    /* Please see man pages for details 
    
     */
    static void DestroyInstance(void);


    /* Please see man pages for details 
    
    
     */
    static DemoFramework* GetInstance(void);


    /* Please see man pages for details 
    
    
     */
    static bool IsCreated(void);


    ~DemoFramework(void);

    /* Please see man pages for details 
    
     */
    void ClearBothDisplays(void);


    /* Please see man pages for details 
    
    
     */
    void SetClearColor(const struct DemoColor &clr);


    /* Please see man pages for details 
    

    
    
    
     */
    void DrawText(s32 x, s32 y, const char *fmt, ...);


    /* Please see man pages for details 
    

    
    
    
    
    
     */
    void DrawRect(s32 x, s32 y, s32 w, s32 h, const struct DemoColor &clr);


    /* Please see man pages for details 
    

    
    
     */
    void BeginFrame(void);


    /* Please see man pages for details 
    

    
    
     */
    void EndFrame(void);


    /* Please see man pages for details 
    

    

    
     */
    bool SetFramePerSecond(s32 fps);


    /* Please see man pages for details 
    

    
     */
    s32  GetFramePerSecond(void) const
    {
        return VSYNCS_PER_SECOND / m_vsyncInterval;
    }


    // The render target after this call is the bottom screen.
    // For SDK 0.10.2 or later, this call is ignored.
    void UseUpperDisplay(void);

    // The render target after this call is the bottom screen.
    // However, if BeginFrame function is called, the render target becomes the top screen.
    // For SDK 0.10.2 or later, this call is ignored.
    void UseLowerDisplay(void);


    /* Please see man pages for details 
    
    
    
     */
    bool IsHold(bit32 button) const;


    /* Please see man pages for details 
    
    
    
     */
    bool IsTrigger(bit32 button) const;


    /* Please see man pages for details 
    
    
    
     */
    bool IsRelease(bit32 button) const;


    /* Please see man pages for details 
    
    
     */
    bool IsPenTriggered(void) const;


    /* Please see man pages for details 
    
    
     */
    bool IsPenTouched(void) const;


    /* Please see man pages for details 
    
    
     */
    bool IsPenReleased(void) const;


    /* Please see man pages for details 
    
    
    
    
     */
    bool GetLatestPenPosition(nn::nex::qUnsignedInt16 *x, nn::nex::qUnsignedInt16 *y) const;


    /* Please see man pages for details 
    
    
     */
    f32 GetAnalogX(void);


    /* Please see man pages for details 
    
    
     */
    f32 GetAnalogY(void);


    /* Please see man pages for details 
    
    
    
     */
    void GetAnalog(f32 *x, f32 *y);


    /* Please see man pages for details 
    
    
    
    
    
     */
    bool GetAccelerationData(s16 *x, s16 *y, s16 *z) const;


    /* Please see man pages for details 
    
     */
    void PrintCurrentThreadInfo(void) const;


    /* Please see man pages for details 
    
    
     */
    void SleepCurrentThreadMilliseconds(s32 msec);


    // {Secret}
    // Gets the device memory.
    void* AllocateDeviceMemory(size_t size);


    // {Secret}
    // Restores the device memory.
    void FreeDeviceMemory(void *ptr);


    /* Please see man pages for details 
    
    
     */
    bool IsInMainThread(void) const;


    // {Secret}
    // Code to identify the padding warning.
    void Checker(void);


    /* Please see man pages for details 
    
    
     */
    void GetPassedTimeString(char str[]);

    /* Please see man pages for details 
    
    */
    s32 GetVsyncCounter() const
    {
        return nngxCheckVSync(NN_GX_DISPLAY0);
    };

private:
    static const size_t GX_HEAP_SIZE = 0x800000;

    static const nn::nex::qUnsignedInt16 INVALID_TOUCH_POS = 0xffff;

    static const s32 INVALID_DISPLAY = -1;

    static const s32 VSYNCS_PER_SECOND = 60;

    static const s16 INVALID_ACCELEROMETER_VALUE = 1024;

    DemoFramework(void);

    static DemoFramework      *s_pInstance;  // Instance for singleton

    Hid                       m_hid;        // Must bring this before m_padStatus or m_padReader!
    u8                        m_padding[3];

    uptr                      m_heapForGx;

    nn::hid::PadStatus        m_padStatus;
    nn::hid::PadReader        m_padReader;

    nn::hid::AccelerometerStatus m_accStatus;
    nn::hid::AccelerometerReader m_accReader;
    bool                         m_accValid;

    nn::hid::TouchPanelStatus m_tpStatus;
    nn::hid::TouchPanelStatus m_tpStatusPrev;
    nn::hid::TouchPanelReader m_tpReader;
    nn::nex::qUnsignedInt16            m_latestX;  // X-coordinate where the stylus last touched
    nn::nex::qUnsignedInt16            m_latestY;  // Y-coordinate where the stylus last touched

#ifdef DEMO_LIB_AVAILABLE
    demo::DrawFrameworkEx    *m_pFramework;
#else
    demo::RenderSystemDrawing *m_pFramework;
#endif

    nn::fnd::ExpHeap          m_appHeap;

    s32                       m_renderingTarget;

    s32                       m_vsyncInterval;  // Variable to support FPS.60 -> 1, 30 -> 2, 20 -> 3, ...

    GLint                     m_beginFrameVsync;  // VSYNC value when BeginFrame function was called
};


}  // end of namespace CTR

#endif  // end of CTR_DEMO_FRAMEWORK_H_
