/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "DispatchThread.h"

#include "../UtilTrace.h"

#include "../NetZManager.h"

#include "../Defines.h"


namespace
{

}  // End of anonymous namespace

namespace CTR
{

//
DispatchThread::DispatchThread(void)
    :m_bDispatchThreadRunning(false),
     m_bPrecise(false),
     m_threadMode(THREAD_MODE_VOID),
     m_updateFreq(0)
{
}


//
DispatchThread::~DispatchThread(void)
{
    NEX_ASSERT(!isRunning());
}


void DispatchThread::StartDispatchThread(void (*func)(), s32 threadMode, s32 updateFreq)
{
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("DispatchThread::StartDispatchThread"));

    NEX_ASSERT(func!=NULL);
    NEX_ASSERT(updateFreq > 0);
    NEX_ASSERT(!isRunning());

    m_pFunc                  = func;
    m_threadMode             = threadMode;
    m_updateFreq             = updateFreq;
    m_bDispatchThreadRunning = true;

    s32 prio = THREAD_SYNC_PRIORITY_SLEEP;

    m_dispatchThread.StartUsingAutoStack(
        DispatchThreadFunc,
        this,
        AUTO_STACK_SIZE,
        prio
    );
}


void DispatchThread::FinishDispatchThread()
{
    UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("DispatchThread::FinishDispatchThread"));

    m_bDispatchThreadRunning = false;
    m_dispatchThread.Join();
    m_dispatchThread.Finalize();
}


void DispatchThread::ChangeDispatchPriority(s32 priority)
{
    m_dispatchThread.ChangePriority(priority);
}


void DispatchThread::DispatchThreadFunc(DispatchThread *pInstance)
{
    NN_NULL_ASSERT(pInstance);

    nn::nex::Pacer* pPacer = NULL;

#if 0  //n1769 Probably OK to delete, I think.
    g_gsStatus.m_uiProcessNexCount = 0;
#endif
    pInstance->m_DispatchThreadSleepPriority = THREAD_SYNC_PRIORITY_SLEEP;

    if(pInstance->getThreadMode() == THREAD_MODE_UNSAFE_USER_MULTI_THREAD)
    {
        nn::nex::ScopedCS slNexLock(NetZManager::GetInstance()->GetNexLock());
        pPacer = new nn::nex::Pacer();
        pPacer->SetFrequency(pInstance->getUpdateFreq());
    }
    else
    {
        pPacer = new nn::nex::Pacer();
        pPacer->SetFrequency(pInstance->getUpdateFreq());
    }

    while(pInstance->isRunning())
    {
        pInstance->m_pFunc();  // Execute the passed function here!

        NN_NULL_ASSERT(pPacer);
        if((pInstance->getThreadMode() == THREAD_MODE_UNSAFE_USER_MULTI_THREAD) && !pInstance->IsPreciseMeasuringMode())
        {
            nn::nex::ScopedCS slNexLock(NetZManager::GetInstance()->GetNexLock());
            pPacer->Pace();
        }
        else
        {
            pPacer->Pace();
        }
    }

    if(pInstance->getThreadMode() == THREAD_MODE_UNSAFE_USER_MULTI_THREAD)
    {
        nn::nex::ScopedCS slNexLock(NetZManager::GetInstance()->GetNexLock());
        NN_NULL_ASSERT(pPacer);
        delete pPacer;
    }
    else
    {
        NN_NULL_ASSERT(pPacer);
        delete pPacer;
    }
}

} // end of namespace CTR
