/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef CTR_DISPATCHTHREAD_H_
#define CTR_DISPATCHTHREAD_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <nn/os.h>


// Forward declaration.
class NetZManager;
class SceneBase;

namespace CTR
{
//
class DispatchThread
{
public:
    DispatchThread(void);
    ~DispatchThread(void);

    //n1769 Members related to the case when the application is multithreading.
    void StartDispatchThread(void (*func)(), s32 threadMode, s32 updateFreq);
    void FinishDispatchThread(void);
    void ChangeDispatchPriority(s32 priority);

    //n1769 The one to replace SAFE_USER_THREAD_DISPATCH_UPDATE_FREQENCY.
    void SetUpdateFrequency(s32 freq);

    //n1769 For {Secret} library developers.
    void EnablePreciseMeasuringMode (void)
    {
        m_bPrecise = true;
    }
    void DisablePreciseMeasuringMode(void)
    {
        m_bPrecise = false;
    }
    bool IsPreciseMeasuringMode     (void) const
    {
        return m_bPrecise;
    }

private:
    // The copy constructor and the assignment operator are private.
    DispatchThread           (const DispatchThread&);
    DispatchThread& operator=(const DispatchThread&);

    // Private constants group.
    static const int AUTO_STACK_SIZE = 4096*8;

    // Private function group.
    static void DispatchThreadFunc(DispatchThread *pInstance);
    s32 getThreadMode  (void) const
    {
        return m_threadMode;
    }
    bool isRunning     (void) const
    {
        return m_bDispatchThreadRunning;    // Is the dispatch thread running?
    }
    s32  getUpdateFreq (void) const
    {
        return m_updateFreq;
    }

    // Private variable group.
    s32             m_DispatchThreadSleepPriority;
    bool            m_bDispatchThreadRunning;
    bool            m_bPrecise;     //n1769 Supports ENABLE_PRECISE_MEASURE.
    nn::os::Thread  m_dispatchThread;
    s32             m_threadMode;
    s32             m_updateFreq;

    void(*m_pFunc) ();
};

} // end of namespace CTR

#endif  // end of CTR_DISPATCHTHREAD_H_
