﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef ERRORHANDLING_H_
#define ERRORHANDLING_H_

#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <nex_P2p.h>

#include <nn/types.h>

namespace nn
{
class Result;
}

namespace nn
{
namespace nex
{
class UserContext;
class qResult;
}
}

//
class ErrorHandling
{
public:
    // Initialization.
    static void Initialize(void);

    // End processing.
    static void Finalize(void);

    // Was an error detected on the NEX side?
    static bool IsErrorDetected(void);

    // Presumes that it is called after it becomes clear that an error that is not platform-dependent was detected on the NEX side.
    //  Gets information related to the error that occurred.
    static const nn::nex::qResult& GetCommonResultCode(void);

    // Called when NEX API returns false. Gets the error code for the cause of the false value.
    static void ProcessError(void);

    // Prints the non-platform dependent result code maintained by this class.
    static void Print(void);

    // Prints the non-platform dependent result code
    static void PrintCommon(const nn::nex::qResult &result);

    // Prints the non-platform dependent result code maintained by this class and the `Extra` value.
    static void PrintCommon(void);

    // Clears both the nn::Result type and qResult type values being maintained in this class.
    static void Clear(void);

    // {secret}
    // n1769
    // Forcibly sets the result code. Intentionally causes an error to check whether the error processing sequence works correctly.
    // 
    static void SetCommonResultCode(nn::nex::qResult result);

private:
    //
    ErrorHandling(void) {}
    ~ErrorHandling(void) {}
    ErrorHandling           (const ErrorHandling&);
    ErrorHandling& operator=(const ErrorHandling&);

    static bool s_bInitialized;
};


#endif  // end of ERRORHANDLING_H_
