/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#include "IdleThread.h"

namespace
{

}  // End of anonymous namespace


BEGIN_NAMESPACE_NEX

namespace CTR
{


IdleThread *IdleThread::s_pInstance = NULL;
int         IdleThread::s_counter   = 0;
bool        IdleThread::s_resetFlag = false;
int         IdleThread::s_value     = 1;
int         IdleThread::s_sleepTimes = 0;
int         IdleThread::s_spinTimes = 0;
bool        IdleThread::s_loopFlag  = false;


void IdleThread::CreateInstance(void)
{
    if(s_pInstance)
    {
        // Instance is already created.  Do nothing.
        return;
    }
    else
    {
        s_pInstance = new IdleThread();
    }
}


void IdleThread::DestroyInstance(void)
{
    if(s_pInstance)
    {
        delete s_pInstance;
        s_pInstance = NULL;
    }
    else
    {
        // Instance is not available.  Do nothing.
    }
}


IdleThread* IdleThread::GetInstance(void)
{
    NN_NULL_ASSERT(s_pInstance);

    return s_pInstance;
}


bool IdleThread::IsCreated(void)
{
    return s_pInstance!=NULL;
}


//
IdleThread::IdleThread(void)
{
}


//
IdleThread::~IdleThread(void)
{
}


void IdleThread::Start(void)
{
    const size_t STACK_SIZE = 4096;
    const s32    PRIO = nn::os::LOWEST_THREAD_PRIORITY - 1;    // Used to be that threads were looped in order of lowest priority.
    s_loopFlag  = true;  // Set flag to continue executing while loop.

#if 0
    m_thread.StartUsingAutoStack(IncProc,      STACK_SIZE, PRIO);
#elif 0
    m_thread.StartUsingAutoStack(Procedure,    STACK_SIZE, PRIO);
#elif 0
    m_thread.StartUsingAutoStack(SleepAndWake, STACK_SIZE, PRIO);
#elif 1
    m_thread.StartUsingAutoStack(SpinWait,     STACK_SIZE, PRIO);
#else
    m_thread.StartUsingAutoStack(SpinWaitEx,   STACK_SIZE, PRIO);
#endif
}


void IdleThread::Join(void)
{
    s_loopFlag = false;  // Clear flag to continue executing while loop.
    m_thread.Join();
}


void IdleThread::IncProc(void)
{
    while(s_loopFlag)
    {
        while(!s_resetFlag)
        {
            ++s_counter;
        }
        s_counter = 0;
        s_resetFlag = false;
    }
}


void IdleThread::Procedure(void)
{
    while(s_loopFlag)
    {
        Calc();
    }
}


// Just wanted to give the CPU some complex calculations to do. The calculations themselves are not important.
void IdleThread::Calc(void)
{
    ++s_value;
    s_value *= 7;
    s_value /= 3;
    --s_value;
    s_value *= s_value;
    if(s_value > 1)
    {
        s_value /= (s_value - 1);
    }
}


void IdleThread::SleepAndWake(void)
{
    // 1ms
    const nn::fnd::TimeSpan c_1ms = nn::fnd::TimeSpan::FromMilliSeconds(1);
#if 0
    const nn::fnd::TimeSpan c_4ms = nn::fnd::TimeSpan::FromMilliSeconds(4);
    const nn::fnd::TimeSpan c_10ms = nn::fnd::TimeSpan::FromMilliSeconds(10);
#endif
    while(s_loopFlag)
    {
        NN_LOG("Caution!  SleepAndWake() is out-of-date!\n");

        nn::os::Thread::Sleep(c_1ms);
        ++s_sleepTimes;
    }
}


void IdleThread::SpinWait(void)
{
    const nn::fnd::TimeSpan c_1ms_timespan = nn::fnd::TimeSpan::FromMilliSeconds(1);
    const nn::os::Tick c_1ms_tick(c_1ms_timespan);
    const s32 c_cycles = static_cast<s32>((s64)c_1ms_tick);
    while(s_loopFlag)
    {
        nn::os::ARM::SpinWaitCpuCycles(c_cycles);
        ++s_spinTimes;
    }
}


// Implementation of a 900 us spin wait and sleep for the remaining 100 us.
// If the CPU doesn't sleep, the system will join a session and then end up losing communication during the session!
void IdleThread::SpinWaitEx(void)
{
    const nn::fnd::TimeSpan c_100us_timespan = nn::fnd::TimeSpan::FromMicroSeconds(100);
    const nn::fnd::TimeSpan c_900us_timespan = nn::fnd::TimeSpan::FromMicroSeconds(900);
    const nn::os::Tick c_900us_tick(c_900us_timespan);
    const s32 c_cycles = static_cast<s32>((s64)c_900us_tick);

    while(s_loopFlag)
    {
        nn::os::ARM::SpinWaitCpuCycles(c_cycles);
        nn::os::Thread::Sleep(c_100us_timespan);
        ++s_spinTimes;
    }
}


}  // end of namespace CTR

END_NAMESPACE_NEX
