/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef IDLETHREAD_H_
#define IDLETHREAD_H_

#include <nn/os.h>

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

namespace CTR
{

/* Please see man pages for details 
 

        
        
 */
class IdleThread
{
public:
    // Singleton pattern.
    static void        CreateInstance (void);
    static void        DestroyInstance(void);
    static IdleThread* GetInstance    (void);
    static bool        IsCreated      (void);

    ~IdleThread(void);

    // Low priority thread, start execution.
    void Start(void);

    // Join IdleThread begun with Start.
    // In the CTR-SDK, Join must be called prior to cleanup processing for threads begun with Start.
    void Join(void);

    // Returns the current count status.
    int GetCounter(void) const
    {
        return s_counter;
    }

    // Reset counter.
    void ResetCounter(void)
    {
        s_resetFlag = true;
    }

    // Gets this thread's ID.
    bit32 GetId(void) const
    {
        return m_thread.GetId();
    }

    // Number of times a 1 ms sleep was executed.
    int GetSleepTimes(void)
    {
        return s_sleepTimes;
    }

    // Resets the sleep count
    void ResetSleepTimes(void)
    {
        s_sleepTimes = 0;
    }

    // Number of times spin was executed.
    int GetSpinTimes(void) const
    {
        return s_spinTimes;
    }

    // Resets the spin count
    void ResetSpinTimes(void)
    {
        s_spinTimes = 0;
    }

private:
    //
    IdleThread           (void);
    IdleThread           (const IdleThread&);
    IdleThread& operator=(const IdleThread&);

    // Function to increment the counter
    static void IncProc(void);

    // Function to call calculations that take time
    static void Procedure(void);

    // Function to execute calculations that take time.
    static void Calc(void);

    // Function to repeat short sleep and wakeup.
    static void SleepAndWake(void);

    // Function to run loops with spin wait.
    static void SpinWait(void);

    // Implementation of a 900 us spin wait and sleep for the remaining 100 us.
    static void SpinWaitEx(void);

    // Instance for singleton
    static IdleThread *s_pInstance;

    // Thread class instance.
    nn::os::Thread m_thread;

    // Counter incremented while a thread is running.
    static int s_counter;

    // Flag to retain reset instruction.
    static bool s_resetFlag;

    // Variable to do calculations.
    static int s_value;

    // How many 1 ms Sleep occurrences?
    static int s_sleepTimes;

    // How many 1 ms Spin occurrences?
    static int s_spinTimes;

    // Loop flag. Continue the while loop for as long as this flag is set.
    static bool s_loopFlag;
};


}  // end of namespace CTR

END_NAMESPACE_NEX

#endif  // end of IDLETHREAD_H_
