/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef MULTIUNITHEAPMANAGER_H_
#define MULTIUNITHEAPMANAGER_H_

#include "./MultiUnitHeapTemplate.h"
#include "./MultiUnitHeapSetting.h"


//
//
//
//
//
//


template<class A> class MultiUnitHeapManager
{
public:
    static bool IsInitialized (void);
    static bool Initialize    (const struct MultiUnitHeapSetting &setting);
    static void Finalize      (void);
    static void PrintDebugInfo(bool);
    static void ResetPeakCounts(void);
    static void Age           (void);
    static size_t GetUsage    (void);

private:
    //
    MultiUnitHeapManager(void);
    ~MultiUnitHeapManager(void);
    MultiUnitHeapManager           (const MultiUnitHeapManager&);
    MultiUnitHeapManager& operator=(const MultiUnitHeapManager&);

    static u32 s_ageCounter;  // Number of times Age function is called.
};


template<class A> u32 MultiUnitHeapManager<A>::s_ageCounter = 0;


// The following is code when MultiUnitHeap is used.
template<class A>
bool MultiUnitHeapManager<A>::IsInitialized(void)
{
    return A::IsInitialized();
}


template<class A>
bool MultiUnitHeapManager<A>::Initialize(const struct MultiUnitHeapSetting &setting)
{
#if NEX_ENABLE_MEMORY_STARVATION_CARE
    if(A::Initialize(setting.memBlock, setting.memSize, setting.unitSize, setting.unitObjs, MultiUnitHeapSetting::LEVEL, setting.reserved))
#else
    if(A::Initialize(setting.memBlock, setting.memSize, setting.unitSize, setting.unitObjs, MultiUnitHeapSetting::LEVEL))
#endif
    {
        if(setting.recordPeak)
        {
            A::EnablePeakRecording();   // Enable recording peak values
        }
        else
        {
            A::DisablePeakRecording();  // Disable recording peak values
        }
        return true;
    }
    else
    {
        NN_ASSERT("MultiUnitHeap::Initialize() failed!");
        return false;
    }
}


template<class A>
void MultiUnitHeapManager<A>::Finalize(void)
{
    if(IsInitialized())
    {
        A::Finalize();
    }
    else
    {
        // Do nothing.
    }
}


template<class A>
void MultiUnitHeapManager<A>::PrintDebugInfo(bool)
{
    if(IsInitialized())
    {
        A::PrintUsage();
        NN_LOG("Age: %u\n", s_ageCounter);
    }
    else
    {
        NN_LOG("MultiUnitHeap is not initialized.\n");
    }
}

template<class A>
void MultiUnitHeapManager<A>::ResetPeakCounts(void)
{
    if(IsInitialized())
    {
        A::ResetPeakCounts();
    }
    else
    {
        NN_LOG("MultiUnitHeap is not initialized.\n");
    }
}


template<class A>
void MultiUnitHeapManager<A>::Age(void)
{
    ++s_ageCounter;
}


template<class A>
size_t MultiUnitHeapManager<A>::GetUsage(void)
{
    return A::GetTotalMemoryUsage();
}


#endif  // end of MULTIUNITHEAPMANAGER_H_
