﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NETZMANAGER_H_
#define NETZMANAGER_H_

#include <NetZ/NetZ.h>
#include <NetZ/src/Core/DOCore/Session.h>

#include "DispatchThread.h"
struct SampleConfig;


class NetZManager
{
public:
    //n1589: Singleton
    static void CreateInstance(void);
    static void DestroyInstance(void);
    static NetZManager* GetInstance(void);
    static bool IsCreated(void);

    ~NetZManager(void);

    // Does a NetZ instance exist?
    bool IsCreatedNetZ(void) const;

    // Does a NetZ instance exist and is it enabled?
    bool IsValid(void) const;

    // Create NetZ instance. true if successfully created. false if creation failed or if the instance is not created.
    // Even if this function is called repeatedly several times, make sure that it will not create multiple NetZ instances.
    // Config (various settings) is given with the arguments.
    bool CreateNetZ(const SampleConfig &cfg);

    // Destroys NetZ instance created with CreateNetZ().
    // If `DestroyNetZ` is called when no instances exist, the function exits without doing anything in particular.
    // 
    void DestroyNetZ(void);

    // Calls NetZ::Terminate function.
    bool TerminateNetZ();
    bool TerminateNetZ(nn::nex::CallContext* pContext);

    bool DispatchNetZJob(qUnsignedInt32 timeout);

    u16 GetNbLocalStations() const;
    qByte GetRole() const;

    void RegisterStationEventCallback(void (*callback)(nn::nex::StationOperation* pOperation));

    //n2788: Delete station status detection callback registration
    void UnregisterStationEventCallback(void);

    nn::nex::CriticalSection& GetNexLock    (void)
    {
        return m_csNexLock;
    }
    nn::nex::CriticalSection& GetSendLock   (void)
    {
        return m_csSendData;
    }
    nn::nex::CriticalSection& GetReceiveLock(void)
    {
        return m_csReceiveData;
    }

    //n1769 The actual processing is left to DispatchThread.
    void StartDispatchThread(void(*func)(), s32 updateFreq);
    void FinishDispatchThread(void);
    void ChangeDispatchPriority(s32 priority);
    void SetPreciseMesuringMode(bool bMode);

private:
    NetZManager(void);

    // Is the NetZ object in a usable state?
    bool isAvaiable(void) const;

    static NetZManager* s_pInstance;
#if 0
    NetZManager           (const NetZManager&);
    NetZManager& operator=(const NetZManager&);
#endif

    nn::nex::NetZ *m_pNetZ;
    DispatchThread *m_pDisp;

    //n1589: Lock must be done in this order to prevent a deadlock
    nn::nex::CriticalSection m_csNexLock;
    nn::nex::CriticalSection m_csSendData;
    nn::nex::CriticalSection m_csReceiveData;

    int m_threadMode;
};

#endif  // end of NETZMANAGER_H_
