/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#include "NexUtil.h"
#include <cstring>

//-----------------------------------------------------------------------------------------
// Name: ConvertHtoLE16
//
// Description:
//      Conversion to NEX network byte order (little endian, 16-bit)
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
nn::nex::qUnsignedInt16 ConvertHtoLE16( nn::nex::qUnsignedInt16 data )
{
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
    return static_cast<nn::nex::qUnsignedInt16>(((data >> 8) | (data << 8)));
#else
    return data;
#endif
}

//-----------------------------------------------------------------------------------------
// Name: ConvertHtoLE32
//
// Description:
//      Conversion to NEX network byte order (little endian, 32-bit)
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
nn::nex::qUnsignedInt32 ConvertHtoLE32( nn::nex::qUnsignedInt32 data )
{
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
    nn::nex::qUnsignedInt32 tmp;
    tmp = ((data >> 8) & 0x00ff00ff) | ((data << 8) & 0xff00ff00);
    return (tmp >> 16) | (tmp << 16);
#else
    return data;
#endif
}

//-----------------------------------------------------------------------------------------
// Name: ConvertLEtoH16
//
// Description:
//      Conversion to host byte order (16 bit)
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
nn::nex::qUnsignedInt16 ConvertLEtoH16( nn::nex::qUnsignedInt16 data )
{
    return ConvertHtoLE16( data );
}

//-----------------------------------------------------------------------------------------
// Name: ConvertLEtoH32
//
// Description:
//      Conversion to host byte order (32 bit)
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
nn::nex::qUnsignedInt32 ConvertLEtoH32( nn::nex::qUnsignedInt32 data )
{
    return ConvertHtoLE32( data );
}

//-----------------------------------------------------------------------------------------
// Name: MemcpyHtoLE16
//
// Description:
//      Convert to NEX network byte order (little endian, 16-bit) and copy the memory.
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
void* MemcpyHtoLE16(void* buf1, const void* buf2)
{
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
    nn::nex::qUnsignedInt16 data = ConvertHtoLE16(*(nn::nex::qUnsignedInt16*)buf2);
    return std::memcpy(buf1, &data, 2);
#else
    return std::memcpy(buf1, buf2, 2);
#endif
}

//-----------------------------------------------------------------------------------------
// Name: MemcpyHtoLE32
//
// Description:
//      Convert to NEX network byte order (little endian, 32-bit) and copy the memory.
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
void* MemcpyHtoLE32(void* buf1, const void* buf2)
{
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
    nn::nex::qUnsignedInt32 data = ConvertHtoLE32(*(nn::nex::qUnsignedInt32*)buf2);
    return std::memcpy(buf1, &data, 4);
#else
    return std::memcpy(buf1, buf2, 4);
#endif
}

//-----------------------------------------------------------------------------------------
// Name: MemcpyLEtoH16
//
// Description:
//      Convert to host byte order (16-bit) and copy the memory.
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
void* MemcpyLEtoH16(void* buf1, const void* buf2)
{
    return MemcpyHtoLE16(buf1, buf2);
}

//-----------------------------------------------------------------------------------------
// Name: MemcpyLEtoH32
//
// Description:
//      Convert to host byte order (32-bit) and copy the memory.
//
// Arguments:
//      data: Data to convert
//
// Returns:
//      Data after conversion
//
//-----------------------------------------------------------------------------------------
void* MemcpyLEtoH32(void* buf1, const void* buf2)
{
    return MemcpyHtoLE32(buf1, buf2);
}


