/*---------------------------------------------------------------------------*
  Project:  NEX
  File:     NexUtil.h

  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#ifndef _NEXUTIL_H_
#define _NEXUTIL_H_

#include <OnlineCore/src/Platform/Core/Recognition.h>


#if defined(CTR_TARGET)
#include <nn.h>
#include <nn/dbg.h>
#elif defined(CAFE_TARGET)
#include <cafe/os.h>
#else /* !CTR_TARGET && !CAFE_TARGET */
#include <cstdio>
#include <cassert>
#include <cstdlib>
#endif /* !CTR_TARGET && !CAFE_TARGET */

#if defined(CTR_TARGET)
#define NEX_DEMO_LOG( ... ) NN_LOG(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_LOG( ... ) OSReport(__VA_ARGS__)
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_LOG( ... ) std::printf(__VA_ARGS__)
#endif /* !CTR_TARGET && !CAFE_TARGET */

#if defined(CTR_TARGET)
#define NEX_DEMO_ASSERT(expr) NN_ASSERT(expr)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_ASSERT(expr) ASSERT(expr)
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_ASSERT(expr) assert(expr)
#endif /* !CTR_TARGET && !CAFE_TARGET */

#if defined(CTR_TARGET)
#define NEX_DEMO_PANIC( ... ) NN_PANIC(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); OSHalt("Panic!")
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); std::exit(-1)
#endif /* !CTR_TARGET && !CAFE_TARGET */


#if defined(CTR_TARGET)
#ifdef NO_NETWORKING
#define MAIN_WRAPPER(USERMAIN)  \
extern "C" void nnMain() { \
  static qChar8* argv[] = {"exename"};  \
  USERMAIN((sizeof(argv)/sizeof(char*)), argv); \
  NN_LOG("Leaving application...\n"); \
  nn::dbg::Break(nn::dbg::BREAK_REASON_USER); \
}
#else /* !NO_NETWORKING */
#include "../common/CTRNetInit.h"
#define MAIN_WRAPPER(USERMAIN)  \
extern "C" void nnMain() { \
  NAMESPACE_NEX CTRNetInit(); \
  static qChar8* argv[] = {"exename"};  \
  USERMAIN((sizeof(argv)/sizeof(*argv)), argv); \
  NN_LOG("Leaving application...\n"); \
  nn::dbg::Break(nn::dbg::BREAK_REASON_USER); \
  NAMESPACE_NEX CTRNetTerm(); \
}
#endif /* !NO_NETWORKING */
#elif defined(CAFE_TARGET)
#include "../common/Cafe/DemoNetInit.h"
#define MAIN_WRAPPER(USERMAIN)  \
nn::nex::qInt __CDECL main(nn::nex::qInt argc, nn::nex::qChar8** argv) { \
    Cafe::DemoNetInit(); \
    USERMAIN(argc, argv); \
    Cafe::DemoNetTerm(); \
}
#else /* !CTR_TARGET && !CAFE_TARGET */
#ifdef NON_SUPPORT_D_FAMILY
#define MAIN_WRAPPER(USERMAIN)  \
int __CDECL main(qInt argc, qChar8** argv) \
{ \
    return USERMAIN(argc, argv); \
}
#else /* !NON_SUPPORT_D_FAMILY */
#define MAIN_WRAPPER(USERMAIN)  \
int __CDECL _tmain(qInt argc, qChar8** argv) \
{ \
    return USERMAIN(argc, argv); \
}
#endif /* !NON_SUPPORT_D_FAMILY */
#endif /* !CTR_TARGET && !CAFE_TARGET */

// Endian conversion related
nn::nex::qUnsignedInt16 ConvertHtoLE16( nn::nex::qUnsignedInt16 data );
nn::nex::qUnsignedInt32 ConvertHtoLE32( nn::nex::qUnsignedInt32 data );
nn::nex::qUnsignedInt16 ConvertLEtoH16( nn::nex::qUnsignedInt16 data );
nn::nex::qUnsignedInt32 ConvertLEtoH32( nn::nex::qUnsignedInt32 data );
void* MemcpyHtoLE16(void* buf1, const void* buf2);
void* MemcpyHtoLE32(void* buf1, const void* buf2);
void* MemcpyLEtoH16(void* buf1, const void* buf2);
void* MemcpyLEtoH32(void* buf1, const void* buf2);

#endif  // end of _NEXUTIL_H_
