/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef DEMO_GRAPHICS_DRAWING_H_
#define DEMO_GRAPHICS_DRAWING_H_

#include <nn/gx.h>
#include <nn/util/util_NonCopyable.h>

#include "demo/RenderData/demo_TextsRenderData.h"
#include "demo/RenderData/demo_TrianglesRenderData.h"

#include "demo/ShaderProgram/demo_ShaderManager.h"

namespace demo
{
/* Please see man pages for details 
    
*/

class GraphicsDrawing : private nn::util::NonCopyable<GraphicsDrawing>
{
public:
    GraphicsDrawing(void);
    virtual ~GraphicsDrawing(void);

public:
    virtual void Initialize(void);
    virtual void Finalize(void);

public:
    /* Please see man pages for details 
    

              
              
              

    
    */
    virtual void InitializeTriangles(const u32 maxTrianglesNum = demo::MAX_TRIANGLES_NUM);

    /* Please see man pages for details 
    

              
              
              

    
    */
    virtual void InitializeSquares(const u32 maxSquaresNum = demo::MAX_SQUARES_NUM);

    /* Please see man pages for details 
    

              
              
              

    
    */
    virtual void InitializeTexts(const u32 maxLength = demo::MAX_TEXT_LENGTH);

public:
    /* Please see man pages for details 
    

    
    
    */
    virtual void SetWindowSize(const f32 windowWidth, const f32 windowHeight);

    /* Please see man pages for details 
    

    
    
    */
    virtual void GetWindowSize(f32& windowWidth, f32& windowHeight);

    /* Please see man pages for details 
    

    
    
    
    
    */
    virtual void SetColor(const f32 red, const f32 green, const f32 blue, const f32 alpha);

    /* Please see man pages for details 
    

    
    
    
    */
    virtual void SetColor(const f32 red, const f32 green, const f32 blue);

    /* Please see man pages for details 
    

    
    */
    virtual void SetDepth(const f32 depth);

    /* Please see man pages for details 
    

    
    */
    virtual void SetAutoDepth(const bool enableAutoDepth);

    /* Please see man pages for details 
    
    */
    virtual void DecrementDepth(void);

public:
    /* Please see man pages for details 
    

    
    */
    virtual void SetPointSize(const f32 pointSize);

    /* Please see man pages for details 
    

    
    
    */
    virtual void DrawPoint(const f32 windowCoordinateX, const f32 windowCoordinateY);

public:
    /* Please see man pages for details 
    

    
    */
    virtual void SetLineWidth(const f32 lineWidth);

    /* Please see man pages for details 
    

    
    
    
    
    */
    virtual void DrawLine(const f32 windowCoordinateX0, const f32 windowCoordinateY0,
                          const f32 windowCoordinateX1, const f32 windowCoordinateY1);

public:
    /* Please see man pages for details 
    

    

    
    
    
    
    
    
    
    
    */
    virtual void FillSquare(const f32 windowCoordinateX0, const f32 windowCoordinateY0,
                            const f32 windowCoordinateX1, const f32 windowCoordinateY1,
                            const f32 windowCoordinateX2, const f32 windowCoordinateY2,
                            const f32 windowCoordinateX3, const f32 windowCoordinateY3);

public:
    /* Please see man pages for details 
    

    
    
    
    
    */
    virtual void FillRectangle(const f32 windowCoordinateX,
                               const f32 windowCoordinateY,
                               const f32 width, const f32 height);
public:
    /* Please see man pages for details 
    

    

    
    
    
    
    
    
    */
    virtual void FillTriangle(const f32 windowCoordinateX0, const f32 windowCoordinateY0,
                              const f32 windowCoordinateX1, const f32 windowCoordinateY1,
                              const f32 windowCoordinateX2, const f32 windowCoordinateY2);

public:
    /* Please see man pages for details 
    

    

    
    */
    virtual void SetFontSize(const f32 fontSize);

    /* Please see man pages for details 
    

    
    
    

    
    
    
    */
    virtual void DrawText(const f32 windowCoordinateX, const f32 windowCoordinateY, const char* format, ...);

    /* Please see man pages for details 
    

    
    
    

    
    
    
    */
    virtual void DrawTextBuffer(const f32 windowCoordinateX, const f32 windowCoordinateY, const char* textBuffer);

public:
    /* Please see man pages for details 
    
    */
    virtual void Flush(void);

protected:
    virtual void FlushColorFillPrimitives(void);
    virtual void FlushTexts(void);

public:
    f32 m_Color[4];
    f32 m_Depth;
    bool m_AutoDepth;
    bool m_Padding0[3];
    f32 m_DepthValue;

    f32 m_WindowWidth;
    f32 m_WindowHeight;

public:
    demo::ShaderManager m_ShaderManager;

protected:
    bool m_InitializeTriangles;
    bool m_InitializeSquares;
    bool m_InitializeTexts;
    bool m_Padding1;

protected:
    u32 m_MaxTrianglesNum;
    u32 m_TotalTrianglesNum;
protected:
    u32 m_MaxSquaresNum;
    u32 m_TotalSquaresNum;
protected:
    u32 m_MaxTextLength;
    u32 m_TotalTextLength;

protected:
    bool m_UseColorFillProgram;
    bool m_UseFontProgram;
    bool m_Padding2[2];

protected:
    f32 m_PointSize;
    f32 m_LineWidth;

    demo::TrianglesRenderData m_TrianglesRenderData;
    demo::TrianglesRenderData m_SquaresRenderData;

    f32 m_FontSize;
    demo::TextsRenderData m_TextsRenderData;
};

}

#endif
