﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#ifndef _NEXUTILDEFS_H_
#define _NEXUTILDEFS_H_

#include <nex.h>

//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

// Name space definition for the sample
#define NEXUTIL_BEGIN namespace nex_demo_util {
#define NEXUTIL_END }


// Game server ID and access key used to log in to NGS, as well as key hash values used on the independent server.
#if defined(CTR_TARGET) // CTR sample demo server.
#define NEXDEMO_GAME_SERVER_ID 0x0FE00304
#define NEXDEMO_ACCESS_KEY _T("12345678")
#define NEXDEMO_KEY_HASH "deed2a88"
#else   // Cafe sample demo server.
#define NEXDEMO_GAME_SERVER_ID 0x1F940002
#define NEXDEMO_ACCESS_KEY _T("12345678")
#define NEXDEMO_KEY_HASH   "deed2a88"
#endif

// String format identifier. "%ls" is defined for UNICODE_VERSION for both CTR and Cafe.
#ifdef UNICODE_VERSION
#define STRING_FORMAT "%ls"
#else
#define STRING_FORMAT "%s"
#endif

// Include log in standard output.
#if defined(CTR_TARGET)
#define NEX_DEMO_LOG( ... ) NN_LOG(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_LOG( ... ) printf(__VA_ARGS__)
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_LOG( ... ) std::printf(__VA_ARGS__)
#endif /* !CTR_TARGET && !CAFE_TARGET */

// In the debugging build, when these conditions aren't met, stop the process.
// In the release build, don't evaluate these conditions.
#if defined(CTR_TARGET)
#define NEX_DEMO_ASSERT(expr) NN_ASSERT(expr)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_ASSERT(expr) ASSERT(expr)
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_ASSERT(expr) SYSTEMCHECK(expr)
#endif /* !CTR_TARGET && !CAFE_TARGET */

// Stop process.
#if defined(CTR_TARGET)
#define NEX_DEMO_PANIC( ... ) NN_PANIC(__VA_ARGS__)
#elif defined(CAFE_TARGET)
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); OSHalt("Panic!")
#else /* !CTR_TARGET && !CAFE_TARGET */
#define NEX_DEMO_PANIC( ... ) NEX_DEMO_LOG(__VA_ARGS__); std::exit(-1)
#endif /* !CTR_TARGET && !CAFE_TARGET */

// In the debugging build, when these conditions aren't met, stop the process.
// Also evaluate these conditions in the release build. However, the process isn't stopped even if the conditions aren't met.
#ifdef RELEASE_VERSION
#define NEX_DEMO_VERIFY(expr) ((void)(expr))
#else
#define NEX_DEMO_VERIFY(expr) NEX_DEMO_ASSERT(expr)
#endif


//-----------------------------------------------------------------------------
// Main function
//-----------------------------------------------------------------------------
#if defined(CAFE_TARGET)
#define NEX_MAIN_WRAPPER(USERMAIN)\
    qInt __CDECL _tmain(qInt argc, qChar** argv) { \
        return USERMAIN(argc, argv);                      \
    }
#elif defined(CTR_TARGET)
#define NEX_MAIN_WRAPPER(USERMAIN)\
    extern "C" void nnMain() {                     \
        USERMAIN(0, NULL);                      \
    }
#else
#define NEX_MAIN_WRAPPER(USERMAIN)\
    qInt __CDECL _tmain(qInt argc, qChar** argv) { \
        return USERMAIN(argc, argv);                      \
    }
#endif


#endif  // end of _NEXUTILDEFS_H_
