﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _CONTAINEROPERATION_H_
#define _CONTAINEROPERATION_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Transport/Interface/Message.h>

BEGIN_NAMESPACE_NEX

#ifdef TRACE_ENABLED

const qChar* GetContainerOperationName(qUnsignedInt uiCode);

#endif

//==============================================================================
// This is an internal template class solely used in the context of member
// containers implementation. Each instance represents a container operation.
//==============================================================================
template<class TDDL_Type>
class ContainerOperation: public RootObject
{
public:
    // Important Note: This enumeration must appear first before the
    // constructors where it used otherwise the compiler misinterprets.
    enum eOpCode
    {
        // Queue operations
        NOP_OPCODE=0,
        PUSH_OPCODE,
        POP_OPCODE,
        EMPTY_OPCODE,
        BACK_OPCODE,
        FRONT_OPCODE,
        SIZE_OPCODE,

        // List operations
        PUSH_FRONT_OPCODE,
        PUSH_BACK_OPCODE,
        POP_FRONT_OPCODE,
        POP_BACK_OPCODE,
        INSERT_PX_OPCODE,
        INSERT_PNX_OPCODE,
        INSERT_PFL_OPCODE,
        ERASE_OPCODE,
        CLEAR_OPCODE,
        ASSIGN_OPCODE,

        // Vector-specific operations
        BRACKET_OPCODE
    };

public:
    // {secret}
    typedef typename TDDL_Type::CPPType Type;
    typedef typename ContainerOperation<TDDL_Type>::eOpCode eOpCode_t;

    // For "elementless" operations
    ContainerOperation()
    {
        m_eOpCode = NOP_OPCODE;
        m_uiPos = 0;
        m_uiRepeat = 0;
    }

    ContainerOperation(eOpCode_t eCode)
    {
        m_eOpCode = eCode;
        m_uiPos = 0;
        m_uiRepeat = 1;
        //m_tElem = (Type)0; // User types can't necessarily be constructed from a qInt
    }

    ContainerOperation(eOpCode_t eCode, const Type& tElem)
    {
        m_eOpCode = eCode;
        m_uiPos = 0;
        m_uiRepeat = 1;
        m_tElem = tElem;
    }

    ContainerOperation(eOpCode_t eCode, qUnsignedInt32 uiPos, const Type& tElem)
    {
        m_eOpCode = eCode;
        m_uiPos = uiPos;
        m_uiRepeat = 1;
        m_tElem = tElem;
    }

    ContainerOperation(eOpCode_t eCode, qUnsignedInt32 uiPos, qUnsignedInt32 uiRepeat, const Type& tElem)
    {
        m_eOpCode = eCode;
        m_uiPos = uiPos;
        m_uiRepeat = uiRepeat;
        m_tElem = tElem;
    }

    ContainerOperation(const ContainerOperation& coOp)
    {
        m_eOpCode = coOp.m_eOpCode;
        m_uiPos = coOp.m_uiPos;
        m_tElem = coOp.m_tElem;
        m_uiRepeat = coOp.m_uiRepeat;
    }

    virtual ~ContainerOperation()
    {
    }

    qUnsignedInt32 GetRepeat() const
    {
        return m_uiRepeat;
    }
    qUnsignedInt32 GetPos() const
    {
        return m_uiPos;
    }
    eOpCode GetOpCode() const
    {
        return m_eOpCode;
    }

    void AddSourceTo(Message *pMsg)
    {
        const qByte bOpCode = static_cast<qByte>(m_eOpCode);
        *pMsg << bOpCode;
        *pMsg << m_uiPos;
        TDDL_Type::Add(pMsg, m_tElem);
        *pMsg << m_uiRepeat;
    }

    void ExtractFrom(Message *pMsg)
    {
        qByte bOpCode;
        *pMsg >> bOpCode;
        m_eOpCode = static_cast<eOpCode>(bOpCode);

        *pMsg >> m_uiPos;
        TDDL_Type::Extract(pMsg, &m_tElem);
        *pMsg >> m_uiRepeat;
    }

#ifdef TRACE_ENABLED
    void Trace(qChar* szFormat, qUnsignedInt64 uiTraceFlag) const
    {
        DUMMY_PARAM(szFormat);
        QTRACE(uiTraceFlag, GetContainerOperationName(qUnsignedInt(m_eOpCode)) <<
               _T(" : Pos:") << dec << ( m_uiPos) << _T(", Repeat:") << dec << ( m_uiRepeat) << _T(", Data: ") << ( m_tElem));
    }
#endif

    Type& GetData()
    {
        return m_tElem;
    }
    ContainerOperation<TDDL_Type>* GetPtr()
    {
        return this;
    }

private:
    eOpCode m_eOpCode;
    qUnsignedInt m_uiPos;
    Type m_tElem;
    qUnsignedInt m_uiRepeat;
};

END_NAMESPACE_NEX

#endif//_CONTAINEROPERATION_H_
