﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOCore_H_
#define _DOCore_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

BEGIN_NAMESPACE_NEX

// The instances it manages.
class StationConnectionManager;
class DOSelections;
class ObjDupProtocol;
class StationManager;
class SessionDiscoveryTable;
class OperationManager;
class SystemComponent;
class ErrorDescriptionTable;
class BundlingPolicy;
class StationIdentification;
class SafetyExecutive;
class DOSubset;
class ConnectionOrientedStream;
class FirstErrorHolder;

//n2788:
class DirectTransport;
class DirectStream;


#define _MANAGEINSTANCE(TYPE,NAME)      \
    public:                             \
        const TYPE* Get##NAME() const {return m_p##NAME;}       \
        TYPE* Get##NAME() {return m_p##NAME;}               \
    private:                                \
        TYPE *m_p##NAME

#define _MANAGEGLOBALINSTANCE(TYPE,NAME)    \
    public:                                 \
        static TYPE* Get##NAME() {return s_p##NAME;}                \
    private:                                \
        static TYPE *s_p##NAME


#define MANAGEINSTANCE(C)   _MANAGEINSTANCE(C,C)
#define MANAGEGLOBALINSTANCE(C) _MANAGEGLOBALINSTANCE(C,C)


/* Please see man pages for details 
 
 
 
 
 
 */
class DOCore: public PseudoSingleton
{
public:
    /*
    Description: Returns a pointer to the DOCore instance.
    Returns: A pointer to the DOCore object.
    */

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    static DOCore* GetInstance()
    {
        return (DOCore*) InstanceControl::FindInstance(IC_TYPE_DOCORE, PseudoSingleton::GetCurrentContext());
    }

    DOCore(FirstErrorHolder *pFirstErrorHolder = NULL);
    virtual ~DOCore();

#ifdef TRACE_ENABLED    // n2370
    void TraceSystemState(qUnsignedInt64 uiTraceFlag);
#endif  // TRACE_ENABLED

    //n1589:memo: Mark the method name to search
    //n1589:mark:GetSafetyExecutive
    MANAGEINSTANCE(SafetyExecutive);
    //n2370:memo: This is the connection pool. DOCore initiates this instance, but it is not used from this class.
    //n1589:mark:GetStationConnectionManager
    MANAGEINSTANCE(StationConnectionManager);
    //n1589:mark:GetStationManager
    MANAGEINSTANCE(StationManager);
    //n1589:mark:GetDOSelections
    MANAGEINSTANCE(DOSelections);
    //n1589:mark:GetObjDupProtocol
    MANAGEINSTANCE(ObjDupProtocol);
    //n1589:mark:GetSessionDiscoveryTable
    MANAGEINSTANCE(SessionDiscoveryTable);
    //n1589:mark:GetBundlingPolicy
    MANAGEINSTANCE(BundlingPolicy);
    //n1589:mark:GetStationIdentification
    MANAGEINSTANCE(StationIdentification);
    //n1589:mark:GetStream()
    _MANAGEINSTANCE(ConnectionOrientedStream,Stream);
    //n1589:mark:GetDOOperationManager
    _MANAGEINSTANCE(OperationManager,DOOperationManager);
    //n1589:mark:GetJoinProcessing
    _MANAGEINSTANCE(SystemComponent,JoinProcessing);
    //n1589:mark:GetDOCoreState
    _MANAGEINSTANCE(SystemComponent,DOCoreState);
    //n1589:mark:GetUserDOs
    _MANAGEINSTANCE(DOSubset,UserDOs);

    // The 'fault processing' system state is in place and set while
    // we process faults, but never initialized and tested for, in
    // the core, yet.
    //n1589:mark:GetFaultProcessing
    _MANAGEINSTANCE(SystemComponent,FaultProcessing);

    //n2788:GetDirectTransport
    MANAGEINSTANCE(DirectTransport);
    MANAGEINSTANCE(DirectStream);

public:
    // Job queueing. Should be moved elsewhere.
    void QueuePeriodicJobs();
    void CancelPeriodicJobs();

    qBool IsReadyToLeave();

    void SetToReadyState(); // After a successful join/create session.
    void SetToTerminatingState(); // When JobClearTheStore begins.
    void SetToTerminatedState(); // When JobClearTheStore ends.
    void SetToCorruptedState();

    qBool IsInNormalOperation() const;
    qBool IsTerminating() const;
    qBool IsTerminated() const;
    qBool IsCorrupted() const;

    // Always stays true, once termination has been initiated.
    qBool HasStartedTermination() const;

    // Fatal error
    void SetFatalError(qResult err);
    FirstErrorHolder *GetFirstFatalErrorHolder() const
    {
        return m_pFirstErrorHolder;
    }
    qBool IsFaultRecoveryJobProcessFailed()
    {
        return m_isFaultRecoveryJobProcessFailed;
    }
    void SetIsFaultRecoveryJobProcessFailed()
    {
        m_isFaultRecoveryJobProcessFailed=true;
    }

    /* Please see man pages for details 
    

    
    

    
    */
    qUnsignedInt32 GetMaxUserDOReliableSendBufferNum() const
    {
        return m_maxUserDoReliableSendBufferNum;
    }
    /* Please see man pages for details 
    

    
    
    

    
    */
    void SetMaxUserDOReliableSendBufferNum(qUnsignedInt32 bufferNum)
    {
        m_maxUserDoReliableSendBufferNum = bufferNum;
    }

    /* Please see man pages for details 
    

    
    

    
    */
    qUnsignedInt32 GetMaxUserRMCReliableSendBufferNum() const
    {
        return m_maxUserRMCReliableSendBufferNum;
    }
    /* Please see man pages for details 
    

    
    
    

    
    */
    void SetMaxUserRMCReliableSendBufferNum(qUnsignedInt32 bufferNum)
    {
        m_maxUserRMCReliableSendBufferNum = bufferNum;
    }

private:
    static qUnsignedInt32 s_uiDOCoreCount;
    static ErrorDescriptionTable s_oErrorTable;

    void CreateGlobals();
    void DeleteGlobals();

    // Fatal error
    FirstErrorHolder *m_pFirstErrorHolder;


    qBool m_isFaultRecoveryJobProcessFailed;

    qUnsignedInt32 m_maxUserDoReliableSendBufferNum; // Number of reliable send buffers
    qUnsignedInt32 m_maxUserRMCReliableSendBufferNum; // Number of reliable send buffers

};

#undef _MANAGEINSTANCE
#undef _MANAGEGLOBALINSTANCE
#undef MANAGEINSTANCE
#undef MANAGEGLOBALINSTANCE

END_NAMESPACE_NEX

#endif
