/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOOperation_H_
#define _DOOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <OnlineCore/src/Core/Operation.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: Encapsulates duplicated object operations.
Remarks: The types of DuplicatedObject operations are AddToStore, ChangeMasterStation,
CallMethod, FaultRecovery, RemoveFromStore, ChangeDupSet, and UpdateDataSet. All
DOOperations pass via the SafetyExecutive to ensure that the given operation is permitted
before proceeding.

Refer to DuplicatedObject::OperationEnd for an example of how DOOperations can be used.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class DOOperation: public Operation
{
    //Group=Methods
protected:
    // The operations always inherit

    //{secret}
    DOOperation(DOHandle hOperationOrigin, DuplicatedObject* pDO);
    //{secret}
    DOOperation(DOHandle hOperationOrigin, DOHandle hTarget);

public:
    //{secret}
    virtual ~DOOperation();

    /*
    Description: Returns the DOHandle of the duplicated object that is associated with this operation.
    Returns: A DOHandle to the DuplicatedObject instance that is associated with the operation.
    See Also: GetAssociatedDO, DOHandle, DuplicatedObject
    */
    /* Please see man pages for details 
     
     
     
     */
    DOHandle GetAssociatedDOHandle() const
    {
        return m_refAssociatedDO.GetHandle();
    }

    /*
    Description: Returns a DOHandle to the Station that initiated the call that resulted
    in the Operation being called.
    Returns: A DOHandle to the Station that initiated the call that resulted in the
    Operation being called.
    Remarks: This method returns a DOHandle to the Station that directly triggered the
    Operation. For example, if a given Station calls DuplicatedObject::FetchDuplica which
    triggers a ChangeDupSetOperation, then this method will return a handle to the station
    where DuplicatedObject::FetchDuplica was called.

    This method is typically used by the SafetyExecutive to determine whether the station that
    triggers an Operation has the right to do so.

    Refer to CallMethodOperation for an example of this method in use.
    See Also: SafetyExecutive
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    DOHandle GetOrigin() const
    {
        return Operation::GetOrigin();
    }


    /*
    Description: Returns a pointer to the DuplicatedObject instance that is associated with this operation.
    Returns: A DuplicatedObject pointer to the instance that is associated with the operation.
    See Also: GetAssociatedDOHandle, DuplicatedObject
    */
    /* Please see man pages for details 
     
     
     
     */
    DuplicatedObject* GetAssociatedDO() const
    {
        return m_refAssociatedDO.GetDOPtr();
    }

public:

    // {secret}
    virtual DOHandle GetImplicitStationConnection() const
    {
        return INVALID_DOHANDLE;
    }

    // {secret}
    virtual DOOperation* Clone() const
    {
        return NULL;
    }

public:

    // {secret}
    virtual qBool CallsBackOnDataSet()=0;
    // {secret}
    virtual qBool CallsBackOnDataSet(qByte byIndex)=0;

protected:

    DuplicatedObject::Ref m_refAssociatedDO;
};

//{secret}
extern qBool FilterOutCoreDOOperations(const Operation* pOperation);


END_NAMESPACE_NEX

#endif
