/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _FetchContext_H_
#define _FetchContext_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>

BEGIN_NAMESPACE_NEX

// Group=Results Classes

/*
Summary: This class defines the context of a call to DuplicatedObject::FetchDuplica.
Remarks: Before a call to DuplicatedObject::FetchDuplica is made, the context of the
call must be defined via the FetchContext class. Within this class you set the station you wish to fetch a Duplica from
and the appropriate flags. You can also obtain information about the CallContext::State and
DOCallContext::Outcome of the call.

Refer to DuplicatedObject::FetchDuplica for an example of how to use a FetchContext.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 */
class FetchContext: public ActiveDOCallContext
{
    //Group=Methods
public:

    /*
    Remarks: If no arguments are passed to the constructor, the FetchContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the
    DuplicatedObject::FetchDuplica call
    will be made synchronously, that is, the system will wait for the FetchDuplica call to
    return a value or an error before performing the next task. The method DOCallContext::Wait
    can be used if the FetchDuplica call is to be made asynchronously.

    Parameters:
    hTargetStation: The station from which we wish to fetch a Duplica.
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */
    /* Please see man pages for details 
     
     
     
     
     */
    FetchContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                 qBool bSynchronousCall=true);
    virtual ~FetchContext();

    /*
    Description: Returns a DOHandle to a fetched object.
    Returns: A DOHandle to a fetched object.
    Remarks: Refer to DuplicatedObject::FetchDuplica for an example of this method in use.
    */
    /* Please see man pages for details 
     
     
     */
    DOHandle GetFetchedObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    void ReleaseFetchedObject()
    {
        m_refTargetObject=INVALID_DOHANDLE;
    }

#ifdef TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return FetchCall;
    }

    // {secret}
    qBool FetchDuplica(DOHandle hFetchedObject);
    // {secret}
    qBool FetchDuplicaImpl(DOHandle hFetchedObject);
    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // {secret}
    void DeleteOrphanOnFailure();

private:
    qBool m_bDeleteOrphanOnFailure;
};


END_NAMESPACE_NEX

#endif
