﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _JobProcessMessageList_H_
#define _JobProcessMessageList_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Job.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Core/PeriodicJob.h>


#ifdef NEX_CLIENT_PBPOOL // Client only.

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
 
        
 */
class Message;

class JobProcessMessageList: public PeriodicJob
{
private:
    JobProcessMessageList();
public:
    virtual ~JobProcessMessageList();
    /* Please see man pages for details 
     
     */
    virtual void Execute();
    /* Please see man pages for details 
     
     */
    void AddMessage(Message* pMessage, qBool processBundle);
private:
    /* Please see man pages for details 
     
     
     */
    void RemoveFrontMessage(qBool toRelease);

public:
    /* Please see man pages for details 
     
     */
    static JobProcessMessageList* GetInstance();
    /* Please see man pages for details 
     
            
            
     */
    static qBool SetWaitInterval(qUnsignedInt32 waitNormal, qUnsignedInt32 waitNoBuffer);

    virtual void SetDefaultPostExecutionState()
    {
        // If PeriodicJob remains as implemented and you set Period to 0ms, it will become JobComplete after Execute. 
        // The virtual function is implemented separately so that processing can progress with a 0ms wait when the depletion is resolved.
        // At each Execute, the state becomes the 0ms waiting state.

        SetToWaiting(GetPeriod()); // Waits with the s_waitNormal or s_waitNoBuffer value.
    }

#ifdef TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 traceFlag)
    {
        // As a trace, add the length of the message list.
        PeriodicJob::Trace(traceFlag);
        QTRACE(traceFlag, _T("MessageListSize:") << m_messageList.size());
    }
#endif // TRACE_ENABLED     

private:
    qList< std::pair<Message*, qBool> > m_messageList;
    static PseudoGlobalVariable<JobProcessMessageList*> s_pInstance;
    static qUnsignedInt32 s_waitNormal; // Waiting in a normal loop.
    static qUnsignedInt32 s_waitNoBuffer; // Wait due to buffer depletion.
    // Default value
    static const qUnsignedInt32 WAIT_NORMAL_DEFAULT = 0;
    static const qUnsignedInt32 WAIT_NORMAL_MAX = 10;
    static const qUnsignedInt32 WAIT_NOBUFFER_DEFAULT = 10;
    static const qUnsignedInt32 WAIT_NOBUFFER_MAX = 100;
};

END_NAMESPACE_NEX

#endif // NEX_CLIENT_PBPOOL

#endif // _JobProcessMessageList_H_
