﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LeaveSessionOperation_H_
#define _LeaveSessionOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/SessionOperation.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

//n1589:memo
/*
Description: This SessionOperation is performed whenever a Station leaves the Session.
Remarks: A LeaveSessionOperation will be called whenever a Station leaves the Session, either voluntarily
or due to a fault. This operation will only be called on the Duplication master of the leaving station's
Station object. Therefore, if a Station voluntarily quits a Session, the operation will be called on the leaving
station's Station object after the player's intention to quit has been indicated. On the other hand, if a fault
occurs the operation will be called on the station to which the faulting Station object migrates, as determined by
the PromotionReferee. When a fault occurs, the Station object of the faulting station will migrate to another station
so that $PRODUCTNAME$ can perform an internal clean-up before the Station object is actually deleted. Specifically,
the Station Duplication master migrates, the Station's fault-tolerant duplicated objects are migrated and the
non-fault-tolerant ones deleted, then the Station object is deleted.

When a Station leaves the Session the following sequence of events occurs:
    o   The leaving Station is removed from the Duplication master store via a RemoveFromStoreOperation,
    o   The method SessionOperation::Begin is called,
    o   An internal clean-up is performed to ensure that the leaving Station is no longer considered part of the Session,
    o   The method SessionOperation::End is called.

Note that the LeaveSessionOperation is only called on one Station. Therefore, if you wish to call a method on all
stations when a given Station leaves the Session due to a fault, you should trap the ChangeMasterStationOperation
on the leaving station's Station object, as this object will migrate before being removed from the store.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
//TODO:	It seems the SessionOperation::Begin and End functions do not exist....
//			DuplicatedObject::OperationBegin, OperationEnd?
class LeaveSessionOperation: public SessionOperation
{
    //Group=Methods
public:
    //{secret}
    LeaveSessionOperation(Station* pStation);
    //{secret}
    virtual ~LeaveSessionOperation();

    IMPL_OPERATION_COMMON_METHODS(LeaveSession);

    /*
    Description: Returns a pointer to the Station that is leaving the Session.
    Returns: A pointer to the Station that is leaving the Session.
    */
    /* Please see man pages for details
		
		 
     */
    const Station* GetLeavingStation() const;

private:

#ifdef TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

private:
    Station::Ref m_refLeavingStation;
};

END_NAMESPACE_NEX

#endif
