/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MessageBundle_H_
#define _MessageBundle_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Transport/Interface/Message.h>


BEGIN_NAMESPACE_NEX

class EndPoint;
class Station;

class MessageBundle: public RootObject
{
    friend class Station;
public:

    MessageBundle();
    virtual ~MessageBundle();

    qBool MustFlushBefore(Message* pMessage, qUnsignedInt32 uiFlags) const ;
    qBool MustFlushNow() const;

    qResult Send(EndPoint* pEndPoint);
    void Add(Message *pMessage, qUnsignedInt32 uiFlags);

    qUnsignedInt32 GetSize() const
    {
        return m_oMessage.GetTotalSize();
    }

private:

    void Clear();
    qBool IsEmpty() const;
    void MarkBundleEnd();
    void UnmarkBundleEnd();

    qUnsignedInt32 GetSendingFlags()
    {
        return m_uiSendingFlags;
    }

    //n1589
    qBool FlagIsSet(qUnsignedInt16 uiFlag)
    {
        return (m_uiSendingFlags & uiFlag) != 0;
    }

private:
    qBool SizeWillBeExceeded(Message *pMessage) const;
    qBool SizeIsExceeded() const;
    qBool TimeIsExpired() const;

#if NEX_ENABLE_FIX_BUNDLE_MAXSIZE_OPTIMIZATION
    //n1769: Made it the same value as the DEFAULT_MTU of RootTransport.
    //n2964: Consider routing, to 1250 bytes
    static const qUnsignedInt32 FIXED_MAXIMUM_SIZE = 1250;
    static qUnsignedInt32 GetMaximumSize()
    {
        return FIXED_MAXIMUM_SIZE;
    }
#else
    static qUnsignedInt32 GetMaximumSize();
#endif

public:

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:

    Message         m_oMessage;
    Time            m_tiFirstAddedMsg;
    qUnsignedInt32  m_uiSendingFlags;
};


END_NAMESPACE_NEX

#endif
