/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MethodIDGenerator_H_
#define _MethodIDGenerator_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

// Group = ID Classes

/*
Description: This class generates unique identifiers for all methods defined in the DDL.
Remarks: $PRODUCTNAME$ uses this class to generate unique identifiers for each method
(Actions, RMC) defined in the DDL file.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 */
class MethodIDGenerator: public RootObject
{
public:

    //group=Methods:

    // {secret}
    MethodIDGenerator() {}
    // {secret}
    virtual ~MethodIDGenerator() {}

    // {secret}
    static MethodID AssignID(String strMethodName, qByte classid);

    /*
    Description: Returns the MethodID of a method given its name.
    Parameters:
    strMethodName: The name of the method as a string.

    Returns: The MethodID of the method that corresponds to the given strMethodName.
    Remarks: Each method (Actions, RMC) defined in the DDL is assigned a unique ID by the
    system generated via the MethodIDGenerator class.
    See Also: GetMethodName, MethodID
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    static MethodID GetID(const String &strMethodName);  //n1769: Appended const.

    /*
    Description: Returns the name of a method given its MethodID.
    Parameters:
    id: The MethodID of the method.
    Returns: The name of the method that corresponds to the given id.
    Remarks: Each method (Actions, RMC) defined in the DDL is assigned a unique ID by the
    system generated via the MethodIDGenerator class.
    See Also: GetID, MethodID
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    static const qChar* GetMethodName(MethodID id);

    // {secret}
    static void CleanupMethodIDTable();

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

private:

    static AtomicValue s_ulNextID;
};

#define _METHODID(CLASS, METHOD)    NAMESPACE_NEX MethodIDGenerator::GetID(#CLASS "::" #METHOD) //See DOClassCGMacros.h _DOC_InitMethod

END_NAMESPACE_NEX

#endif
