/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MigrationContext_H_
#define _MigrationContext_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>

BEGIN_NAMESPACE_NEX

// Group=Results Classes

#ifdef FOR_DOC_PURPOSE
/*
Summary: This class defines the context of a Migration call.
Remarks: Before the migration of an object is performed the context of the call must be
defined via this class.  Within this class you set the appropriate flags and can also
obtain information about the CallContext::State and DOCallContext::Outcome of the call.
Migration is performed via the calls DuplicatedObject::Emigrate or DuplicatedObject::AttemptEmigration.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class MigrationContext: public DOCallContext
{
#else
class MigrationContext: public ActiveDOCallContext
{
#endif

    //Group=Methods
public:

    /*
    Remarks: By default, object migration will be made synchronously, that is, the system will
    wait for the migration call to return a value or an error before performing the next task.
    The method DOCallContext::Wait can be used if the migration call is to be made asynchronously.
    Migration can be performed via either of the calls DuplicatedObject::Emigrate
    or DuplicatedObject::AttemptEmigration.

    Parameters:
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    MigrationContext(qBool bSynchronousCall=true);
    virtual ~MigrationContext();

    //Description: Returns the DOHandle of a migrating object.
    //Returns: The DOHandle of the object that is in the process of migrating or will migrate.
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    DOHandle GetMigratingObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    DuplicatedObject* GetMigratingObjectPtr() const
    {
        return m_refTargetObject.GetDOPtr();
    }

#ifdef TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return MigrationCall;
    }

    // {secret}
    qBool MigrateObject(DOHandle hMigratingObject, DOHandle hNewStation);
    // {secret}
    qBool MigrateObjectImpl(DOHandle hMigratingObject, DOHandle hNewStation);

    // {secret}
    virtual qResult SendCallMessage(Message* pCallMessage);

    // {secret}
    virtual void ProcessOutcome(DOHandle hSourceStation, Outcome uOutcome);

    // {secret}
    qByte GetNewMasterVersion()
    {
        return m_lcNewMasterVersion.GetValue();
    }

    // {secret}
    virtual void ProcessCallCompletion();

private:
    LogicalClock m_lcNewMasterVersion;

};


END_NAMESPACE_NEX

#endif
