﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UpdateDataSetOperation_H_
#define _UpdateDataSetOperation_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: This DOOperation is performed every time a Duplica receives an update from its Duplication master.
Remarks: When a Duplica receives an update from its Duplication master this operation is called on the duplica
and the following sequence of events occurs:

    o   The SafetyExecutive is called to determine if the object may be updated. If so, we continue, if not we exit.
    o   The method DataSet::OperationBegin is called,
    o   The method DuplicatedObject::OperationBegin is called,
    o   The dataset members of the Duplica are refreshed from the Duplication master,
    o   The method DataSet::OperationEnd is called,
    o   The method DuplicatedObject::OperationEnd is called.

*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 */
class UpdateDataSetOperation: public DOOperation
{
    //Group=Methods
public:
    //{secret}
    UpdateDataSetOperation(DOHandle hOperationOrigin, DuplicatedObject* pDO, qByte byDataSetIndex, Message* pMessage);
    //{secret}
    UpdateDataSetOperation(DOHandle hOperationOrigin, DuplicatedObject* pDO, Message* pMessage);
    //{secret}
    virtual ~UpdateDataSetOperation();

    IMPL_OPERATION_COMMON_METHODS(UpdateDataSet);

    // {secret}
    qByte GetDataSetIndex() const
    {
        return m_byDataSetIndex;
    }
    // {secret}
    qBool CompleteUpdate() const
    {
        return m_bCompleteUpdate;
    }
    // {secret}
    Message* GetMessage() const
    {
        return m_pMessage;
    }

    // {secret}
    qBool IsNeedUpdateStationStatus() const
    {
        return m_bNeedUpdateStationStatus;
    }
    void NeedUpdateStationStatus()
    {
        m_bNeedUpdateStationStatus = true;
    }

private:

#ifdef TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return true;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte byIndex)
    {
        return CompleteUpdate() || m_byDataSetIndex==byIndex;
    }

private:
    qBool m_bCompleteUpdate;
    qByte m_byDataSetIndex;
    Message* m_pMessage;
    qBool m_bNeedUpdateStationStatus; // Whether StationManager::UpdateStationStatus is executed in Station::OperationEnd.
};

END_NAMESPACE_NEX

#endif
