﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UpdatePolicy_H_
#define _UpdatePolicy_H_

#include <NetZ/src/Core/DOCore/ObjDupImpl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;
class StationUpdateFilter;
class GlobalUpdateFilter;
class UpdateProtocol;
class Message;

class Selection;

class UpdatePolicy: public RootObject
{
public:
    UpdatePolicy();
    virtual ~UpdatePolicy();

    // Called to update a regular dataset.  Has the responsibility
    // to filter, and send message if appropriate.
    qResult Update(DuplicatedObject* pDO,
                   void* pDS, qByte byIndex,
                   Time tTimestamp,
                   SubStreamID subStreamID);

    // Called to add the dataset to the discovery message.  The
    // dataset is prepended with a bool that indicates if it is
    // there or not.
    void AddToDiscoveryMessage(DuplicatedObject* pDO,
                               void* pDS, qByte byIndex,
                               Station* pTarget,
                               Message* pMessage);

    void ExtractFromDiscoveryMessage(DuplicatedObject *pDO, void* pDS, qByte byIndex, Message* pMsg);
    void ExtractFromUpdateMessage(DuplicatedObject *pDO, void* pDS, qByte byIndex, Message* pMsg);

    void AddFilter(StationUpdateFilter* pFilter);
    void AddFilter(GlobalUpdateFilter* pFilter);
    void RegisterProtocol(UpdateProtocol* pProtocol);

private:

    // Called by update.
    Message* CreateUpdateMessage(DuplicatedObject *pDO, void* pDS, qByte byIndex, Time tTimestamp);
    void AddToUpdateMessage(DuplicatedObject* pDO,
                            void* pDS, qByte byIndex,
                            Time tTimestamp,
                            Message* pMessage);

private:
    qVector<GlobalUpdateFilter*> m_lstGlobalFilters;
    qVector<StationUpdateFilter*> m_lstStationFilters;
    UpdateProtocol *m_pUpdateProtocol;

    qUnsignedInt32 m_uiUpdateLoopNumber;
};


END_NAMESPACE_NEX

#endif
