﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NetZ_H_
#define _NetZ_H_

#include <NetZ/src/Core/Facade/FacadeHeaderBegin.h>

#include <NetZ/src/Core/Facade/NetZDecl.h>
#include <NetZ/src/Core/Facade/Exports.h>
#include <NetZ/src/Core/Facade/ProductFacade.h>
#include <NetZ/src/Core/DOCore/NetworkTopology.h>

BEGIN_NAMESPACE_NEX
/*
{group: Session Management}
{noAutoLink}
Description: This class encapsulates the run time of the NetZ system.
Remarks: NetZ is the class that encapsulates the run time of the NetZ system. It must be
instantiated to use NetZ. Upon destruction of this class NetZ will cease to function.
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class NetZ: public ProductFacade
{
public:
    //group=Methods

    /*
    Remarks: Only one instance of NetZ is allowed. Once this instance is
    destroyed, the currently running session (if any) is terminated and the run time stops.
    Returns: The created NetZ object.
    SYSTEMERROR_GEN_MULTIPLE_INSTANCES is signaled if more than one NetZ object is created.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    NetZ();

    /*
    Remarks: When the NetZ instance is destroyed, it cleanly shuts down all NetZ related instances.

    All the duplicated objects (Duplication master and Duplica) stored on the local station are destroyed, as long as no references to the objects still exist, /e.g./ those created via the RefTemplate or DuplicatedObject::Ref.

    This includes the local Session object, which is deleted by NetZ during shut down thereby terminating the currently running Session.

    Therefore, in order to join another Session you need to create a new NetZ instance once the current NetZ object has been deleted.

    NetZ destruction is performed in a Synchronous manner and in single thread mode, even if you have previously called UseThreads.

    Therefore, this method will block while NetZ is shut down and the objects destroyed.

    If you do not wish the destructor to block, you can call Terminate before the NetZ destructor.

    In this case, Terminate causes all NetZ related instances to be cleanly shut down however, the objects are not actually destroyed until the NetZ destructor is called.

    Note: All references to all duplicated objects must be released before the NetZ object destructor is called.
    If not,  NetZ cannot call the destructor of the referenced objects.
    See Also: Duplication master, Duplica, Terminate
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    virtual ~NetZ();

    /*
    Description: Ends, but does not destroy, this NetZ session.
    Parameters:
    pContext: A pointer to the CallContext for this call.
    Returns: true if the NetZ session terminated successfully, false if not or an error occurred.
    Remarks: When this method is called all NetZ related instances are cleanly shut down however,
    the objects are not actually destroyed until the NetZ destructor is called. As this method
    is called in an Asynchronous manner, it enables processing to continue while a NetZ session
    is in the process of shutting down.

    Whether you call the NetZ destructor, or Terminate and then the NetZ destructor to end a NetZ
    Session, the termination of processes and subsequent destruction of objects is still performed
    in the same manner. The only difference is that with the latter option, the termination part of
    the operation is called asynchronously.
    Example:
                CallContext oContext;

                NetZ::GetInstance()->Terminate(&oContext);
                // Directly wait, or continue
                // running the game loop until the call context is
                // completed.
                oContext.Wait();

                // Now, when the destructor is called, the method
                // will return instantaneously
                delete pNetZ;

    See Also: ~NetZ
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    virtual qBool Terminate(CallContext* pContext)
    {
        return ProductFacade::Terminate(pContext);
    }

    //{secret}
    virtual qResult Terminate()
    {
        return ProductFacade::Terminate();
    }

    /*
    Description: Returns a pointer to the NetZ instance.
    Returns: A pointer to the NetZ instance.
    Remarks: Use the reference returned by this method to control the NetZ object.

    Refer to Terminate for an example of this method in use.
    */
    /* Please see man pages for details 
     
     
     
     
     */
    static NetZ* GetInstance()
    {
        return static_cast<NetZ*>(ProductFacade::GetInstance());
    }

    /*
    Description:  Returns whether the NetZ instance is valid.
    Returns: true if the NetZ instance is valid, false if not or an error occurred.
    Remarks: The NetZ instance will not be valid if more than one is created.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    virtual qBool IsValid() const
    {
        return ProductFacade::IsValid();
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    virtual qResult GetFatalError()const
    {
        return ProductFacade::GetFatalError();
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    virtual void SetTerminateImmediately(qBool immediately)
    {
        ProductFacade::SetTerminateImmediately(immediately);
    }

    /* Please see man pages for details 
     
     
     
     */
    virtual qBool GetTerminateImmediately()
    {
        return ProductFacade::GetTerminateImmediately();
    }

    // {secret}
    /*
    Allows you to manually appoint the router for ClientServer sessions.
    Remarks: Applicable only when the selected mode is ClientServer. Manually appoints the
    router for Net-Z sessions by specifying the router's address. By default, if no router is defined,
    the session master (the station that created the session) is the router.
    Parameters:
    szMachine: The URL of the router
    ui16Port: The Net-Z port on the router.
    */
    void SetRouterAddress(const qChar *szMachine, const qUnsignedInt16 ui16Port);

    /*
    Description: Defines the type of network topology that Net-Z will establish.
    Remarks: Use this method before creating the NetZ facade object and Net-Z will establish the network
    of your choice. The default network topology is (the traditional) Peer2Peer where Net-Z establishes direct
    communications between all stations in a game session. Using this command, you can manually instruct
    Net-Z to establish a ClientServer network.
    In a ClientServer network, the session master acts as a server relaying game messages for all
    participants.
    Parameters:
    eSelectedTopology:  The networking topology that a Net-Z session will establish. This parameter
    must be Peer2Peer or ClientServer or Hybrid.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    static void SetNetworkTopology(NetworkTopology::Mode eSelectedTopology);


    /* Please see man pages for details 
     

     
     
     

     
     */
    static void SetDOClassInitializationCallback(InitDOClassesFunction fInitDOClasses);

private:
    // no copying and copy assignment allowed

    NetZ (const NetZ&);
    NetZ& operator= (const NetZ&);

    static InitDOClassesFunction s_fInitDOClasses;

};


END_NAMESPACE_NEX

#include <NetZ/src/Core/Facade/FacadeHeaderEnd.h>

#endif
