﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PHBDRTimeVector_H_
#define _PHBDRTimeVector_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Extensions/PHBDR/ContinuityBreak.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qQueue.h>

BEGIN_NAMESPACE_NEX

// Action
// These values cannot change.
#define DISCARD_VALUE               0x0
#define INSERT_VALUE_BEFORE_SECOND  0x1
#define INSERT_VALUE_BEFORE_THIRD   0x2
#define INSERT_VALUE_AT_THE_END     0x3
#define INSERT_VALUE_AT_THE_END_x2  0x4
#define INSERT_VALUE_AT_THE_END_x3  0x5

#define DELAY_VALUE                 0x6

#define UNKNOWN_POSITION            0xFF


class PHBDRTimeVector: public RootObject
{
public:
    PHBDRTimeVector();
    PHBDRTimeVector(qUnsignedInt32 uiTime);

    virtual ~PHBDRTimeVector();

    qUnsignedInt32 NewValue(Time tValue=SessionClock::GetTime(),
                            ContinuityBreak oContinuityBreak=CONTINUITY_BREAK_NONE, qBool bCanDelay=true);
#ifdef TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;
#endif  // TRACE_ENABLED

    void DelayValue(Time tValue, qByte byContinuityBreak);
    qUnsignedInt32 UseDelayedValue(qByte* pbyDelayedCB);
    qBool DelayedValueReadyToUse();
    qBool ValueReadyToUse(Time tValue, ContinuityBreak oContinuityBreak);

    qBool IsInTheFuture(Time tSomeTime) const;

    void Reset(qByte iNbValues);

    TimeInterval GetInterval(qInt i) const;

    qByte NbValues() const;

    TimeInterval ComputeInterval(Time tSomeFuture) const;
    Time T2() const
    {
        return m_t2;
    }
    Time T1() const;
    Time T0() const;

    Time GetValue(qUnsignedInt i) const;

    String FormatQueueTrace() const;

    qUnsignedInt GetQueueSize() const
    {
        return m_queueDelayedValues.size();
    }
    Time GetQueuedValue(qUnsignedInt iIndex) const
    {
        return (m_queueDelayedValues.at(iIndex)).tValue;
    }


#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

private:
    // Time vector itself.
    TimeInterval m_tInterval[2];
    Time m_t2;

    // Used by duplicas.
    typedef struct TimePoint
    {
        Time tValue;
        qByte byContinuityBreak;
    } TimePoint;

    qQueue<TimePoint>   m_queueDelayedValues;
};

END_NAMESPACE_NEX

#endif

