﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _CallPolicy_H_
#define _CallPolicy_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>

BEGIN_NAMESPACE_NEX

// No specific protection.  The specific class is responsible to
// protect its members.

// This is an abstract CallPolicy.  CallPolicy are used to implement
// specific restrictions and checks on methods calls.  For instance
// (the only example at the moment), there is a SingleThreadCallPolicy
// that is used to enforce that a given method is never executed by
// more than one thread.

class CallPolicy: public RootObject
{
public:
    CallPolicy() {}
    virtual ~CallPolicy() {}

    // ProDG must see the inline declarations before their
    // use.
    void BeginCall()
    {
        BeginCallImpl();
    }
    void EndCall()
    {
        EndCallImpl();
    }

    class CallScope
    {
    public:
        CallScope(CallPolicy& refPolicy): m_refPolicy(refPolicy)
        {
            m_refPolicy.BeginCall();
        }
        ~CallScope()
        {
            m_refPolicy.EndCall();
        }
    private:
        CallScope(const CallScope&);
        CallScope& operator=(const CallScope&);
    private:
        CallPolicy& m_refPolicy;
    };

private:

    virtual void BeginCallImpl()=0;
    virtual void EndCallImpl()=0;
};

END_NAMESPACE_NEX

#endif
