﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Callback_H_
#define _Callback_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/CallbackRoot.h>


BEGIN_NAMESPACE_NEX

//Group=Template Classes

/*
Description: Use this template class to construct custom callbacks.
Remarks: Via this template class you can construct and define user-defined completion callbacks that can
be called as part of a CallContext, for example. Typically, a Callback is constructed as follows:

        Callback* pCallback = qNew Callback<Inventory, DOHandle>
                            (pInventory, &Inventory::AddItem, hItem);

An example of the use of a completion callback is given in the Inventory class of the Real-time 3D Framework.
Refer to the source code of this class for further details.

See Also: CallContext, CallContext::RegisterCompletionCallback
*/
/* Please see man pages for details 
 

 
 

 
 

 
 */
template<class TargetClass, class ArgumentType> class Callback: public CallbackRoot
{
public:

    /*
    Parameters:
    pTargetObject: The duplicated object that the callback method, pMethod, will be called on.
    pMethod: The callback method that is part of the TargetClass class.
    arg: The arguments of the callback method, pMethod.
    */
    /* Please see man pages for details 
     

     
     
     
     */
    Callback(TargetClass* pTargetObject,
             void (TargetClass::*pMethod)(ArgumentType arg),
             ArgumentType arg):
        m_pTargetObject(pTargetObject),
        m_pMethod(pMethod),
        m_arg(arg)
    {
    }

    /* Please see man pages for details 
     

     
     
     
     
     */
    virtual ~Callback() {}

private:

    virtual void CallObjectMethod()
    {
        void (TargetClass::*pMethod)(ArgumentType arg)=m_pMethod;
        (m_pTargetObject->*pMethod)(m_arg);
    }

private:
    TargetClass* m_pTargetObject;
    void (TargetClass::*m_pMethod)(ArgumentType arg);
    ArgumentType m_arg;
};

#ifdef TEST_ENABLED
//{secret}
qBool CallbackTest();
#endif


END_NAMESPACE_NEX

#endif
