﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MD5Checksum_H
#define _MD5Checksum_H

#include <OnlineCore/src/Core/Checksum/Interface/ChecksumDecl.h>
#include <OnlineCore/src/Core/Checksum/Interface/ChecksumAlgorithm.h>

BEGIN_NAMESPACE_NEX
class Buffer;
class KeyCache;
class MD5;
//Group=Checksum Algorithms

/*
Description: Computes a checksum using the RSA MD5 algorithm.
Remarks: The RSA MD5 algorithm takes as input a message of arbitrary length
and produces as output a 128-bit (16 octet) "fingerprint" or "message digest"
of the input. It is conjectured that it is computationally infeasible to produce
two messages that have the same message digest or to produce a message that has
a given prespecified target message digest. This checksum is tamper-proof and is
believed to be collision-proof.
*/
class MD5Checksum: public ChecksumAlgorithm
{
public:
    //Group=Methods
    MD5Checksum();
    virtual ~MD5Checksum();

    // Summary: Calculates the checksum using the RSA MD5 algorithm.
    // Remarks:
    virtual qBool ComputeChecksum(const Buffer &oBuffer, Buffer *pChecksum);

    virtual qBool ComputeChecksum(const qByte** pBuffer, const qUnsignedInt32 nSize[], qInt length, SignatureBytes &signature);

    // {secret}
    virtual qUnsignedInt32 ComputeChecksumForTransportArray( const qByte* pBuffer[], const qUnsignedInt32 nSize[], qInt length);

    //Summary: Returns the checksum length.
    //Returns: The checksum length in bytes. The default value is 128-bits however,
    //it can be reduced via ChecksumAlgorithm::ReduceChecksumLength.
    //See Also: ChecksumAlgorithm::ReduceChecksumLength
    virtual qByte GetChecksumLength();

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

    //{secret}
    Key DeriveKey(const qChar8 *szPassword, qUnsignedInt32 uiNbIterations)
    {
        return ChecksumAlgorithm::DeriveKey(szPassword,uiNbIterations);
    }

    Key DeriveKey(const Buffer & oBuffer, qUnsignedInt32 uiNbIterations)
    {
        return ChecksumAlgorithm::DeriveKey(oBuffer,uiNbIterations);
    }

    static qBool DeriveKey(CallContext* pCallContext,
                           const qChar8 *szPassword, qUnsignedInt32 uiNbIterations,
                           Key* pkeyResult, KeyCache* pKeyCache=NULL, Job::JobType jobTypeToUse = Job::RV)
    {
        return ChecksumAlgorithm::DeriveKey(pCallContext, qNew MD5Checksum, szPassword, uiNbIterations, pkeyResult, pKeyCache, jobTypeToUse);
    }

    static qBool DeriveKey(CallContext* pCallContext,
                           const Buffer & oBuffer, qUnsignedInt32 uiNbIterations,
                           Key* pkeyResult, KeyCache* pKeyCache=NULL, Job::JobType jobTypeToUse = Job::RV)
    {
        return ChecksumAlgorithm::DeriveKey(pCallContext, qNew MD5Checksum, oBuffer, uiNbIterations, pkeyResult, pKeyCache, jobTypeToUse);
    }

    static Key MD5DerivedKey(const qChar8 *szPassword, qUnsignedInt32 uiNbIterations=0xFFFF)
    {
        MD5Checksum oMD5;
        return oMD5.DeriveKey(szPassword, uiNbIterations);
    }

    static Key MD5DerivedKey(const Buffer & oBuffer, qUnsignedInt32 uiNbIterations=0xFFFF)
    {
        MD5Checksum oMD5;
        return oMD5.DeriveKey(oBuffer, uiNbIterations);
    }
private:
    void ChecksumComputeHelper( const qByte** pBuffer, const qUnsignedInt32 nSize[], qInt length, MD5 &oOuter);
};

END_NAMESPACE_NEX

#endif

