﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __Key_H_
#define __Key_H_

// I'm not sure why.  But if the protection if _Key_H_, then the
// compilation, nameclash, complete namespace test fails.  See bug 561.

#include <OnlineCore/src/Core/PluginsDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/Buffer.h>

BEGIN_NAMESPACE_NEX

//Group=Encryption

/*
Description: Specifies the attributes of the key used by a given encryption algorithm.
Remarks: This class is used to specify specific attributes of the key used by a given
encryption algorithm.
See Also: EncryptionAlgorithm
*/
class Key: public RefCountedObject
{
public:
    //Group=Methods

    /*
    Parameters:
    pbyContent: A pointer to a buffer of bytes that contains the key.
    uiLength: The length of the key as a qUnsignedInt.
    oKey: The key as a Key object.
    oString: The key as a string.
    */
    Key();
    Key(const qByte *pbyContent, qUnsignedInt uiLength);  //n1769: Added const to pbyContent.
    Key(qUnsignedInt uiLength);
    Key(const Key &oKey);
    Key(const String &oString);
    virtual ~Key();

    Key& operator=(const Key& oKey);
    qBool   operator==(const Key& oKey) const;
    qBool   operator!=(const Key& oKey) const;

    //Summary: Returns the key length.
    //Returns: The key length as a qUnsignedInt32.
    qUnsignedInt32  GetLength() const;

    //Summary: Returns a pointer to a buffer of bytes that contains the key.
    //Returns: A pointer to a buffer of bytes that contains the key.
    const qByte *GetContentPtr() const;
    qByte *GetContentPtr();

    //Summary: Converts the key to a string.
    //Returns: The key as a string.
    String ToString() const;

    //Summary: Converts the key to a string.
    //Returns: The key as a string.
    void ExtractToString(String* pString) const;

    /*
    Summary: Traces the content of the Key used for the checksum and encryption algorithms.
    */
    void    Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;

    // {secret}
    qByte* PrepareContentPtr(qUnsignedInt32 uiLength);

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

    //Summary: Generates a random key.
    //Returns: A randomly generated key as a Key object.
    //Parameters:
    //uiKeyLength: The length of the key that will be generated as a qUnsignedInt32.
    static Key GenerateRandomKey(qUnsignedInt32 uiKeyLength);

private:
    qVector<qByte> m_vbyData;
};

#define TRACE_KEY(F,V)  {_QTRACE(F,(_T("%s=\"%s\""),_T(#V),V.ToString().CStr()));}

END_NAMESPACE_NEX

#endif
