﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PseudoGlobalVariable_H_
#define _PseudoGlobalVariable_H_

#include <OnlineCore/src/Core/PseudoGlobalVariableRoot.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

#include <OnlineCore/src/Core/CoreDecl.h>


BEGIN_NAMESPACE_NEX

template <class T>
class PseudoGlobalVariable : public PseudoGlobalVariableRoot
{
public:
    PseudoGlobalVariable(const T& oDefaultValue)
    {
        m_oDefaultValue = oDefaultValue;
        m_oValueInDefaultContext = m_oDefaultValue;
        m_pValueInContextList = NULL;

        s_oList.AddVariable((PseudoGlobalVariableRoot*)this);

        // If the size has been specified, we can allocate contexts. Else, we will allocate it when it will be specified later
        if(s_uiNbOfExtraContexts > 1)
        {
            AllocateExtraContexts();
        }
    }

    PseudoGlobalVariable()
    {
        m_oValueInDefaultContext = m_oDefaultValue;
        m_pValueInContextList = NULL;

        s_oList.AddVariable((PseudoGlobalVariableRoot*)this);

        // If the size has been specified, we can allocate contexts. Else, we will allocate it when it will be specified later
        if(s_uiNbOfExtraContexts > 1)
        {
            AllocateExtraContexts();
        }
    }


    virtual ~PseudoGlobalVariable()
    {
        s_oList.RemoveVariable((PseudoGlobalVariableRoot*)this);

        FreeExtraContexts();
    }

    PseudoGlobalVariable<T>& operator=(const T& oValue)
    {
        SetValue(GetCurrentContext(), oValue);
        return *this;
    }

    operator T()
    {
        return GetValue(GetCurrentContext());
    }

    // All pseudo global and pseudo singleton share the same
    // context.
    ICContext GetCurrentContext()
    {
        return PseudoSingleton::GetCurrentContext();
    }

    void SetValue(ICContext icInstanceContext, const T& oValue)
    {
        SYSTEMCHECK(icInstanceContext < s_uiNbOfExtraContexts);

        // Index are used like in InstanceTable,
        // Element 0 is m_oValueInDefaultContext,
        // Element 1 is m_pValueInContextList[1], Element 2 is m_pValueInContextList[2] ...
        if(icInstanceContext == 0)
        {
            m_oValueInDefaultContext = oValue;
        }
        else
        {
            SYSTEMCHECK(m_pValueInContextList != NULL);
            m_pValueInContextList[icInstanceContext] = oValue;
        }
    }


    T& GetValue()
    {
        return GetValue(GetCurrentContext());
    }

    T& GetValue(ICContext icInstanceContext)
    {
        SYSTEMCHECK(icInstanceContext < s_uiNbOfExtraContexts);

        if(icInstanceContext == 0)
        {
            return m_oValueInDefaultContext;
        }
        else
        {
            SYSTEMCHECK(m_pValueInContextList != NULL);
            return m_pValueInContextList[icInstanceContext];
        }
    }

    T* GetValuePtr(ICContext icInstanceContext)
    {
        SYSTEMCHECK(icInstanceContext < s_uiNbOfExtraContexts);

        if(icInstanceContext == 0)
        {
            return &m_oValueInDefaultContext;
        }
        else
        {
            SYSTEMCHECK(m_pValueInContextList != NULL);
            return &m_pValueInContextList[icInstanceContext];
        }
    }

    virtual void AllocateExtraContexts()
    {
        if (s_uiNbOfExtraContexts==qUnsignedInt(-1))
        {
            // This is always false.
            new T[s_uiNbOfExtraContexts];
        }
        m_pValueInContextList = qAlloc(T, s_uiNbOfExtraContexts);
        qUnsignedInt cnt;
        for(cnt=0; cnt<s_uiNbOfExtraContexts; cnt++)
        {
            new (m_pValueInContextList+cnt) T(m_oDefaultValue);
        }
    }


    virtual void FreeExtraContexts()
    {
        if(m_pValueInContextList != NULL)
        {
            for(qUnsignedInt cnt=0; cnt<s_uiNbOfExtraContexts; cnt++)
            {
                m_pValueInContextList[cnt].~T();
            }
            qFree(m_pValueInContextList);
            m_pValueInContextList = NULL;
        }
    }

    virtual void ResetContext(ICContext icContext)
    {
        SYSTEMCHECK(icContext<s_uiNbOfExtraContexts);
        m_pValueInContextList[icContext] = m_oDefaultValue;
    }


    virtual PseudoGlobalVariableRoot* GetNext()
    {
        return m_pNext;
    }


    virtual void SetNext(PseudoGlobalVariableRoot* pVariable)
    {
        m_pNext = pVariable;
    }

private:
    T* m_pValueInContextList;
    T m_oValueInDefaultContext;

    T m_oDefaultValue;
};

END_NAMESPACE_NEX

#endif
