﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _RandomSelector_H_
#define _RandomSelector_H_

#include <OnlineCore/src/Core/UtilityDecl.h>
#include <OnlineCore/src/Platform/Core/qMultiMap.h>
#include <algorithm>

BEGIN_NAMESPACE_NEX

// A utility class, to randomly select elements based on weight.

template<class T> class RandomSelector
{
public:
    RandomSelector(): m_uiTotalWeight(0) {}
    void Add(const T& t, qUnsignedInt32 uiWeight=1)
    {
        m_uiTotalWeight += uiWeight;
        m_mapValues.insert(std::pair<qUnsignedInt32,T>(m_uiTotalWeight,t));
    }
    qBool HasEntries() const
    {
        return m_uiTotalWeight>0;
    }
    T PickRandom()
    {
        qUnsignedInt32 uiRandom = Platform::GetRandomNumber(m_uiTotalWeight)+1;
        typename qMultiMap<qUnsignedInt32,T>::iterator it = m_mapValues.lower_bound(uiRandom);
        return it->second;
    }

private:
    qUnsignedInt32 m_uiTotalWeight;
    typedef qMultiMap<qUnsignedInt32,T>     MapValues;
    MapValues       m_mapValues;
};

#ifdef TEST_ENABLED
qBool TestRandomSelector();
#endif

END_NAMESPACE_NEX

#endif
