﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SingleThreadCallPolicy_H_
#define _SingleThreadCallPolicy_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/CallPolicy.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// This CallPolicy enforces that only one thread calls a method, at a given
// time. It is also possible to enforce that only one specific thread calls it
// (AcceptOneThread), as well as to restrict the maximum recursion level for a
// method. See the tests for examples.
//
// NOTE: Using the class in release mode as no effect (only ASSERT)
//==============================================================================
class SingleThreadCallPolicy : public CallPolicy
{
public:
    SingleThreadCallPolicy();
    virtual ~SingleThreadCallPolicy() {}

    void AcceptOneThread(ThreadID uiThreadID);
    void AcceptAnyThread();

    void SetMaximumRecursionLevel(qUnsignedInt32 uiMaximumRecursionLevel);
    qUnsignedInt32 GetCurrentRecursionLevel() const
    {
        return m_uiCurrentRecursionLevel;
    }

    qBool CurrentThreadIsDispatchingJobs() const;

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    // Not implemented
    SingleThreadCallPolicy(const SingleThreadCallPolicy&);
    SingleThreadCallPolicy& operator=(const SingleThreadCallPolicy&);

    virtual void BeginCallImpl();
    virtual void EndCallImpl();

private:
    // Current call sequence information
    qUnsignedInt32 m_uiCurrentRecursionLevel;
    ThreadID m_uiCurrentThread;

    // Restrictions
    qUnsignedInt32 m_uiMaximumRecursionLevel;
    ThreadID m_uiValidThread;
};

END_NAMESPACE_NEX

#endif
