﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SmoothedAverage_H_
#define _SmoothedAverage_H_

#include <OnlineCore/src/Core/UtilityDecl.h>


BEGIN_NAMESPACE_NEX

template <class Type> class SmoothedAverage: public RootObject
{
public:
    SmoothedAverage(qReal dNewWeight = QREAL(0.1))
    {
        Reset();
        m_dWeight = dNewWeight;
    }


    virtual ~SmoothedAverage()
    {
    }


    void Reset()
    {
        m_dAverage = QREAL(0.0);
        m_dTotalWeightOfEntries=QREAL(0.0);
    }


    void Add(Type tyNewValue, qReal dWeight)
    {
        qReal dAdjustedWeight=dWeight;
        if (m_dTotalWeightOfEntries + dWeight<1)
        {
            dAdjustedWeight= dWeight / (m_dTotalWeightOfEntries + dWeight);
        }

        m_dAverage = tyNewValue * dAdjustedWeight +
                     m_dAverage * (1 - dAdjustedWeight);

        m_dTotalWeightOfEntries+=dWeight;
    }

    void Add(Type tyNewValue)
    {
        Add(tyNewValue, m_dWeight);
    }

    void SetWeight(qReal dNewWeight)
    {
        m_dWeight = dNewWeight;
    }


    Type GetAverage() const
    {
        return (Type)m_dAverage;
    }


private:
    qReal m_dAverage;

    qReal m_dWeight;
    qReal m_dTotalWeightOfEntries;
};


#ifdef TEST_ENABLED
qBool SmoothedAverageTest();
#endif

END_NAMESPACE_NEX

#endif

