﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _WorkerThreads_H_
#define _WorkerThreads_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class WorkerThreads: public RootObject
{
public:
    typedef enum _State {Running, Stopping, Stopped} State;

    WorkerThreads();
    virtual ~WorkerThreads();

    qBool Start(qUnsignedInt uiNbThreads);

    qBool Stop();

    qUnsignedInt GetNbWorkers() const;
    qBool IsRunning() const;

    qBool IsAWorker(qUnsignedInt uiThreadID) const;
    qBool CurrentThreadIsAWorker() const;

    State GetState();

protected:

    virtual void Initialize() {}
    virtual void Teardown() {}

private:

    void Run(qInt iDummy);

    virtual void Work()=0;

public:

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    typedef ObjectThread<WorkerThreads, qInt> WorkerThread;

    CriticalSection m_csState;
    State m_eState;

    qVector<WorkerThread*> m_vecThreads;
};

END_NAMESPACE_NEX

#endif
