﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BandwidthCounter_H_
#define _BandwidthCounter_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

BEGIN_NAMESPACE_NEX

class BandwidthCounter : public RootObject
{
public:
    CHAINABLE(BandwidthCounter);

    typedef qChain<BandwidthCounter*> List;

    static const List& GetList();
    // This (static) critical section protects the (static) list.  It needs to be lock whenever the list is accessed.
    // It is not locked when we update the counters, so some inconsistencies are possible but this is not critical.
    static CriticalSection& GetCriticalSection();
    static void ResetAll();
    static void LogAll(qBool skipUnused = true);


    BandwidthCounter(const String& name);
    virtual ~BandwidthCounter();

    BandwidthCounter& operator+=(qUnsignedInt32 value);

    void Reset();
    void Trace(qUnsignedInt64 uiTraceFlags);
    void Log();

    const String&  GetName() const
    {
        return m_name;
    }
    qUnsignedInt32 GetMin() const
    {
        return m_min;
    }
    qUnsignedInt32 GetMax() const
    {
        return m_max;
    }
    qUnsignedInt32 GetTotal() const
    {
        return m_total;
    }
    qUnsignedInt32 GetOccurences() const
    {
        return m_occurences;
    }

    void SetName(const String& name)
    {
        m_name = name;
    }

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    BandwidthCounter(const BandwidthCounter&);
    BandwidthCounter& operator=(const BandwidthCounter&);

    static List s_lstBWCounters;
    //n1589:lock:Default
    static CriticalSection s_cs;

    String m_name;
    qUnsignedInt32 m_min;
    qUnsignedInt32 m_max;
    qUnsignedInt32 m_total;
    qUnsignedInt32 m_occurences;
};


// Utility class to define counters for both input and output for given contexts
class BandwidthCounterMap : public RootObject
{
public:
    class IOBandwidthCounter : public RootObject
    {
    public:
        IOBandwidthCounter(const String& name);

        BandwidthCounter& Incoming()
        {
            return m_incoming;
        }
        BandwidthCounter& Outgoing()
        {
            return m_outgoing;
        }

    private:
        BandwidthCounter m_incoming;
        BandwidthCounter m_outgoing;
    };

    BandwidthCounterMap(const String& name);
    virtual ~BandwidthCounterMap();

    IOBandwidthCounter& operator[](qUnsignedInt32 index);

private:
    typedef qMap<qUnsignedInt32, IOBandwidthCounter*> InternalMap;

    String m_name;
    InternalMap m_map;
};

END_NAMESPACE_NEX

#endif
