/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _CachedLogDevice_H_
#define _CachedLogDevice_H_

#include <OnlineCore/src/Platform/Core/LogDevice.h>

#ifdef CTR_TARGET

#include <nn/os.h>

BEGIN_NAMESPACE_NEX

//group=Log Devices

/* Please see man pages for details 
  
            
            

  

  
*/
class CachedLogDevice : public LogDevice
{
public:
    CachedLogDevice();
    CachedLogDevice( char *pCacheBuffer, qInt nBufferSize );
    virtual ~CachedLogDevice();

    void Initialize( char *pCacheBuffer, qInt nBufferSize );

    virtual void Output(const qChar *pLog);

    void SetCacheConut( qInt cacheCount )
    {
        nn::os::CriticalSection::ScopedLock lock( m_csBufferLocker );
        m_nCacheCount = cacheCount;
    }
    void SetSleepMilliSec( qInt sleep )
    {
        nn::os::CriticalSection::ScopedLock lock( m_csBufferLocker );
        m_nWatcherSleep = sleep;
    }

private:
    static void WatchThread( CachedLogDevice* pCachedLogDevice );

    enum
    {
        // Default number of cache count
        CACHEDLOG_DEFAULT_COUNT = 8,

        // Default monitoring thread sleep (in milliseconds)
        CACHEDLOG_DEFAULT_SLEEP = 100,

        // Monitoring thread stack size
        CACHEDLOG_WATCHER_STACK = 4 * 1024
    };

private:
    //
    char* m_pCacheBuffer;

    //
    qInt m_nBufferSize;

    //
    qInt m_nCacheCount;

    //
    qInt m_nFilledSize;

    //
    qInt m_nCalledCount;

    //
    qInt m_nWatcherSleep;

    //
    nn::os::Thread m_threadWatcher;

    //
    nn::os::CriticalSection m_csBufferLocker;

    //
    bool m_bIsFinalizing;
};

END_NAMESPACE_NEX

#endif  // CTR_TARGET
#endif  // _CachedLogDevice_H_
