﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DateTime_H_
#define _DateTime_H_

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/TimeDefs.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#ifdef CTR_TARGET
#include <nn/fnd.h>
#endif
#ifdef CAFE_TARGET
#include <cafe/os.h>
#endif
#ifdef CAFEIOP_TARGET
#include <cafe/os/OSTime.h> // for OSCalendarTime
#endif

BEGIN_NAMESPACE_NEX

// Group = Time Management

/*
Description: This class enables a date and time to be specified.
Remarks: The valid format of the date and time are respectively 'YYYY-MM-DD HH:MM:SS'.
*/
/* Please see man pages for details 




*/
class DateTime: public RootObject
{
public:
    //group=Methods

    /*
    Parameters:
    uiYear: The year of this DateTime object. The valid range is 1-9999 although
    this may differ on various platforms.
    uiMonth: The month of this DateTime object. The valid range is 01-12.
    uiDay: The day of this DateTime object. The valid range is 01-31.
    uiHour: The hour of this DateTime object. The valid range is 00-23.
    uiMinute: The minute of this DateTime object. The valid range is 00-59.
    uiSecond: The second of this DateTime object. The valid range is 00-59.
    */
    /* Please see man pages for details 
     

     

     
     */
    DateTime();
    /* Please see man pages for details 
     

     
     */
    DateTime(const DateTime& tTime);
    /* Please see man pages for details 
     

     
     
     
     
     
     
     
     */
    DateTime(qUnsignedInt16 uiYear, qUnsignedInt8 uiMonth, qUnsignedInt8 uiDay,
             qUnsignedInt8 uiHour, qUnsignedInt8 uiMinute, qUnsignedInt8 uiSecond);

#ifdef CTR_TARGET
    /* Please see man pages for details 
     

     
     
     */
    DateTime(const nn::fnd::DateTime& dateTime);
#endif
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    /* Please see man pages for details 
     

     
     
     */
    DateTime(const OSCalendarTime& dateTime);
#endif


    // {Secret}
    DateTime(qUnsignedInt64 uiTime):m_ui64Value(uiTime) {}

    /* Please see man pages for details */
    DateTime& operator=(const DateTime& tTime);

    operator qUnsignedInt64();
    operator qUnsignedInt64() const;

    /* Please see man pages for details */
    qBool operator==(const DateTime& tTime) const;
    /* Please see man pages for details */
    qBool operator!=(const DateTime& tTime) const;
    /* Please see man pages for details */
    qBool operator<(const DateTime& tTime) const;
    /* Please see man pages for details */
    qBool operator>(const DateTime& tTime) const;
    /* Please see man pages for details */
    qBool operator<=(const DateTime& tTime) const;
    /* Please see man pages for details */
    qBool operator>=(const DateTime& tTime) const;

    LargeTimeInterval operator-(const DateTime& tTime) const;

    /*
    Description: Returns the year specified by this DateTime object.
    Returns: The year specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt16 GetYear() const;

    /*
    Description: Returns the month specified by this DateTime object.
    Returns: The month specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt8 GetMonth() const;

    /*
    Description: Returns the day specified by this DateTime object.
    Returns: The day specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt8 GetDay() const;

    /*
    Description: Returns the hour specified by this DateTime object.
    Returns: The hour specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt8 GetHour() const;

    /*
    Description: Returns the minute specified by this DateTime object.
    Returns: The minute specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt8 GetMinute() const;

    /*
    Description: Returns the seconds specified by this DateTime object.
    Returns: The seconds specified by this DateTime object.
    */
    /* Please see man pages for details 
    

    
    */
    qUnsignedInt8 GetSecond() const;

    // Description: Traces the content of this DateTime object.
    /* Please see man pages for details 
     
     
     */
    void Trace(qUnsignedInt64 uiTraceFlags);

    // Description: Returns whether the DateTime instance is valid.
    // Remarks: This method will return false if you pass a date or time of the wrong type.
    /* Please see man pages for details 
     

     

     
     

     
     */
    qBool IsValid() const;

    /* Please see man pages for details 
     

     
     

     
     */
    qBool IsNever() const;

    /*
    Description: Returns the current time specified by the system where this method is called.
    Parameters:
    tDateTime: The DateTime object that describes the system time.
    Returns: The current time specified by the system (Station) where this method is called.
    */
    /* Please see man pages for details 
     

     
     */
    static void GetSystemTime(DateTime &tDateTime);

#ifdef CTR_TARGET
    /* Please see man pages for details 
     

     

     
     */
    nn::fnd::DateTime ToCTRDateTime() const;
#endif
#ifdef CAFE_TARGET
    /* Please see man pages for details 
     

     

     
     */
    OSCalendarTime ToCafeDateTime() const;
#endif

#if defined(WIN_FAMILY) || defined(UNIX_FAMILY)
    /* Please see man pages for details 
    
    
    */
    static void GetUTCSystemTime(DateTime &tDateTime);
#endif

    /*
    Description: An effectively infinite constant.
    Remarks: This constant is primarily used to set expiration times that never expire. In Python,
    this value is passed as "None".
    */
    /* Please see man pages for details 
    

    
    */      static const DateTime& Never;

    /* Please see man pages for details 
    

    
    */      static const DateTime& Future;

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif // TEST_ENABLED
    

    
    /* Please see man pages for details 
     
     
     
     */
    
    qInt64 ToEpochTime() const;
    
    /* Please see man pages for details 
     
     
     
     */
    qInt64 ToUnixEpochTime() const;
    
    /* Please see man pages for details 
     
     
     
     
     */
    
    static DateTime FromEpochTime(qInt64 epochTime);
    
    /* Please see man pages for details 
     
     
     
     
     */
    static DateTime FromUnixEpochTime(qInt64 epochTime);


private:
    /* Fields are packed into an unsigned integer 64-bits wide

      year     10 bits, 1-9999
      month     4 bits, 1-12
      day       5 bits, 1-31
      hour      5 bits, 0-23
      minute    6 bits, 0-59
      second    6 bits, 0-59
    */
    // {secret}
    qUnsignedInt64 m_ui64Value;

    qInt32 IsLeapYear(qInt32 year) const;
    qInt32 DateToDays(qInt32 year, qInt32 month, qInt32 day) const;
    
    /* Please see man pages for details 
     
     
     
     */
    void DaysToDate(qInt32 pastDays);
    
    /* Please see man pages for details 
     
     
     
     
     
     */
    static DateTime FromCustomEpochTime(qInt64 epochTime, qInt32 pastDaysOffset = 0);
};

//
// {secret}
extern StringStream& operator<<(StringStream& oStr, const DateTime& oDateTime);

END_NAMESPACE_NEX


#endif // _DateTime_H_
