﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------
*/

#ifndef _NEX_EPOCHTIME_H_
#define _NEX_EPOCHTIME_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>

BEGIN_NAMESPACE_NEX

//Estimate current EpochTime from local clock.
class EpochTime: public RootObject
{
public:
    EpochTime() : m_startEpochTime(0) {}
    ~EpochTime() {} //Do not inherit.

    void SetCurrentTime(qInt64 startEpochTime)
    {
        m_startEpochTime = startEpochTime;
        m_startTime = Time::GetTime();
    }

    qInt64 GetNowEpochTime() const //Uses the cached Time value, so Time::GetTime must be called in advance.
    {
        return m_startEpochTime + ((Time::GetCachedTime() - m_startTime)/1000);
    }

    qInt64 GetNowEpochTimeNoCached() const
    {
        return m_startEpochTime + ((Time::GetTime() - m_startTime)/1000);
    }

    qBool IsEnabled() const
    {
        return m_startTime != Time(0);
    }

private:
    Time m_startTime;
    qInt64 m_startEpochTime;
};

END_NAMESPACE_NEX

#endif /* _NEX_EPOCHTIME_H_ */
