﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _EventLog_H_
#define _EventLog_H_

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/Log.h>

BEGIN_NAMESPACE_NEX

#ifdef PERFCOUNTER_ENABLED
// forward declare PerfCounter to avoid having to include PerfCounter.h
// at this point.
class PerfCounter;
#endif // PERFCOUNTER_ENABLED

class String;
class StringStream;
class qResult;

#ifdef LOG_ENABLED

#if defined(CTR_UNICODE)

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX QLOGSTR(LEVEL, __ss.CStr());} else {}

#else

#if defined(CAFE_UNICODE)

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
    NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, _T("%ls"), __ss.CStr());} else {}

#else

#define QLOG(LEVEL,ARGS) \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, _T("%s"), __ss.CStr());} else {}

#endif

#endif


#if defined(CTR_UNICODE) || defined(CAFE_UNICODE)

#define QTLOG(LEVEL,ARGS)             \
      if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
              NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,_T("%ls"), __ss.CStr());} else {}

#else

#define QTLOG(LEVEL,ARGS)       \
    if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
        NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,_T("%s"), __ss.CStr());} else {}
#endif

// The StringStream is all done inline because we want to be able to get and use the qResult return code from the PrintErrorCode function.

#define _QLOG_ERRORCODE(LEVEL, FACILITY, ERRORCODE, TEXT) \
    PrintErrorCode(EventLog::LEVEL, QRESULT_ERROR(FACILITY, ERRORCODE), \
        StringStream() << _T("[[Error Code == ") << _T(#FACILITY) << _T("::") << _T(#ERRORCODE) \
                       << _T(" (") << hex << QERROR(FACILITY, ERRORCODE) << dec << _T(")]]   ") \
                       << TEXT)

#define QLOG_FATAL(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Fatal, FACILITY, ERRORCODE, TEXT)
#define QLOG_ERROR(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Error, FACILITY, ERRORCODE, TEXT)
#define QLOG_WARNING(FACILITY, ERRORCODE, TEXT) _QLOG_ERRORCODE(Warning, FACILITY, ERRORCODE, TEXT)
#define QLOG_INFO(FACILITY, ERRORCODE, TEXT)    _QLOG_ERRORCODE(Info, FACILITY, ERRORCODE, TEXT)
#define QLOG_DEBUG(FACILITY, ERRORCODE, TEXT)   _QLOG_ERRORCODE(Debug, FACILITY, ERRORCODE, TEXT)

#else //LOG_ENABLED

#define QLOG(LEVEL,ARGS) \
    if(false){ \
        if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
            NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QLOG(LEVEL, _T("%s"), __ss.CStr());} else {}} else {}
#define QTLOG(LEVEL,ARGS) \
    if(false){ \
        if ((LEVEL >= NAMESPACE_NEX EventLog::GetInstance()->GetLogLevel())) {\
            NAMESPACE_NEX StringStream __ss; __ss << ARGS; NAMESPACE_NEX _QTLOG(LEVEL,_T("%s"), __ss.CStr());} else {}} else {}

#define _QLOG_ERRORCODE(LEVEL, FACILITY, ERRORCODE, TEXT) \
    if(false){ \
        PrintErrorCode(EventLog::LEVEL, QRESULT_ERROR(FACILITY, ERRORCODE), \
            StringStream() << _T("[[Error Code == ") << _T(#FACILITY) << _T("::") << _T(#ERRORCODE) \
                           << _T(" (") << hex << QERROR(FACILITY, ERRORCODE) << dec << _T(")]]   ") \
                           << TEXT);} else {}

#define QLOG_FATAL(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_ERROR(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_WARNING(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_INFO(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)
#define QLOG_DEBUG(FACILITY, ERRORCODE, TEXT) QRESULT_ERROR(FACILITY, ERRORCODE)

#endif //LOG_ENABLED


// group=Logging Events

// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet.
#ifdef FOR_DOC_PURPOSE
/*
Description : Logs a string using printf syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.
szFormat: The log message. The format is the same as a standard printf function.
Remarks: The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        _QLOG(EventLog::Info, _T("This uses an int: %d"), 5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, QLOG
*/
void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

/*
Description : Logs a string using a stream-like syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.

szString: The log message in a string stream format
Remarks: The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        QLOG(EventLog::Info, _T("This uses an int")<<5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, _QLOG
*/
void QLOG( EventLog::LogLevel lLevel, const qChar *szString, ... );

/*
Description : Logs data to the trace log using printf syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event.
szFormat: The log message. The format is the same as a standard printf function.
Remarks: This method is similar to _QLOG except that it will output data to the trace log.

The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        _QTLOG(EventLog::Info, _T("This uses an int: %d"), 5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel, QTLOG
*/
void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

/*
Description : Logs data to the trace log and event log using a stream-like syntax.
Parameters:
lLevel: The EventLog::LogLevel of the event. See below.
szString: The log message.
Remarks: This method is similar to QLOG except that it will output data to the trace log and the event log.

The EventLog::LogLevel specifies the severity of the message. A given message will not
be displayed if its level is lower than the EventLog::LogLevel set via EventLog::SetLogLevel.
For example, if the EventLog::LogLevel is set to Warning, Verbose and Info messages will not be logged.

For example, a message at the Info level can be printed using the following syntax:

        QTLOG(EventLog::Info, _T("This uses an int")<<5);

See Also: EventLog::LogLevel, EventLog::SetLogLevel
*/
void QTLOG( EventLog::LogLevel lLevel, const qChar *szString, ... );

#endif

// group=Logging Events

/*
Description : Logs events that occur in your $PRODUCTNAME$ application.
Remarks: When you instantiate an EventLog object you need to attach a LogDevice and a OutputFormat
object to the EventLog object to define to which output device the data is logged and how it
is outputted.

This class implements persistent logging capabilities via a LogLevel system.
When a log output request is made, it will only be outputted to the associated LogDevice if its
LogLevel is equal to or higher than the EventLog's minimum allowed LogLevel set via SetLogLevel.
The _QLOG and _QTLOG functions are used to define log messages.
See Also: LogDevice, OutputFormat, SetLogLevel, _QLOG, _QTLOG
*/
/* Please see man pages for details 
 

 
 
 */
class EventLog : public Log
{
public:
    // group=Methods

    //{secret}
    virtual ~EventLog();

    /*
    {group: Type Definitions}
    Description : The valid LogLevel s that may be used with an EventLog.
    Remarks: The LogLevel specifies the severity of the message. A given message will not be
    displayed if its level is lower than the LogLevel set via SetLogLevel. For example, if the
    LogLevel is set to Error, then Warning, Info and Debug messages will not be logged.
    See Also: SetLogLevel, _QLOG, _QTLOG
    */
    /* Please see man pages for details 
     
     */
    enum LogLevel
    {
        // Severity level = 1. Messages with this LogLevel provide
        // information that can be used to debug the system.  In a
        // debug version of the system, the TraceLog is generally more
        // appropriate.
        Debug = 1,    //
        // Severity level = 2. Messages with this LogLevel provide information about the processes that
        // the system performs and are designed to let you know what the system is currently doing or the steps
        // that have been performed.
        Info = 2,    //
        // Severity level = 3. Messages with this LogLevel indicate a possible problem that should be fixed,
        // although it is not essential to do so.
        Warning = 3,    //
        // Severity level = 4. Messages with this LogLevel indicate an
        // error that must be fixed, but for which the system can
        // recover.
        Error = 4,    //
        // Severity level = 5. Messages with this LogLevel indicate an
        // error for which the system can not recover.
        Fatal = 5,    //
        // Severity level = 6. Messages with this LogLevel are always logged.
        Always = 6    //
    };

#ifdef LOG_ENABLED
    // {secret}
    friend void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

    // {secret}
    friend void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );

    // {secret}
    friend void QLOGSTR( EventLog::LogLevel lLevel, const qChar *szString, ... );
#endif

    /*
    Description : Sets the LogLevel of this EventLog.
    Parameters:
    lLevel: The LogLevel. This may be one of, in order of increasing severity, Debug,
    Info, Warning, Error, or Fatal.

    Remarks: The LogLevel specifies the severity of the message. A given message will not
    be displayed if its level is lower than the LogLevel set via SetLogLevel. For example,
    if the LogLevel is set to Warning, Verbose and Info messages will not be logged.
    See Also: GetLogLevel, LogLevel
    */
    /* Please see man pages for details 
     

     

     
     */
    void SetLogLevel( LogLevel lLevel );

    /*
    Description : Returns the LogLevel of this EventLog.
    Returns: The LogLevel set via SetLogLevel.
    See Also: SetLogLevel, LogLevelToString
    */
    /* Please see man pages for details 
     

     
     */
    LogLevel GetLogLevel();

    /*
    Description : Returns the LogLevel of this EventLog as a string.
    Returns: The LogLevel as a string or Unknown if no string is defined by the LogLevel.
    See Also: GetLogLevel
    */
    /* Please see man pages for details 
     

     
     */
    static const qChar *LogLevelToString( LogLevel lLevel );

    /*
    Description : Returns a pointer to the EventLog instance.
    Returns: A reference to the EventLog instance.
    Remarks: Use the reference returned by this function to control what information is
    to be traced and how. For example:

          EventLog::GetInstance()->SetLogLevel(EventLog::Warning);
    */
    /* Please see man pages for details 
     

     
     */
    static EventLog *GetInstance();

    // {secret}
    void TraceEvents()
    {
        m_bTraceEvents = true;
    }
    void TurnOffTraceEvents()
    {
        m_bTraceEvents = false;
    }

#ifdef PERFCOUNTER_ENABLED
    // {secret}
    static PerfCounter  s_ErrorCounter;
    // {secret}
    static PerfCounter  s_WarningCounter;
    // {secret}
    static PerfCounter  s_FatalCounter;
    // {secret}
    static PerfCounter * WarningCounterInstance();
    // {secret}
    static PerfCounter * FatalCounterInstance();
    // {secret}
    static PerfCounter * ErrorCounterInstance();
#endif // PERFCOUNTER_ENABLED

private:
    EventLog();

    void OutputImpl( LogLevel lLevel, const qChar *szFormat, va_list vlArgPtr );
    void TraceOutput( LogLevel lLevel, const qChar *szFormat, va_list vlArgPtr );

    virtual void AddCustomPrefix( qChar *szString, qInt iStrSize, void *pLogLevel );

    LogLevel m_lLevel;

    qBool m_bTraceEvents;
    qByte padding1[2];  // Avoids the compiler warning.

    friend class GlobalVariables;

#ifdef TEST_ENABLED
public:
    //{secret}
    static qBool Test();
#endif // TEST_ENABLED
};

// {secret}
class ScopedLogLevel
{
public:
    ScopedLogLevel(EventLog::LogLevel level);
    ~ScopedLogLevel();

private:
    const EventLog::LogLevel m_llOld;

    ScopedLogLevel& operator=(ScopedLogLevel& oSLL); //not implemented, keeps compiler happy
};

#ifdef TEST_ENABLED

//{secret}
class QLogTest
{
public:
    QLogTest(LogDevice* poNewLogDevice, qBool& testResult);
    virtual ~QLogTest();

    void SetLogExpected(qBool bLogExpected);
    void Begin();
    void End();

    qBool Test();

protected:
    virtual qUnsignedInt32 GetLogSize() = 0;
    LogDevice* GetLogDevice();
    void SetLogDevice(LogDevice* poNewLogDevice);

private:
    QLogTest& operator=(QLogTest&); //not implemented, keeps compiler happy

    qBool& m_pTestResult;
    qBool m_bLogExpected;
    qUnsignedInt32 m_uiSizeBefore;
    qUnsignedInt32 m_uiSizeAfter;

    LogDevice* m_poNewLogDevice;
    LogDevice* m_poOldLogDevice;
};

#ifdef FILE_SUPPORT

// for in memory log testing see Tests/RVFramework/SpecificPaths/TestRVGameContext.h::QLogMemTest 1.2.2.2
// the in memory tester works much more reliable and still prints the log messages to the console

//{secret}
class QLogFileTest: public QLogTest
{

public:
    QLogFileTest(const String& strFileName, qBool& testResult);
    virtual ~QLogFileTest();

protected:
    virtual qUnsignedInt32 GetLogSize();

private:
    const String* m_pstrFileName;
};

#endif // FILE_SUPPORT

//{secret}
class ScopedQLogTest
{

public:
    ScopedQLogTest(QLogTest& qTest, qBool bLogExpected);
    ~ScopedQLogTest();

private:
    QLogTest* m_pqTest;
};
#endif // TEST_ENABLED

// These used to not be declared here and it was working
// presumably because of the friends declarations above.
// When upgrading to gcc4.1.1 on NON_SUPPORT_C it broke and adding the
// prototypes here fixes it.
// {secret}
#ifdef LOG_ENABLED
void _QLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );
// {secret}
void QLOGSTR( EventLog::LogLevel lLevel, const qChar *szString, ... );
// {secret}
void _QTLOG( EventLog::LogLevel lLevel, const qChar *szFormat, ... );
#else
inline void _QLOG( EventLog::LogLevel, const qChar*, ... ) {}
inline void _QTLOG( EventLog::LogLevel, const qChar*, ... ) {}
inline void QLOGSTR( EventLog::LogLevel, const qChar*, ... ) {}
#endif

#ifdef LOG_ENABLED
#if defined(CAFE_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) OSReport(__VA_ARGS__)
#elif defined(CAFEIOP_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) printf(__VA_ARGS__)
#elif defined(CTR_TARGET)
#define NEX_NO_ALLOC_PRINT( ... ) NN_LOG(__VA_ARGS__)
#else
#define NEX_NO_ALLOC_PRINT( ... ) printf(__VA_ARGS__)
#endif
#else // LOG_ENABLED
#define NEX_NO_ALLOC_PRINT( ... )
#endif

#ifdef LOG_ENABLED
#define NO_ALLOC_QLOG(logLevel, ...) if(EventLog::GetInstance()->GetLogLevel() <= logLevel) { NEX_NO_ALLOC_PRINT(__VA_ARGS__); }
#else
#define NO_ALLOC_QLOG(logLevel, ...) if(false && logLevel) { NEX_NO_ALLOC_PRINT(__VA_ARGS__); }
#endif


// {secret}
qResult PrintErrorCode(EventLog::LogLevel level, qResult errorCode, StringStream& ssText);

END_NAMESPACE_NEX

#endif

