/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#ifndef _PlatformCoreHash_H_
#define _PlatformCoreHash_H_

#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)

#include <cstring>
#include <functional>
#include <cstddef>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
BEGIN_NAMESPACE_NEX

//The content of the API function for compatibility with Boost::hash_combine accesses the FNV-1a algorithm.
//Note: We get the hash for v using std::hash<T>()(v).
template <typename T>
inline void HashCombine(std::size_t& seed, const T& v)
{
#if defined(_WIN64) || defined(__x86_64__)
    std::size_t FNV_prime = 1099511628211ULL;
    std::size_t offset_basis = 14695981039346656037ULL;
#else
    std::size_t FNV_prime = 16777619UL;
    std::size_t offset_basis = 2166136261UL;
#endif

    seed ^= offset_basis; //Considering cases where it is 0, xor is used for offset_basis
    seed ^= std::hash<T>()(v);;
    seed *= FNV_prime;
}

END_NAMESPACE_NEX

#endif

#endif /* _PlatformCoreHash_H_ */
