﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LockChecker_H_
#define _LockChecker_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

// This class is part of the deadlock detection system.  See
// For NEX Developer http://wiki/Deadlock Detection for more details.

class LockChecker: public RootObject
{
public:
    LockChecker(qUnsignedInt32 uiLockMask);
    virtual ~LockChecker();

    // Copy constructing a critical section will create a new object.
    LockChecker(const LockChecker &);

    // Assigning a critical section alters neither object.
    LockChecker& operator=(const LockChecker &);

    qUnsignedInt32 GetCurrentLockRecursion() const
    {
        return m_uiRecursionLevel;
    }
    //{secret}
    qUnsignedInt32 GetMaximumLockRecursion() const
    {
        return m_uiMaximumLockRecursion;
    }
    //{secret}
    void SetMaximumLockRecursion(qUnsignedInt32 uiMaximumLockRecursion)
    {
        m_uiMaximumLockRecursion=uiMaximumLockRecursion;
    }
    //{secret}
    void ForceRecursionLevel(qUnsignedInt32 uiRecursionLevel)
    {
        m_uiRecursionLevel = uiRecursionLevel;
    }

    //n1589
    qBool MaskIsSet(qUnsignedInt32 uiFlag)
    {
        return (m_uiLockMask & uiFlag) == uiFlag;
    }

    //n1589:moved from private
    qUnsignedInt32 GetLockMask() const
    {
        return m_uiLockMask;
    }

    //n1589
    void SetLockMask(qUnsignedInt32 uiLockMask)
    {
        m_uiLockMask = uiLockMask;
    }

#ifdef LOCK_RESTRICTIONS_ENABLED
    //n1589
    static const qChar* GetLockName(qUnsignedInt32 lockMask);

    static qUnsignedInt32& CurrentLockState();
    static qBool LockIsOwned(qUnsignedInt32 uiLockMask);

    void BeginEnter();
    void EndEnter();
    void BeginLeave();
    void EndLeave();
#else
    void BeginEnter() {}
    void EndEnter() {}
    void BeginLeave() {}
    void EndLeave() {}
#endif

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    qBool m_bLockRestrictionsInUse;
    qByte padding1[3];  // to avoid compiler warning.
    qUnsignedInt32 m_uiLockMask;
    qUnsignedInt32 m_uiRecursionLevel;
    qUnsignedInt32 m_uiMaximumLockRecursion;
};

END_NAMESPACE_NEX

#endif
