﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LockMasks_H_
#define _LockMasks_H_

BEGIN_NAMESPACE_NEX

// This defines the lock ordering, as part of the deadlock detection
// system.

// If two locks are to be acquired together, the one with a lower mask
// must always be acquired first.  This is how deadlocks are prevented.

namespace LockMasks
{

enum LockMasks
{
    //n1589
#if 1
    //n1589:memo:Ignored should not be 0 because CriticalSection::FlagIsSet returns false if flag is 0
    //n1589:memo:if Ignored, LochChecker ignores lock checking
    Ignored = 1 << 0,
    GlobalLock = 1 << 1,
    WorkerSchedulerLock = 1 << 2,

    SystemLock = 1 << 3,
    TransportLock = 1 << 4,

    SchedulerQueue = 1 << 10,
    SQLLock = 1 << 11,
    PythonGIL = 1 << 12,
    SandboxLoadBalancerLock = 1 << 13,

    //n1589: Lock that was released by an original bug
    PRUDPEndPointLock = 1 << 14,
#if NEX_ENABLE_TRANSPORT_BUFFER_THREAD
    TransportBufferSocketLock = 1 << 15,
    TransportBufferSocketLockPerThread = 1 << 16,
#endif
    IOCompletionContextTableLock = 1 << 19,

    TransportBufferSet = 1 << 20,
    TransportBuffer = 1 << 21,

    VirtualSocketReceivedMessage = 1 << 22,
    PeerAddressLock = 1 << 23,

    SecondLastLock = 1<<28,
    Default = 1<<29,
    Dummy = 1<<30
#else
    Ignored = 0,
    GlobalLock = 1<<1,
    WorkerSchedulerLock = 1<<2,
    SystemLock = 1<<3,
    SchedulerQueue = 1<<4,

    SQLLock = 1<<5,
    PythonGIL = 1<<6,
    SandboxLoadBalancerLock = 1<<7,

    Default = 1<<30
#endif
};

} //namespace LockMasks

END_NAMESPACE_NEX

#endif
