﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SpinTest_H_
#define _SpinTest_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>

// Group=Platform Subsystem

#define INFINITE_SPIN   0xFFFFFFFF

BEGIN_NAMESPACE_NEX

class SpinTest: public RootObject
{
    // Group=
public:
    SpinTest(qUnsignedInt32 uiSpinPeriod, qUnsignedInt32 uiTimeout = INFINITE_SPIN);
    virtual ~SpinTest();

    void SetTrace(qUnsignedInt32 uiSpinPeriod, qUnsignedInt64 uiTraceFlag = D_TRACE_THREAD);
    qUnsignedInt64 GetTraceFlag();

    qBool SpinOnce(qChar *strFileName, qUnsignedInt32 uiLine, qChar *strCondition);
    void AssertOnTimeout();
    void LeaveOnTimeout();

    qUnsignedInt32 GetElapsed();
    qUnsignedInt32 GetTimeout();

    qUnsignedInt32 GetRemainingTime();

    qBool HasTimedOut()
    {
        return m_bTimedOut;
    }
    void Reset()
    {
        m_bTimedOut=false;
    }

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

private:
    qUnsignedInt16 m_ui16TimeoutConsequence;
    qUnsignedInt32  m_uiTimeout;
    Time           m_tBegin;
    qUnsignedInt32  m_uiElapsed;
    qUnsignedInt32  m_uiSpinPeriod;
    qUnsignedInt32  m_uiTracePeriod;
    qUnsignedInt64  m_uiTraceFlag;
    qBool           m_bTimedOut;
};

#define SpinOn(oSpinTest,Cond)  \
    { while ((Cond) && oSpinTest.SpinOnce(_T(__FILE__),__LINE__,_T(#Cond))) {}; }

#define SpinOnAction(oSpinTest,Cond,SpinAction) \
    { \
        while (Cond) { \
            SpinAction; \
            if (!oSpinTest.SpinOnce(_T(__FILE__),__LINE__,_T(#Cond))) { \
                break; \
            }\
        } \
    }

END_NAMESPACE_NEX

#endif
