﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _String_H_
#define _String_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#include <OnlineCore/src/Platform/Core/StringEncoding.h>

BEGIN_NAMESPACE_NEX

//$ifdef SPARK$
//Group=UI
//$else$
//Group=Data Types
//$endif$

/*
Description: The core string type used by $PRODUCTNAME$.
$ifdef RENDEZVOUS$
Remarks: This class performs basic string functions and is the core string type used by $PRODUCTNAME$.
$else$
Remarks: This class performs basic string functions and is the core string type used by Duplicated Objects.
$endif$
*/

/* Please see man pages for details 







*/
class String : public RootObject
{
public:
    //Group=Methods

    /*
    Parameters:
    szString: The string as a qChar8 or qChar16.
    */

    /* Please see man pages for details 
    

    

    
    */
    String();
    /* Please see man pages for details 
    

    

    
    */
    String(const qChar8* szString);
#ifdef UNICODE_SUPPORT
    /* Please see man pages for details 
    

    

    
    */
    String(const qWideChar* szString);
#endif
    /* Please see man pages for details 
    

    

    
    */
    String(const String& oString);
    /* Please see man pages for details 
    
    */
    virtual ~String();

    /* Please see man pages for details 
    

    

    
    */
    String& operator=(const qChar8* szString);
#ifdef UNICODE_SUPPORT
    /* Please see man pages for details 
    

    

    
    */
    String& operator=(const qWideChar* szString);
#endif
    /* Please see man pages for details 
    
    

    
    */
    String& operator=(const String& oString);
    /* Please see man pages for details 
    

    
    */
    String& operator+=(const String& strString);

    /* Please see man pages for details 
    

    

    
    */
    qBool operator==(const String& oString) const
    {
        return IsEqual(this->m_szContent,oString.m_szContent);
    }

    /* Please see man pages for details 
    

    

    
    */
    qBool operator==(const qChar* szString) const
    {
        return IsEqual(this->m_szContent,szString);
    }

    /* Please see man pages for details 
    

    

    
    */
    qBool operator!=(const String& oString) const
    {
        return !IsEqual(this->m_szContent,oString.m_szContent);
    }

    /* Please see man pages for details 
    

    

    
    */
    qBool operator!=(const qChar* szString) const
    {
        return !IsEqual(this->m_szContent,szString);
    }

    /* Please see man pages for details 
    

    
    */
    qBool operator<(const String& oString) const;

    /* Please see man pages for details 
    

    
    */
    qBool operator<=(const String& oString) const
    {
        return (*this)<oString || (*this)==oString;
    }

    /* Please see man pages for details 
    

    
    */
    qBool operator>(const String& oString) const
    {
        return !((*this)<=oString);
    }

    /* Please see man pages for details 
    

    
    */
    qBool operator>=(const String& oString) const
    {
        return !((*this)<oString);
    }

    /*
    Description: Truncates the string to the specified number of characters.
    Parameters:
    iLength: The new length of the string in number of characters.
    */
    /* Please see man pages for details 
     

     
     
     */
    String Truncate(qUnsignedInt uiLength) const;

    /*
    Description: Returns a pointer to a constant C-style version of the string.
    Returns: A pointer to a constant C-style version of the string.
    */
    /* Please see man pages for details 
     
     
     */
    const qChar* CStr() const
    {
        return m_szContent;
    }

    /*
    Description: Returns a pointer to the underlying string buffer.
    Returns: A pointer to the underlying string buffer to enable direct access to it.
    */
    /* Please see man pages for details 
     

     
     
     */
    qChar* GetBufferPtr()
    {
        return m_szContent;
    }
    /* Please see man pages for details 
     

     
     
     */
    const qChar* GetBufferPtr() const
    {
        return m_szContent;
    }

    /*
    Description: Preallocates memory for the string.
    Parameters:
    iLength: The new length of the string buffer in number of characters.
    Remarks: This function deletes the current string regardless of its size. It guarantees
    that the buffer returned by GetBufferPtr can hold at least iLength qChar(s).
    See Also: GetBufferPtr
    */
    /* Please see man pages for details 
     

     

     
     */
    void Reserve(qInt iLength);

    /*
    Description: Copies the string into a newly allocated buffer.
    Parameters:
    szCopy: A pointer to a pointer containing the address of the newly allocated buffer.
    Remarks: This function allocates the memory pointed to by szCopy. ReleaseCopy must be called
    after CreateCopy is called. If the string is Unicode, this function
    will convert the string to ANSI before copying it.
    See Also: ReleaseCopy
    */

    /* Please see man pages for details 
     

     

     
     

     

     
     */
    void CreateCopy(qChar8 **szCopy) const ;

    /*
    Description: Releases a copy of a string buffer copied by the CreateCopy function.
    Parameters:
    szCopy: A pointer to the buffer that was allocated by CreateCopy
    Remarks: This function must be called after CreateCopy has been called. You should
    not delete the buffer.
    See Also: CreateCopy
    */
    /* Please see man pages for details 
     

     

     

     
     */
    static void ReleaseCopy(qChar8 *szCopy);
#ifdef UNICODE_SUPPORT
    /* Please see man pages for details 
     
     

     

     

     
     */
    void CreateCopy(qWideChar **szCopy) const;
    /* Please see man pages for details 
     

     

     

     
     */
    static void ReleaseCopy(qWideChar *szCopy);
#endif

    /*
    Description: Returns the length of the string in number of characters.
    Returns: The length of the string in number of characters.
    */
    /* Please see man pages for details 
     

     
     */
    qUnsignedInt32 GetLength() const;

    /*
    Description: Uses a printf style format specifier to create a string.
    Parameters:
    szFormat: The format specifier string.

    Note: The parameter "..." represents all other arguments requested by szFormat.

    Remarks: The resulting string cannot be larger than NEX_LOG_STRING_BUFFER_SIZE (256) characters, otherwise it will
    be truncated. Refer to the documentation of the standard C function printf (in any good C++ book)
    for a description of the syntax and a list of the different format specifiers.
    */
    void __CDECL Format(const qChar* szFormat, ...);

    /*
    Description: Convert a string to uppercase
    */
    /* Please see man pages for details 
     
     */
    void ToUpper();

    /*
    Description: Convert a string to lowercase
    */
    /* Please see man pages for details 
     
     */
    void ToLower();

    /*
    Description: Find the location of a substring, regarding of the
    letter case
    Returns: position of first occurence of substring; -1 if not
    found
    */
    /* Please see man pages for details 
     

     
     
     
     */
    qInt FindSubstringCase(const qChar* szSubstring, qInt poi = 0) const;

    /*
    Description: Find the location of a substring, without regarding of the
    letter case
    Returns: position of first occurence of substring; -1 if not
    found
    */
    /* Please see man pages for details 
     

     

     
     */
    qInt FindSubstringNoCase(const qChar* szSubstring) const;

    /*
    Description: Determines if a String contains a certain substring without
    ignoring case.
    Returns: true if the string contains the string parameter, false otherwise
    */
    /* Please see man pages for details 
     

     

     
     
     */
    qBool ContainsCase(const String &strSubstr) const;

    /*
    Description: Determines if a String contains a certain substring while
    ignoring case.
    Returns: true if the string contains the string parameter, false otherwise
    */
    /* Please see man pages for details 
     

     

     
     
     */
    qBool ContainsNoCase(const String &strSubstr) const;

    // Converts String to qUnsignedInt64. If conversion fails, returns 0.
    qUnsignedInt64 ToUInt64() const;

    /* Please see man pages for details 
     

     

     
     
     
     

     
     
     
     */
    static int ByteArrayToBase64(const qByte* pbSrc, qUnsignedInt uiSrcLength, qChar* szADest, qUnsignedInt32 uiDestLength);

    /* Please see man pages for details 
     

     

     

     
     
     
     

     
     
     
     */
    static int Base64ToByteArray(const qChar* szASrc, qUnsignedInt32 uiSrcLength, qByte* pbDest, qUnsignedInt32 uiDestLength);

    /* Please see man pages for details 
     

     

     

     
     
     

     
     
     
     */
    static int Base64ToByteArray(const String& strSrc, qByte* pbDest, qUnsignedInt32 uiDestLength);

    //  Trace String contents at byte level
    // {secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Defines how your game/application encodes strings in
    $PRODUCTNAME$ RMCs.
    Parameters:
    uiCode: Accepted values are Latin1 and UTF8 (the default).
    Remarks: $PRODUCTNAME$ uses UTF-8 in its RMCs. Therefore, if the application
    uses Windows ANSI Latin-1, you must call this method before creating
    $FACADENAME$. When you set the default string encoding to "Latin1",
    $PRODUCTNAME$ converts between Windows ANSI Latin-1 and UTF-8 when
    packing and unpacking its RMC strings.
    */
    static void SetDefaultStringEncoding(qUnsignedInt32 uiCode);

    /*
    Description: Returns the type of encoding your game/application uses
    for strings in calls to/from $PRODUCTNAME$.
    Remarks: Returns values: StringEncoding::Latin1, StringEncoding::UTF8 or StringEncoding::Unicode
    */
    static qUnsignedInt32 GetDefaultStringEncoding()
    {
        return s_uiDefaultStringEncoding;
    }

    // {secret}
    static qBool IsEqual(const qChar* lhs, const qChar* rhs);

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

private:
    qChar* m_szContent;
    static qUnsignedInt32 s_uiDefaultStringEncoding;
};

// Additional String operators.  These are overloaded for speed.
// {secret}
String operator+(const String & strLeft, const String & strRight);
// {secret}
String operator+(const String & strLeft, const qChar * szRight);
// {secret}
String operator+(const qChar * szLeft, const String & strRight);

// String streaming.
// {secret}
extern StringStream& operator<<(StringStream& oStream, const String& strValue);

// Utility functions
extern OSDK_API void CopyString(qChar8 **pszDestination, const qChar8 *szSource);
extern OSDK_API void CopyString(qWideChar **pszDestination, const qChar8 *szSource);
extern OSDK_API void CopyString(qChar8 **pszDestination, const qWideChar *szSource);
extern OSDK_API void CopyString(qWideChar **pszDestination, const qWideChar *szSource);

END_NAMESPACE_NEX

#endif
