﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StringConversion_H_
#define _StringConversion_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>


BEGIN_NAMESPACE_NEX

class StringConversion: public RootObject
{
public:

    // Unicode
#ifdef UNICODE_VERSION
    static void Char8_2T(const qChar8* szASrc, qChar* szTDest, qUnsignedInt uiDestLength);
    static void T2Char8(const qChar* szTSrc, qChar8* szADest, qUnsignedInt uiDestLength);
    static void T2Char8Alloc(const qChar* szTSrc, qChar8** pszADest);

    static void T2Char16(const qChar* szTSrc, qChar16* szWDest, qUnsignedInt uiDestLength);
    static void Char16_2T(const qChar16* szWSrc, qChar* szTDest, qUnsignedInt uiDestLength);

    static qUnsignedInt GetUtf8ToTBufferSize( const qChar8* szASrc );

#endif

    static void Utf8ToT(const qChar8* szASrc, qChar* szTDest, qUnsignedInt uiDestLength);
    static void TToUtf8(const qChar* szTSrc, qChar8* szADest, qUnsignedInt uiDestLength);

    static qUnsignedInt GetTToUtf8BufferSize( const qChar* szTSrc );

    static void Char8ToUtf8Alloc( const qChar8* szASrc, qChar8** szADest);
    static void WideCharToUtf8Alloc( const qWideChar* szWSrc, qChar8** szADest);
    static void Utf8ToWideCharAlloc( const qChar8* szASrc, qWideChar** szWDest);

    static void T2Char8Release(qChar8* szADest);

    static void Utf8ToWideChar( const qChar8* szASrc, qWideChar* szWDest, qUnsignedInt uiDestLength);
    static void WideCharToUtf8( const qWideChar* szWSrc, qChar8* szADest, qUnsignedInt uiDestLength);


    // ANSI
#ifdef ANSI_VERSION
    static void Char8_2T(const qChar8* szASrc, qChar* szTDest, qUnsignedInt uiDestLength);
    static void T2Char8(const qChar* szTSrc, qChar8* szADest, qUnsignedInt uiDestLength);
    static void T2Char8Alloc(const qChar* szTSrc, qChar8** pszADest);

#ifdef UNICODE_SUPPORT
    static void T2Char16(const qChar* szTSrc, qWideChar* szWDest, qUnsignedInt uiDestLength);
    static void Char16_2T(const qWideChar* szWSrc, qChar* szTDest, qUnsignedInt uiDestLength);
#endif
#endif

    static qBool Encode(const qByte *szByteBuffer, qUnsignedInt uiBufferSize, qChar *szEncodedString, qUnsignedInt uiStringSize);
    static qBool Decode(const qChar *szEncodedString, qByte *szByteBuffer, qUnsignedInt uiBufferSize);

    static void StringToHex(const qChar8* szASrc, qByte* pbDest, qUnsignedInt uiDestLength);

    static const qChar base64_chars[];
    static const qByte base64_values[];
    static qUnsignedInt32 HexToString(const qByte *pbSrc, qChar8* szADest, qUnsignedInt32 uiSrcLength);
    static void ByteArrayToBase64(const qByte* pbSrc, qUnsignedInt uiSrcLength, qChar* szADest, qUnsignedInt32* puiDestLength);
    static qBool Base64ToByteArray(const qChar* szASrc, qUnsignedInt uiSrcLength, qByte* pbDest, qUnsignedInt32* puiDestLength);

#ifdef UNICODE_SUPPORT
    static void StringToHex(const qWideChar* szWSrc, qByte* pbDest, qUnsignedInt uiDestLength);
    static qUnsignedInt32 HexToString(const qByte *pbSrc, qWideChar* szWDest, qUnsignedInt32 uiSrcLength);
#endif

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif
private:
    static qByte GetCharValue(qChar8 cNumber);
#ifdef UNICODE_SUPPORT
    static qByte GetCharValue(qWideChar cNumber);
#endif
};

END_NAMESPACE_NEX

#endif
