﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifdef CTR_TARGET // [NEX1][LINUS]

#ifndef _SystemErrorResult_H_
#define _SystemErrorResult_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#include <nn/Result.h>


BEGIN_NAMESPACE_NEX

//n1769: A macro for creating NEX P2P result codes.
#define NN_DEFINE_NEX_RESULT_CONST(name, level, summary, description)  NN_DEFINE_RESULT_CONST(name, level, summary, Result::MODULE_NN_NEX, description)

//n1589:memo:0x80 = 128
const u32 NEX_ERROR_BIT   = 0x80;  //
//n1589:memo:0x40 = 64
const u32 NEX_WARNING_BIT = 0x40;  //
//n1769
const u32 NEX_DESCRIPTION_MASK = 0x3f;

//n1769: There can be up to 64 Descriptions. Of the 8 bits, the upper 2 bits are as above
//      This is because they are used to differentiate between warnings and errors.
//      Always add new error constants at the end.

//---------------------------------------------------------------------------
//  A NEX P2P Description. Information is included with the documentation for the nn:Result type.
//---------------------------------------------------------------------------
enum Description
{
    DESCRIPTION_VOID                       =                     0, //

    DESCRIPTION_OUT_OF_BOUNDARY            = NEX_ERROR_BIT   |   1, //
    DESCRIPTION_DUPLICATED_KEY             = NEX_WARNING_BIT |   2, //
    DESCRIPTION_ITEM_NOT_FOUND             = NEX_WARNING_BIT |   3, //
    DESCRIPTION_END_OF_COLLECTION          = NEX_WARNING_BIT |   4, //
    DESCRIPTION_OBJ_NOT_INIT               = NEX_ERROR_BIT   |   5, //
    DESCRIPTION_NOT_IMPLEMENTED            = NEX_ERROR_BIT   |   6, //
    DESCRIPTION_TIMEOUT                    = NEX_ERROR_BIT   |   7, //
//  DESCRIPTION_CHILDPROCESS_FAILED        = NEX_WARNING_BIT |   8, //!< Corresponds to SYSTEMERROR_GEN_CHILDPROCESS_FAILED.        This was removed because the CTR environment does not use it (is not MULTIPROCESS_SUPPORT).
    DESCRIPTION_INVALID_OPERATION          = NEX_ERROR_BIT   |   9, //
    DESCRIPTION_MULTIPLE_INSTANCES         = NEX_ERROR_BIT   |  10, //
//  DESCRIPTION_EXPIRED                    = NEX_ERROR_BIT   |  11, //!< Corresponds to SYSTEMERROR_GEN_EXPIRED.                    Description: An attempt to use an expired version of $PRODUCTNAME$ was made.  This was removed because it is not used on the CTR system.
    DESCRIPTION_OUT_OF_RANGE               = NEX_ERROR_BIT   |  12, //
    DESCRIPTION_INVALID_PARAMETER          = NEX_ERROR_BIT   |  13, //
//  DESCRIPTION_INVALID_PRODUCT_KEY        = NEX_ERROR_BIT   |  14, //!< Corresponds to SYSTEMERROR_GEN_INVALID_PRODUCT_KEY.        Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.  This was removed because it is not used on the CTR system.
    DESCRIPTION_BUFFER_EXTRACTION_OVERFLOW = NEX_ERROR_BIT   |  15, //
    DESCRIPTION_INVALID_WAIT               = NEX_ERROR_BIT   |  16, //
    DESCRIPTION_ACCESS_DENIED              = NEX_ERROR_BIT   |  17, //
//  DESCRIPTION_TRANSPORT_GENERAL_RELIABLE = NEX_ERROR_BIT   |  18, //!< Corresponds to SYSTEMERROR_TRANSPORT_GENERAL_RELIABLE.     This code was removed because it does not appear to be signaled anywhere.
//  DESCRIPTION_TRANSPORT_UDS              = NEX_ERROR_BIT   |  19, //!< Corresponds to SYSTEMERROR_TRANSPORT_UDS.                  This code was removed because it does not appear to be signaled anywhere.
    DESCRIPTION_ROLE_MISMATCH              = NEX_ERROR_BIT   |  20, //
    DESCRIPTION_INVALID_OPERATION_WARNING  = NEX_WARNING_BIT |  21, //
//  DESCRIPTION_RELIEABLE_SEND_FAILURE     = NEX_ERROR_BIT   |  22, //!< Corresponds to SYSTEMERROR_OBJDUP_RELIABLE_SEND_FAILURE.   Removed.
    DESCRIPTION_RELIEABLE_SEND_WARNING     = NEX_WARNING_BIT |  23, //
    DESCRIPTION_OBJDUP_DO_NOT_FOUND        = NEX_WARNING_BIT |  24, //
                                                                    //                                             // Omitted number.
    DESCRIPTION_OUT_OF_MEMORY              = NEX_ERROR_BIT   |  26, //
    DESCRIPTION_SOCKET_SEND_WARNING        = NEX_WARNING_BIT |  27, //
    DESCRIPTION_INVALID_NEW_STATION        = NEX_ERROR_BIT   |  28, //                                             // Description: ExecChangeDupSet was called with invalid new station.
    DESCRIPTION_HOST_MIGRATION_FAILURE     = NEX_ERROR_BIT   |  29, //

    DESCRIPTION_DC_CLUSTER_REDEFINITION        = NEX_ERROR_BIT   | 30, //
    DESCRIPTION_DC_CLUSTER_JOIN_DENIED         = NEX_ERROR_BIT   | 31, //
    DESCRIPTION_DC_CLUSTER_JOIN_FAILED         = NEX_ERROR_BIT   | 32, //
    DESCRIPTION_DC_CLUSTER_NOT_INITIALIZED     = NEX_ERROR_BIT   | 33, //
    DESCRIPTION_DC_NOT_CLUSTER_MASTER          = NEX_ERROR_BIT   | 34, //
    DESCRIPTION_DC_INVALID_STATE_TRANSITION    = NEX_ERROR_BIT   | 35, //
    DESCRIPTION_DC_CLUSTER_ALREADY_INITIALIZED = NEX_ERROR_BIT   | 36  //

      //n1769: Pay attention to ErrorCodeConverter when you add a DESCRIPTION here.
};




// SYSTEMERROR_GEN_NOT_ENOUGH_MEM   //n1769 Not used[2010/10/01 16:04:17]

// SYSTEMERROR_GEN_ILLEGAL_ACCESS   //n1769 Not used[2010/10/01 11:59:21]

// Description: An underlying WIN_FAMILY operation has failed.  The WIN_FAMILY error code can be
// retrieved with SystemError::GetLastExt
// SYSTEMERROR_GEN_WIN32_ERROR  // n1769: This shouldn't have been used with CTR TARGET, but it was!


// SYSTEMERROR_GEN_INDEX_OOB
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfBoundary,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_OUT_OF_BOUNDARY);


// n1769: memo: Duplicate selection?  This only appears to be used by the Selection class.
// SYSTEMERROR_GEN_DUPLICATED_KEY
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDuplicatedKey,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DUPLICATED_KEY);


// SYSTEMERROR_GEN_ITEM_NOT_DELETED  //n1769:memo Not used[2010/10/01 13:22:35]

// SYSTEMERROR_GEN_ITEM_NOT_FOUND
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultItemNotFound,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_ITEM_NOT_FOUND);


// SYSTEMERROR_GEN_END_OF_COLLECTION
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultEndOfCollection,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_END_OF_COLLECTION);

// SYSTEMERROR_GEN_OBJ_NOT_INIT
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjectIsNotInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_OBJ_NOT_INIT);

// SYSTEMERROR_GEN_NOT_IMPLEMENTED // n1769: memo Not used [2010/10/01 13:56:59] Of course, it would be bad if this were being used...
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultNotImplemented,
    Result::LEVEL_FATAL,
    Result::SUMMARY_NOT_SUPPORTED,
    DESCRIPTION_NOT_IMPLEMENTED);


// SYSTEMERROR_GEN_OUT_OF_ID //n1769:memo Not used[2010/10/01 13:59:10]

// SYSTEMERROR_GEN_INTERNAL //n1769:memo Not used[2010/10/01 13:59:10]


// Description: The delay allowed for an operation to occur has expired.
// SYSTEMERROR_GEN_TIMEOUT
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultTimeout,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_TIMEOUT);


//n1769: This system error is only valid in environments with MULTIPROCESS_SUPPORT.
//From the beginning, in the NEX_CLIENT_OPTIMIZATION environment, the code that throws this error is never compiled. 
//Consequently, an nn::Result value is not created either.
//SYSTEMERROR_GEN_CHILDPROCESS_FAILED


// Description: An operation was called in an invalid context.
// SYSTEMERROR_GEN_INVALID_OPERATION
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidOperation,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_OPERATION);


// Description: An attempt was made at instantiating more than one $PRODUCTNAME$ object in the same process.
// SYSTEMERROR_GEN_MULTIPLE_INSTANCES
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultMultipleInstances,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_MULTIPLE_INSTANCES);

// Description: An attempt to register more than one ExceptionHandler was made.
// SYSTEMERROR_GEN_MULTIPLE_EXCEPTION_HANDLER //n1769: memo Not used[2010/10/01 14:17:23]


//n1769: This system error is not used because the product key does not need to be verified on the CTR system.
// Description: An attempt to use an expired version of $PRODUCTNAME$ was made.
// SYSTEMERROR_GEN_EXPIRED


// Description: A method was called with an out-of-range parameter.
// SYSTEMERROR_GEN_OUT_OF_RANGE
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfRange,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_OUT_OF_RANGE);

// SYSTEMERROR_GEN_ASSERTION_FAILED //n1769: memo Not used[2010/10/01 14:32:19]

// SYSTEMERROR_GEN_DEADLOCK_DETECTED //n1769: memo Not used[2010/10/01 14:32:42]

// Description: $PRODUCTNAME$ has not been initialized.
// SYSTEMERROR_GEN_NOT_INITIALIZED   //n1769: memo Not used[2010/10/01 15:11:09]

// Description: An invalid parameter, which is typically NULL, was passed to a method.
// SYSTEMERROR_GEN_INVALID_PARAMETER
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidParameter,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_INVALID_PARAMETER);

// n1769: memo: The following Description does not seem to be correct. Apparently this error code is used when file operations (I/O) fail.
// 
// This code is actually used by the File class. Furthermore, it looks like this is limited to Windows and Unix environments.
// I do not think that it is used with NEX_CLIENT_OPTIMIZATION.
// Description: An invalid input was passed to a method.
// SYSTEMERROR_GEN_IO_ERROR


//n1769: memo: Product keys are not verified in CTR environments. Consequently, an nn::Result value is not created.
// Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.
// SYSTEMERROR_GEN_INVALID_PRODUCT_KEY


// n1769: memo: The places in which this is used are extremely limited (only CopyContent in Buffer.cpp).
// Description: A buffer extraction tried to go beyond the end of the buffer.
// SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultBufferExtractionOverflow,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_ARGUMENT,
    DESCRIPTION_BUFFER_EXTRACTION_OVERFLOW);

//n1769: memo: The timeout value is invalid.
// Description: The timeout value set is invalid.
// Remarks: This error is returned if DuplicatedObject::Create or DuplicatedObject::Publish are called
// in a single-thread context with a timeout value other than 0.
// See Also: DuplicatedObject::Create, DuplicatedObject::Publish
// SYSTEMERROR_GEN_INVALID_WAIT
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidWait,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_WAIT);

// n1769: This, too, is actually not used in very many places.
// Only once in the DuplicatedObject::ExecuteOperation function, it seems.
// Description: Access to the requested resource was refused.
// SYSTEMERROR_GEN_ACCESS_DENIED
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultAccessDenied,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_WRONG_ARGUMENT,
    DESCRIPTION_ACCESS_DENIED);

// Description: The operation was denied.
// SYSTEMERROR_GEN_OP_DENIED  // n1769: We couldn't find anywhere that attempts to "set" this code.

//n1769: Unused
// SYSTEMERROR_TRANSPORT_UDS
//NN_DEFINE_NEX_RESULT_CONST(
//    ResultUdsFailure,
//    Result::LEVEL_STATUS,
//    Result::SUMMARY_CANCELLED,
//    DESCRIPTION_TRANSPORT_UDS);

//n1769: This error occurs when a UDS client attempts to create a session or a UDS master attempts to join a session.
// 
// SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidUdsStatus,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_ROLE_MISMATCH);

//n1769: This error occurs when a UDS client attempts to create a session or a UDS master attempts to join a session.
// 
// SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidVirtualNetworkStatus,
    Result::LEVEL_USAGE,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_ROLE_MISMATCH);


// SYSTEMERROR_GEN_INVALID_OPERATION_WARNING
// {Secret}
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidOperationWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_OPERATION_WARNING);

// SYSTEMERROR_OBJDUP_RELIABLE_SEND_FAILURE
//NN_DEFINE_NEX_RESULT_CONST(
//    ResultObjDupReliableSendFailure,
//    Result::LEVEL_STATUS,
//    Result::SUMMARY_CANCELLED,
//    DESCRIPTION_RELIEABLE_SEND_FAILURE);


// SYSTEMERROR_OBJDUP_RELIABLE_SEND_WARNING
// {Secret}
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjDupReliableSendWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_RELIEABLE_SEND_WARNING);


// SYSTEMERROR_OBJDUP_DO_NOT_FOUND
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultObjDupDoNotFound,
    Result::LEVEL_STATUS,
    Result::SUMMARY_NOT_FOUND,
    DESCRIPTION_OBJDUP_DO_NOT_FOUND);

/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultOutOfMemory,
    Result::LEVEL_STATUS,
    Result::SUMMARY_OUT_OF_RESOURCE,
    DESCRIPTION_OUT_OF_MEMORY);


// SYSTEMERROR_TRANSPORT_SOCKET_SEND_WARNING
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultSocketSendWarning,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_SOCKET_SEND_WARNING);


// Description: ExecChangeDupSet was called with invalid new station.
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultInvalidNewStation,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_INVALID_NEW_STATION);


/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultUdsHostMigrationFailure,
    Result::LEVEL_STATUS,
    Result::SUMMARY_STATUS_CHANGED,
    DESCRIPTION_HOST_MIGRATION_FAILURE);




/*
Description: An attempt was made to create or join the session more than once.
See Also: Session::CreateSession, Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_REDEFINITION
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterRedefinition,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_REDEFINITION);

/*
Description: An attempt was made to join a session and the station hosting the session
denied the join request.
See Also: Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_JOIN_DENIED
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterJoinDenied,
    Result::LEVEL_STATUS,
    Result::SUMMARY_CANCELLED,
    DESCRIPTION_DC_CLUSTER_JOIN_DENIED);


/*
Description: The join operation failed due to a transient network failure.
See Also: Session::JoinSession.
*/
// SYSTEMERROR_DC_CLUSTER_JOIN_FAILED
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterJoinFailed,
    Result::LEVEL_STATUS,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_JOIN_FAILED);


/*
Description: An operation that requires the session to be initialized was called, but
the session had not been initialized beforehand.
See Also: Session::CreateSession, Session::JoinSession
*/
// SYSTEMERROR_DC_CLUSTER_NOT_INITIALIZED
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterNotInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_NOT_INITIALIZED);


// Description: An operation that must be called on the session master was called on a secondary node.
// SYSTEMERROR_DC_NOT_CLUSTER_MASTER
/* Please see man pages for details 
    
    
    
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcNotClusterMaster,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_NOT_CLUSTER_MASTER);


// Description: An invalid system state transition occurred.
// SYSTEMERROR_DC_INVALID_STATE_TRANSITION
/* Please see man pages for details 
    
    
    
           
           
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcInvalidStateTransition,
    Result::LEVEL_FATAL,
    Result::SUMMARY_NOT_SUPPORTED,
    DESCRIPTION_DC_INVALID_STATE_TRANSITION);


/*
Description: A method that must be called prior to calling Session::CreateSession, Session::JoinSession, or
Session::CreateOrJoinSession was called after any one of these three methods.
See Also: Session::CreateSession, Session::JoinSession, Session::CreateOrJoinSession
*/
// SYSTEMERROR_DC_CLUSTER_ALREADY_INITIALIZED
/* Please see man pages for details 
    
    
    
           
 */
NN_DEFINE_NEX_RESULT_CONST(
    ResultDcClusterAlreadyInitialized,
    Result::LEVEL_PERMANENT,
    Result::SUMMARY_INVALID_STATE,
    DESCRIPTION_DC_CLUSTER_ALREADY_INITIALIZED);



/* Please see man pages for details 
    
 */
class SystemErrorResult
{
public:
    /* Please see man pages for details 
    
     */
    SystemErrorResult(void);


    /* Please see man pages for details 
    
     */
    ~SystemErrorResult(void);


    /* Please see man pages for details 
    
    
    
     */
    static nn::Result Convert(qUnsignedInt32 err);


    /* Please see man pages for details 
    
    
    
     */
    static bool IsNexError(nn::Result result);


    /* Please see man pages for details 
    
    
    
     */
    static bool IsNexWarning(nn::Result result);


    /* Please see man pages for details 
    
    
    
     */
    static bool IsReinitializeRequired(nn::Result result);


    /* Please see man pages for details 
    
    
    
     */
    static bool IsResetRequired(nn::Result result);


    /* Please see man pages for details 
    
    
    
     */
    static bool IsProgrammingError(nn::Result result);


private:
    // The copy constructor and the assignment operator are private.
    SystemErrorResult           (const SystemErrorResult&);
    SystemErrorResult& operator=(const SystemErrorResult&);
};


END_NAMESPACE_NEX

#endif  // end of _SystemErrorResult_H_

#endif // #ifdef CTR_TARGET // [NEX1][LINUS]
