/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Type_H_
#define _Type_H_


#if defined(NINTENDO_FAMILY)
#include <cstddef>
using std::size_t;
#endif

// STL not optimized for level 4 warnings
#ifdef WIN_FAMILY
#pragma warning( push, 1 )
#pragma warning(disable : 4786)
#endif // WIN_FAMILY
#ifdef TURN_MATHWARN_OFF
#pragma mathwarn off
#endif // TURN_MATHWARN_OFF

#ifndef CAFEIOP_TARGET
#include <string>
#endif

#ifdef WIN_FAMILY
#pragma warning( pop ) // restore previous warning
#endif // WIN_FAMILY
#ifdef TURN_MATHWARN_OFF
#pragma mathwarn on
#endif // TURN_MATHWARN_OFF

#if defined(LINUX_TARGET) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(MACOSX_TARGET) || defined(CAFEIOP_TARGET)
// On Linux, uint64 constants have to be appended with ULL
#define _ULL(CONSTANT)  CONSTANT##ULL
#define _LL(CONSTANT)   CONSTANT##LL
#else
// On Windows, prior to .NET03, that was not a valid syntax
#define _ULL(CONSTANT)  CONSTANT
#define _LL(CONSTANT)   CONSTANT
#endif

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
typedef bool                    qBool;

/* Please see man pages for details 
    
*/
typedef unsigned char           qByte;

/* Please see man pages for details 
    
*/
typedef signed char             qInt8;

/* Please see man pages for details 
    
*/
typedef unsigned char           qUnsignedInt8;

/* Please see man pages for details 
    
*/
typedef short                   qInt16;

/* Please see man pages for details 
    
*/
typedef unsigned short          qUnsignedInt16;

/* Please see man pages for details 
    
*/
typedef int                     qInt32;

/* Please see man pages for details 
    
*/
typedef unsigned int            qUnsignedInt32;

/* Please see man pages for details 
    
*/
typedef int                     qInt;

/* Please see man pages for details 
    
*/
typedef unsigned int            qUnsignedInt;


/* Please see man pages for details 
    
*/
typedef float                   qFloat;

/* Please see man pages for details 
    
*/
typedef double                  qDouble;

/* Please see man pages for details 
    
*/
typedef float                   qReal;
#define QREAL(_num) (_num##f)

#ifdef WIN_FAMILY
typedef __int64                 qInt64;
typedef unsigned __int64        qUnsignedInt64;
typedef qInt32                  s32;
typedef qUnsignedInt32          u32;
typedef qInt64                  s64;
typedef qUnsignedInt64          u64;
#endif // WIN_FAMILY

#ifdef UNIX_FAMILY
typedef long long int           qInt64;
typedef long long unsigned int  qUnsignedInt64;
#endif // UNIX_FAMILY

#if defined(NON_SUPPORT_B_TARGET)
typedef long                    qInt64;
typedef unsigned long           qUnsignedInt64;
#endif  // NON_SUPPORT_B_TARGET

#if defined(NON_SUPPORT_A_TARGET)
typedef long long               qInt64;
typedef unsigned long long      qUnsignedInt64;
#endif  // NON_SUPPORT_B_TARGET

#if defined(NON_SUPPORT_C_TARGET)
typedef int64_t                 qInt64;
typedef uint64_t                qUnsignedInt64;
#endif  // NON_SUPPORT_C_TARGET

#if defined(NINTENDO_FAMILY)

/* Please see man pages for details 
    
*/
typedef long long               qInt64;

/* Please see man pages for details 
    
*/
typedef unsigned long long      qUnsignedInt64;
#endif  // NINTENDO_FAMILY

/* Please see man pages for details 
    
*/
typedef char                    qChar8;

#if defined(DISTINCT_UNICODE_TYPE)
typedef wchar_t                 qChar16;
#else

/* Please see man pages for details 
    
*/
typedef qUnsignedInt16          qChar16;
#endif

/* With Linux, wchar_t is 32-bit wide, probably to accommodate UTF-32. */
#if defined(UNIX_FAMILY)
typedef wchar_t                 qWideChar;
#else

/* Please see man pages for details 
    
*/
typedef qChar16                 qWideChar;
#endif

#ifdef UNICODE_VERSION
#define USE_UNICODECHAR
#endif

#if defined(UNICODE_VERSION) && !defined(UNICODE_SUPPORT)
/* This is a special condition that was added specifically for Unreal,
 * where UNICODE is defined and so is detected UNICODE_VERSION, even on
 * platforms were we do not offer UNICODE_SUPPORT (for example,
 * platforms).  In this case, UNICODE will be defined, but we must link
 * with the ANSI version of our libs */
#undef USE_UNICODECHAR
#endif

#ifdef USE_UNICODECHAR
/* Please see man pages for details 
    
*/
typedef qChar16                 qChar;
#else
/* Please see man pages for details 
    
*/
typedef qChar8                  qChar;
#endif

#if !defined(WIN_FAMILY) && !defined(CTR_TARGET) && !defined(CAFE_TARGET)
#define HANDLE                  void*
#define LPSTR                   qChar8*
#define LPTSTR                  LPSTR
#endif // ndef WIN_FAMILY

//Description: An ID generated by an IDGenerator object that can be assigned to a given entity.
//See Also: IDGenerator, DuplicatedObject::Create
/* Please see man pages for details 
    
*/
typedef qUnsignedInt32          ID;

#define QBYTE_BIT             8                     /* number of bits in a qByte */
#define QBYTE_MAX             0xff                  /* maximum qByte value */
#define QBYTE_MIN             0                     /* minimum qByte value */

#define QINT8_BIT             8                     /* number of bits in a qInt8 */
#define QINT8_MAX             127                   /* maximum qChar value */
#define QINT8_MIN           (-QINT8_MAX - 1)        /* minimum qChar value */

#define QUNSIGNEDINT8_BIT     QBYTE_BIT             /* number of bits in a qUnsignedInt8 */
#define QUNSIGNEDINT8_MAX     QBYTE_MAX
#define QUNSIGNEDINT8_MIN     QBYTE_MIN

#define QINT16_BIT            16                    /* number of bits in a qInt16 */
#define QINT16_MAX            32767                 /* maximum qInt16 value */
#define QINT16_MIN          (-QINT16_MAX - 1)       /* minimum qInt16 value */

#define QUNSIGNEDINT16_BIT    QINT16_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT16_MAX    0xffff                /* maximum qUnsignedInt16 value */
#define QUNSIGNEDINT16_MIN    0                     /* minimum qUnsignedInt16 value */

#define QINT32_BIT            32                    /* number of bits in a qInt32 */
#define QINT32_MAX            2147483647            /* maximum qInt32 value */
#define QINT32_MIN          (-QINT32_MAX - 1)       /* minimum qInt32 value */

#define QUNSIGNEDINT32_BIT    QINT32_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT32_MAX    0xffffffff            /* maximum qUnsignedInt32 value */
#define QUNSIGNEDINT32_MIN    0                     /* minimum qUnsignedInt32 value */

#define QINT_BIT              QINT32_BIT
#define QINT_MAX              QINT32_MAX            /* maximum qInt value */
#define QINT_MIN              QINT32_MIN            /* minimum qInt value */

#define QUNSIGNEDINT_BIT      QUNSIGNEDINT32_BIT
#define QUNSIGNEDINT_MAX      QUNSIGNEDINT32_MAX    /* maximum qUnsignedInt value */
#define QUNSIGNEDINT_MIN      QUNSIGNEDINT32_MIN    /* minimum qUnsignedInt value */

#define QINT64_BIT            64                    /* number of bits in a qInt64 */
#define QINT64_MAX            _ULL(9223372036854775807)   /* maximum qInt64 value */
#define QINT64_MIN            (-QINT64_MAX - 1)     /* minimum qInt64 value */

#define QUNSIGNEDINT64_BIT    QINT64_BIT            /* number of bits in a qInt16 */
#define QUNSIGNEDINT64_MAX    _LL(0xffffffffffffffff)    /* maximum qUnsignedInt64 value */
#define QUNSIGNEDINT64_MIN    0                     /* minimum qUnsignedInt64 value */

#define QCHAR8_BIT         QINT8_BIT             /* number of bits in a Unicode qChar */
#define QCHAR8_MAX         QINT8_MAX             /* maximum Unicode qChar value */
#define QCHAR8_MIN         QINT8_MIN             /* minimum Unicode qChar value */

#define QCHAR16_BIT      QUNSIGNEDINT16_BIT    /* number of bits in a Unicode qChar */
#define QCHAR16_MAX      QUNSIGNEDINT16_MAX    /* maximum Unicode qChar value */
#define QCHAR16_MIN      QUNSIGNEDINT16_MIN    /* minimum Unicode qChar value */

#ifdef UNICODE_VERSION
#define QCHAR_BIT             QCHAR16_BIT      /* number of bits in a Unicode qChar */
#define QCHAR_MAX             QCHAR16_MAX      /* maximum Unicode qChar value */
#define QCHAR_MIN             QCHAR16_MIN      /* minimum Unicode qChar value */
#else
#define QCHAR_BIT             QCHAR8_BIT         /* number of bits in an ANSI qChar */
#define QCHAR_MAX             QCHAR8_MAX         /* maximum ANSI qChar value */
#define QCHAR_MIN             QCHAR8_MIN         /* minimum ANSI qChar value */
#endif // UNICODE_VERSION

#ifdef TEST_ENABLED
extern qBool TypeTest();
#endif

END_NAMESPACE_NEX

#endif // _Type_H_
