/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _InetAddress_H_
#define _InetAddress_H_


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Core/Hash.h>

BEGIN_NAMESPACE_NEX

//n1589
#define NEX_INET_ADDRESS_STRING_LEN_MAX sizeof(_T("255.255.255.255"))

class InetAddress: public RootObject
{
public:
    //n1589
    static void EnableAutoLookup(qBool o);

    InetAddress();
    InetAddress(const qChar *szMachine, const qUnsignedInt16 uiPort);
    InetAddress(const qUnsignedInt32 uiAddress, const qUnsignedInt16 uiPort);
    InetAddress(void *pAddr, qUnsignedInt uiSizeOfType);
    InetAddress(const InetAddress &oInetAddress);
    virtual ~InetAddress();

    void Init();

    qBool SetAddress(const qChar *szAddress);
    void SetAddress(const qUnsignedInt32 uiAddress);
    void SetNetworkAddress(const qUnsignedInt32 uiAddress);
    qBool GetAddress(qChar *szAddress, qUnsignedInt32 uiAddressLen) const;
    qUnsignedInt32 GetAddress() const;
    String GetAddressStr() const;

    void SetPortNumber(const qUnsignedInt16 ui16Port);
    void SetNetworkPortNumber(const qUnsignedInt16 ui16Port);
    qUnsignedInt16 GetPortNumber() const;
    String GetPortNumberStr() const;
    qBool GetPortNumber(qChar *szPort, qUnsignedInt32 uiPortLen) const;

    qBool SetLocalHost();
    qBool IsLocalHost() const;

    //n2967: With Windows, if Winsock is included, there are conflicts with max, etc., so it is not optimized
#ifdef WIN32_TARGET
    qInt64 GetKey() const;
    qBool operator==(const InetAddress &oInetAddress) const
    {
        return (GetKey()==oInetAddress.GetKey());
    }
#else
    inline qInt64 GetKey() const
    {
        return
            ( static_cast<qInt64>(GetSoackAddrInAddress(GetSockAddrIn())) << 32 ) |
            ( static_cast<qInt64>(GetSoackAddrInPort(GetSockAddrIn())) ); //n2343: Used without paying attention to byte order to increase speed
    }

    inline qBool operator==(const InetAddress &oInetAddress) const
    {
        return (GetSoackAddrInAddress(GetSockAddrIn()) == GetSoackAddrInAddress(oInetAddress.GetSockAddrIn()))
               && (GetSoackAddrInPort(GetSockAddrIn()) == GetSoackAddrInPort(oInetAddress.GetSockAddrIn()));
    }
#endif

    qBool operator!=(const InetAddress &oInetAddress) const
    {
        return !operator==(oInetAddress);
    }
    qBool operator<(const InetAddress &oInetAddress) const
    {
        return (GetKey()<oInetAddress.GetKey());
    }

    InetAddress &operator=(const InetAddress &oInetAddress);

    void Trace(qUnsignedInt64 uiTraceFlag) const;

    void ToStr(qChar *tszStr) const;
    String ToStr() const;


    void* GetAddressPtr()
    {
        return (void*)&m_oAddressBytes;
    }

#if 0
    qBool SetAddress(XNADDR &refXNADDR);
    qBool GetXNADDR(XNADDR *pXNADDR);
#endif

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    //n1589
    static qBool s_bAutoLookup;

    static qUnsignedInt32 String2Address(const qChar8 *cp);

#ifdef WIN32_TARGET
    qUnsignedInt32 m_oAddressBytes[32];
#else
    SockAddrStorage m_oAddressBytes;

    inline SockAddrIn &GetSockAddrIn() const
    {
        return *((NAMESPACE_NEX SockAddrIn*)&m_oAddressBytes);
    }

    static inline qUnsignedInt32 GetSoackAddrInAddress(const SockAddrIn& x);
    static inline qUnsignedInt16 GetSoackAddrInPort(const SockAddrIn& x);
#endif

};

StringStream& operator<<(StringStream& oStr, const InetAddress& oAddress);

#ifndef WIN32_TARGET

#if defined(WII_STACK) || defined(CTR_TARGET)
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.addr.addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.port;
}
#elif defined(BERKELEY_STACK_SUPPORT)
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.sin_addr.s_addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.sin_port;
}
#else
qUnsignedInt32 InetAddress::GetSoackAddrInAddress(const SockAddrIn& x)
{
    return x.sin_addr.S_un.S_addr;
}
qUnsignedInt16 InetAddress::GetSoackAddrInPort(const SockAddrIn& x)
{
    return x.sin_port;
}
#endif
#endif

#if (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY)


inline std::size_t hash_value (const InetAddress& d)
{
    std::size_t hash = 0;
    HashCombine(hash, d.GetAddress());
    HashCombine(hash, d.GetPortNumber());

    return hash;
}

#endif

END_NAMESPACE_NEX

#endif
