﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _InterfaceInfo_H_
#define _InterfaceInfo_H_

//Group = NetworkSubsystem

#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>


/*
Description: Indicates that the interface is a point-to-point interface.  A dial-up (PPP)
connection is a good example where this flag would be set.
See Also: EnumerateLANInterfaces, GetFlags
*/
#define INTERFACE_FLAG_PT2PT        (1)

/*
Description: Indicates that this interface supports broadcasting.  Normally this
flag could be set on a LAN and cleared on a dial-up connection.
See Also: EnumerateLANInterfaces, GetFlags
*/
#define INTERFACE_FLAG_BROADCAST    (1<<1)

/*
Description: Indicates that this interface is the local loopback address.
See Also: EnumerateLANInterfaces, GetFlags
*/
#define INTERFACE_FLAG_LOOPBACK     (1<<2)

BEGIN_NAMESPACE_NEX

class InterfaceInfo: public RootObject
{

public:
    InterfaceInfo();
    ~InterfaceInfo();

    void SetAddress(qUnsignedInt32 uiAddress);
    void SetBroadcastAddress(qUnsignedInt32 uiBroadcastAddress);
    void SetMask(qUnsignedInt32 uiMask);
    void SetFlags(qUnsignedInt32 uiFlags);
    void SetName(qChar* szName);

    /*
    Description: Returns the IP address of this network interface.
    See Also: EnumerateLANInterfaces, GetBroadcastAddress, GetMask, GetFlags, GetName
    */
    qBool GetAddress(qChar *tszAddress, qUnsignedInt32 uiSize);
    qUnsignedInt32 GetAddress();

    /*
    Description: Returns the broadcast address of this network interface.
    See Also: EnumerateLANInterfaces, GetAddress, GetMask, GetFlags, GetName
    */
    qBool GetBroadcastAddress(qChar *tszBroadcastAddress, qUnsignedInt32 uiSize);
    qUnsignedInt32 GetBroadcastAddress();

    /*
    Description: Returns the subnet mask of this network interface.
    See Also: EnumerateLANInterfaces, GetAddress, GetBroadcastAddress, GetFlags, GetName
    */
    qBool GetMask(qChar *tszMask, qUnsignedInt32 uiSize);
    qUnsignedInt32 GetMask();

    /*
    Description: Returns the name associated with this network interface.

    Remarks: This method only returns the name of the local station when used on a Linux
    platform. On any other platform the method does not return any value.

    See Also: EnumerateLANInterfaces, GetAddress, GetBroadcastAddress, GetMask, GetFlags
    */
    qBool GetName(qChar *tszName, qUnsignedInt32 uiSize);
    const qChar* GetName();

    /*
    Description: Returns the flags associated with this network interface.

    See Also: EnumerateLANInterfaces, GetAddress, GetBroadcastAddress, GetMask, GetName,
    INTERFACE_FLAG_PT2PT, INTERFACE_FLAG_BROADCAST, INTERFACE_FLAG_LOOPBACK
    */
    qBool GetFlags(qChar *tszFlags, qUnsignedInt32 uiSize);
    qUnsignedInt32 GetFlags();


    void Trace(qUnsignedInt64 uiTraceFlags);

private:
    qBool   Addr2Str(qUnsignedInt32 uiAddress, qChar *tszAddress, qUnsignedInt32 uiSize);

    qUnsignedInt32  m_uiAddress;
    qUnsignedInt32  m_uiBroadcastAddress;
    qUnsignedInt32  m_uiMask;
    qUnsignedInt32  m_uiFlags;
    String m_strName;
};


END_NAMESPACE_NEX

#endif
