﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>

#ifndef _NN_NEX_JobNameResolve_H_
#define _NN_NEX_JobNameResolve_H_

BEGIN_NAMESPACE_NEX

class RootTransport;

class JobNameResolve : public StepSequenceJob, public NonCopyable
{
public:
    static const qUnsignedInt32 DEFAULT_JOB_DELAY_INTERNAL_THREAD = 10;
    static const qUnsignedInt32 THREAD_RUNNING_CHECK_WAIT = 50;

    typedef qBool (RootTransport::*ResolveMethod)();

    JobNameResolve(qUnsignedInt threadPriority, qInt32 coreNo, ResolveMethod resolveMethod);
    virtual ~JobNameResolve();

    void Initialize(qUnsignedInt32 uiAssociatedCallID);


protected:
    void StepRunThread();
    void StepCheck();
    void StepCancelWait();

    void CheckThread(int);

    void CompleteJob(const qResult& result);

    qUnsignedInt32 m_AssociatedCallID;
    qResult m_result;
    ObjectThread<JobNameResolve, int>* m_pThread;
    ForcedCriticalSection m_lockResult;

    qUnsignedInt m_threadPriority;
    qInt32 m_coreNo;
    ResolveMethod m_resolveMethod;

private:
    static PseudoGlobalVariable<qBool> g_isJobNameResolveRunning;
};


END_NAMESPACE_NEX

#endif // _NN_NEX_JobNameResolve_H_

