﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NATRelayInterface_H_
#define _NATRelayInterface_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/RelaySignatureKey.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalResult.h>

BEGIN_NAMESPACE_NEX

// Forward declaration
class Buffer;
class CallContext;
class NATTraversalEngine;
class SignatureKey;
class RelayAddress;
class CallContext;
class NATTraversalRelayClient;

class NATRelayInterface: public RootObject
{
public:
    NATRelayInterface();
    virtual     ~NATRelayInterface();
    
    // Added a registration-use member function in order to abstract between NATTraversalRelayClient and NATRelayInterface.
    virtual void RegisterRelayClient(NATTraversalRelayClient *pClient) = 0;
    virtual void UnregisterRelayClient() = 0;

    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets) = 0;
    virtual void RelayProbeRequest(const qList<StationURL> &lstURLRequestTargets, const StationURL &oURLRequester) = 0;
    virtual void CheckCurrentPublicPort(CallContext *pCallContext, const InetAddress &rInterfaceAddress, qBool bPublicPortIncremental) = 0;
    virtual void DetectSimplifiedNATProperties(CallContext *pCallContext, const InetAddress &rInterfaceAddress) = 0;
    virtual void RequestProbe(const StationURL &oURLStationToProbe) = 0;

    virtual void ReportNATTraversalResult(const qUnsignedInt32 &pid, const qBool &result, const qUnsignedInt32& rtt) = 0;
    virtual void ReportNATProperties(qUnsignedInt32 natmapping, qUnsignedInt32 natfiltering, qUnsignedInt32 rtt) = 0;

    virtual qBool GetRelaySignatureKey(CallContext* pCallcontext, RelaySignatureKey* pRelaySignatureKey) = 0;

    virtual void ReportNATTraversalResultDetail(const qUnsignedInt32 &pid, const qBool &result, const NATTraversalResult &detail, qUnsignedInt32& rtt) = 0;

    // Interface for passing to RVNATRelay and RVEcho.
    virtual void UpdateConnectionState(const ConnectionID cid, const StationURL& oURL) = 0;

#ifdef TEST_ENABLED
    static qBool Test();
#endif
};

END_NAMESPACE_NEX

#endif
