﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef PacketEncDec_H_
#define PacketEncDec_H_

#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>

BEGIN_NAMESPACE_NEX

class Packet;
class EncryptionAlgorithm;

class PacketEncDec : public RootObject
{
    friend class OnlineCore_Transport_Interface_PacketEncDec_Test;
public:
    PacketEncDec();
    ~PacketEncDec();

    void Encode(Packet* pPacket, Stream::Type streamType, qBool needCopy = true);
    void Decode(Packet* pPacket);
    void SetEncryptionKey(const Key &oKey);
    void SetReliableEncryptionKey(const Key &oKey, qByte sessionID, qByte peerSessionID);
    void SetUnreliableEncryptionKey(const Key &oKey);
    Key *GetEncryptionKey();

    void DecideMaxUserReliableSubStreams(qUnsignedInt16 maxUserReliableSubStreams);

    void ReinitUserReliableSubStreams();


private:
    void GenerateKey(const Key &baseKey,
                     const qByte* additionalKey1, const qByte* additionalKey2, Key *targetKey);


    qVector<EncryptionAlgorithm*> m_encoder;
    qVector<EncryptionAlgorithm*> m_decoder;
    EncryptionAlgorithm* m_pUnreliableEncoderDecoder;
    Key m_oKey;
    Key m_oUnreliableEncryptionKey;
    Key m_oUnreliableBroadcastEncryptionKey;
    qBool m_isMultiChunk;

};

END_NAMESPACE_NEX

#endif /* PacketEncDec_H_ */
