/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n2342

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

#ifndef _TransportBuffer_H_
#define _TransportBuffer_H_

#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>

BEGIN_NAMESPACE_NEX

class Buffer;

//Buffer pool for the transport layer
class TransportBuffer: public RootObject
{
    friend void OnlineCore_TransportBuffer_Test();
public:
    TransportBuffer(qUnsignedInt32 queueSize, qUnsignedInt32 bufferSize);
    ~TransportBuffer();

    //Allocate buffer
    Buffer* AllocateBuffer( qUnsignedInt32 uiSize = 0 );
    //Deallocate buffer
    void FreeBuffer( Buffer* pBuffer );

    //n2850 Deplete BufferList and return the number of times packets have been dropped
    qUnsignedInt32 GetBufferListEmptyDropCount()
    {
        return m_uiBufferListEmptyDropCount;
    }

    //n1769 For investigating the Dash issue.
    qUnsignedInt32 GetFreeBufferListSize(void)
    {
        if(m_pFreeBufferList)
        {
            return m_pFreeBufferList->size();
        }
        else
        {
            return 0xffffffff;
        }
    }


#ifdef TEST_ENABLED
    // {secret}
    qBool Test();
#endif

private:
    //Reserve a buffer in advance
    void ReserveBuffer( qUnsignedInt32 uiBufferSize, qUnsignedInt32 uiSize = 0 );

private:
    //Reuse buffers
    //This is static to allow it to be used by multiple sockets
    typedef qReservedQueue<Buffer*> BufferList;
    //List of deallocated buffers
    BufferList* m_pFreeBufferList;
#ifdef SYSTEMCHECK_ENABLED
    // To perform an assert if something not prepared as a pool is returned
    BufferList* m_pFreeBufferListForReleaseCheck;
#endif

    ForcedCriticalSection       m_BufferOperationLock;
    qUnsignedInt32 m_uiBufferListEmptyDropCount;
};

END_NAMESPACE_NEX

#endif      //_TransportBuffer_H_

