/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n2342
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#endif

#if NEX_ENABLE_TRANSPORT_BUFFER_THREAD

//Buffer pool for the transport layer

#ifndef _TransportBufferSet_H_
#define _TransportBufferSet_H_


#include <OnlineCore/src/Platform/Core/MemoryManager.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/Interface/TransportBuffer.h>
#include <OnlineCore/src/Platform/Core/qReservedQueue.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class RootTransport;
class IOCompletionContext;

struct TransportBufferInfo
{
    Buffer*         pBuffer;
    InetAddress     targetAddress;
#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
    Time bufferedTime;
#endif
};

//Data to use for actual communication
class TransportBufferSet
{
private:
    //List
    typedef qReservedQueue<TransportBufferInfo*> TransportBufferInfoQueue;
    volatile qBool m_isListExists;
    volatile qBool m_isQueueFull;

public:
    //Constructor
    // n2343: Will definitely be sufficient if TransportBufferInfo is equal to TransportBufferPoolNum
    TransportBufferSet(TransportBuffer *pTransportBuffer, qInt reserveSize = RootTransport::GetTransportBufferPoolNum() );

    ~TransportBufferSet();

    //Generate
    void Create( qUnsignedInt32 bufferSize, IOCompletionContext* pContext )
    {
        m_uiBufferAllocSize = bufferSize;
        m_pCompletionContext = pContext;
    }

    //Destroy
    void Destroy()
    {
        //Delete all buffers left in list(s) being processed and list(s) that have been deallocated
        ClearList_( m_TransportBufferInfoList );
    }

    //"POP"
    TransportBufferInfo*    PopNewUseInfo();

    qBool IsBufferListExists() const
    {
        // Use volatile and do not lock here
        return m_isListExists;
    }

    qBool QueueIsFull() const
    {
        // Use volatile and do not lock here
        return m_isQueueFull;
    }


    //"PUSH": Data is set externally because it is difficult to create data for using Push
    TransportBufferInfo*    PopNewSetInfo();

    void    FreeOneBuffer( TransportBufferInfo* pInfo );

    IOCompletionContext*    GetContextPtr()
    {
        return m_pCompletionContext;
    }

    CriticalSection&        GetLock()
    {
        return m_Lock;
    }

    // Number of times dropped due to m_TransportBufferInfoPool depletion
    qUnsignedInt32 GetInfoPoolEmptyDropCount() const
    {
        return m_uiInfoPoolEmptyDropCount;
    }

#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
    qBool DequeueExpiredBuffer(Time& time);

    void SetLatencyLimit(qUnsignedInt32 o)
    {
        m_uiLatencyLimit = o;
    }

    qUnsignedInt32 GetLatencyLimit()
    {
        return m_uiLatencyLimit;
    }

    qUnsignedInt32 GetLatencyLimitDequeueCount()
    {
        return m_uiLatencyLimitDequeueCount;
    }
#endif

    //n1769: Prepare to get the number of send and receive buffers for the transport buffer.
    qUnsignedInt32 GetInfoListSize() const
    {
        return m_TransportBufferInfoList.size();
    }

private:
    //Internal processing for data generation
    TransportBufferInfo* CreateInfo_( Buffer* pBuffer );

    //Destroy all lists
    void ClearList_( TransportBufferInfoQueue& list );

    //Release a single information object
    void DeleteOneBuffer_( TransportBufferInfo* pInfo );

private:
    //List being processed
    TransportBufferInfoQueue m_TransportBufferInfoList;
    //Buffer pool
    TransportBufferInfoQueue m_TransportBufferInfoPool;
#ifdef SYSTEMCHECK_ENABLED
    // To perform an assert if something not prepared as a pool is returned
    TransportBufferInfoQueue m_TransportBufferInfoPoolForReleaseCheck;
#endif

    //End check
    IOCompletionContext*                m_pCompletionContext;

    qUnsignedInt32                      m_uiBufferAllocSize;

    //Lock
    mutable ForcedCriticalSection           m_Lock;

    qUnsignedInt32 m_uiInfoPoolEmptyDropCount;

    TransportBuffer *m_pTransportBuffer;

#if NEX_ENABLE_LATENCY_LIMIT_TRANSPORT_BUFFER_SEND
    qUnsignedInt32 m_uiLatencyLimit;
    qUnsignedInt32 m_uiLatencyLimitDequeueCount;
#endif
};


END_NAMESPACE_NEX



#endif // _TransportBuffer_H_

#endif // NEX_ENABLE_TRANSPORT_BUFFER_THREAD

