/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _TransportBufferThread_H_
#define _TransportBufferThread_H_


#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qSortedVector.h>
#include <OnlineCore/src/Platform/Core/TimeProvider.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferThreadInterface.h>

BEGIN_NAMESPACE_NEX

class QueuingSocket;

/* Please see man pages for details 
 
 
 
 
 */
class TransportBufferThread: public TransportBufferThreadInterface
{
public:
    /* Please see man pages for details 
     
     */
    TransportBufferThread()
        : m_bShutdown(false)
        , m_csTransportBufferSocketLock(LockMasks::TransportBufferSocketLock)
        , m_thread(_T("Transport Buffer Thread")
                   , RootTransport::GetTransportBufferThreadPriority())
    {
        //n2967
        if (RootTransport::GetTransportBufferThreadCoreNo() != RootTransport::DEFAULT_TRANSPORT_BUFFER_CORE_NO)
        {
            m_thread.SetCoreNo(RootTransport::GetTransportBufferThreadCoreNo());
        }
    }

    /* Please see man pages for details 
     
     */
    virtual ~TransportBufferThread()
    {
        Stop();
    }

    /* Please see man pages for details 
     
     */
    void Start();

    /* Please see man pages for details 
     
     */
    void Stop();

    /* Please see man pages for details 
     
     
     
     */
    void TransportLoop(void *pDummyParam);

    /* Please see man pages for details 
     
     
     
     
     */
    qBool InsertSocketPort(qUnsignedInt16 port, QueuingSocket *pSocket)
    {
        ScopedCS cs(m_csTransportBufferSocketLock);
        m_mapSocketPort.insert(std::make_pair(port, pSocket));

        return true;
    }

    /* Please see man pages for details 
     
     
     
     */
    void EraseSocketPort(qUnsignedInt16 port)
    {
        ScopedCS cs(m_csTransportBufferSocketLock);
        m_mapSocketPort.erase(port);
    }

    void SetPriority(qUnsignedInt32 priority);
    qUnsignedInt32 GetPriority() const;

private:
    volatile qBool m_bShutdown;  //

    mutable ForcedCriticalSection m_csTransportBufferSocketLock;

    typedef qSortedVector<qUnsignedInt16, QueuingSocket *> SocketPortMap;
    SocketPortMap m_mapSocketPort;  //

    ObjectThread<TransportBufferThread, void *> m_thread;  //
};

END_NAMESPACE_NEX

#endif
