﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _TransportCaps_H_
#define _TransportCaps_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

//Group=Transport Parameters

// Summary: The transport should return true if it is an IP based protocol and
// supports construction of a URL using an IP address and a port number.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_IP_BASED                     0x1

// Summary: The transport should return true if it is capable of doing network
// broadcasts.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_BROADCAST                    0x2

// Summary: The transport should return true if it is implementing device emulation
// using the EmulationDevice class provided in the transport interface.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_DEVICE_EMULATION             0x3

// Summary: The transport should return true if it is implementing performance counting
// using the TransportPerfCounters class provided in the transport interface.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_PERF_COUNTERS                0x4

// Summary: The transport should return true if it is supporting EndPoint grouping.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_GROUPING                     0x5

// Summary: The transport should return true if it supports listening on a well-known
//contact point, /e.g./ a static port number for IP based protocols.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_WELLKNOWN_LISTEN_SUPPORTED       0x6

// Summary: The transport should return true if it supports listening to an
// arbitrary contact point, /e.g./ dynamic port numbers for IP based protocols.
// See Also: Transport::IsCapable
#define TRANSPORT_CAPS_ARBITRARY_LISTEN_SUPPORTED       0x7

#endif
