﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _TransportError_H_
#define _TransportError_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

BEGIN_NAMESPACE_NEX

//group=Error Code Enumerations

// Description: This enumeration defines the error codes that may be returned for the Transport facility.
/* Please see man pages for details 
 
 */
namespace TransportErrorCode
{
//group= Type Definitions

/*
Description: Enumerates the error codes for the Transport facility that a call may return.
Remarks: Details as to the cause of each ErrorCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/
/* Please see man pages for details 
 
 */
enum ErrorCode
{

    // The reason for the error is unknown.
    Unknown                     = 1,    //
    // Network connection was unable to be established.
    ConnectionFailure           = 2,    //
    // The URL contained in the StationURL is invalid. The syntax may be incorrect.
    InvalidUrl                  = 3,    //
    // The key used to authenticate a given station is invalid. The secure transport layer
    // uses secret-key based cryptography to ensure the integrity and confidentiality of data
    // sent across the network.
    InvalidKey                  = 4,    //
    // The specified transport type is invalid.
    InvalidURLType              = 5,    //
    // The Station is already connected via another EndPoint.
    DuplicateEndpoint           = 6,    //
    // The data could not be sent across the network. This could be due to an invalid message, packet, or buffer.
    IOError                     = 7,    //
    // The operation did not complete within the specified timeout for that operation.
    Timeout                     = 8,    //
    // The network connection was reset.
    ConnectionReset             = 9,    //
    // The destination Station did not authenticate itself properly.
    IncorrectRemoteAuthentication = 10, //
    // 3rd-party server or device answered with an error code
    // according to protocol used
    // for example, an HTTP error code
    ServerRequestError          = 11,   //
    // An error occurred while decompressing received data. Make sure that the same compression algorithm is
    // used on both sides.
    DecompressionFailure        = 12,   //
    // It is not possible anymore to send data to a given end point, since it is congested.
    ReliableSendBufferFullFatal = 13,   //
    // Impossible to retrieve necessary information to perform UPnP operation on router.
    UPnPCannotInit              = 14,   //
    // Failed to add port mapping to router through UPnP.
    UPnPCannotAddMapping        = 15,   //
    // Impossible to retrieve necessary information to perform NatPMP operation on router.
    NatPMPCannotInit            = 16,   //
    // Failed to add port mapping to router through NatPMP.
    NatPMPCannotAddMapping      = 17,   //

    // n2781: ???NAT
    UnsupportedNAT          = 19,       //

    // n2222: Failed to resolve DNS.
    DnsError                    = 20,   //
    // n2222: Proxy related error.
    ProxyError                  = 21,   //
    // n2222: The data is remaining to receive. Internal use only.
    DataRemaining               = 22,   //
    // n2222: There is no buffer to receive data.
    NoBuffer                    = 23,   //
    // n2222: The requested resource is not found.
    NotFound                    = 24,   //
    // n2222: Temporary server error occurred. You can retry later.
    TemporaryServerError        = 25,   //
    // n2222: Permanent server error occurred. You can not retry.
    PermanentServerError        = 26,   //
    // n2222: Service unavailable error occurred. You can not retry.
    ServiceUnavailable          = 27,   //
    // It is not possible anymore to send data to a given end point temporary, since it is congested.
    ReliableSendBufferFull      = 28,   //
    // It is not exist station.
    InvalidStation              = 29,   //
    // It is not a valid substream ID
    InvalidSubStreamID          = 30,   //
    // It is not possible anymore to send data to a given end point temporary, since it is congested. out of packetbuffer pool.
    PacketBufferFull            = 31,   //
    //
    NatTraversalError           = 32,   //
    //
    NatCheckError               = 33,   //
    // Last marker
    LastError                           // End
};
//Note: When adding new results, it is necessary to change NEX_QRESULT_TO_STR in OnlineCore/src/Platform/Core/Result.cpp.
//See operations done when adding error codes to the wiki.
//If the errors the application should handle have increased, add them to the list of errors the application should handle in theoverall.rst of the common manual.

}

END_NAMESPACE_NEX


#endif

