﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UPnPProperties_H_
#define _UPnPProperties_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

//group=NAT Traversal Service

/*
Description: This class is a container for UPnP properties.
Remarks: Contains the UPnP properties retrieved during the login process to
$PRODUCTNAME$. Accessing this class before the login will result in UPnP being unsupported.
See Also: RootTransport::EnableUPnPSupport, RootTransport::GetUPnPProperties
*/
class UPnPProperties: public RootObject
{
public:
    //group=Methods

    UPnPProperties();
    virtual ~UPnPProperties();

    //{secret}
    void SetUPnPPortOpen(qBool bUPnPPortOpen = true);

    /*
    Description: Returns whether a port mapping has been added using UPnP.
    Returns: True if a port mapping has been opened; false otherwise.
    */
    qBool IsUPnPPortOpen() const;

    //{secret}
    void SetMappedPort(qUnsignedInt16 ui16MappedPort);

    /*
    Description: Returns the external mapped port in the NAT.
    Returns: The external mapped port in the NAT.
    */
    qUnsignedInt16 GetMappedPort() const;

    // Data members
private:
    qBool m_bUPnPPortOpen;
    qUnsignedInt16 m_ui16MappedPort;
};

END_NAMESPACE_NEX

#endif // #ifndef _UPnPProperties_H_
