/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#ifndef _VirtualSocketDriver_H_
#define _VirtualSocketDriver_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>

#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

//n1589
#define NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB 1

//class VirtualSocketDriver;
//n1589:memo:incomplete type is not allowed
//class VirtualSocketDriver::Socket;

// n2343:
// Jobs and threads are not needed on the VirtualSocketDriver layer.
// It is all right for the actual Driver to perform Send and Recv only when there are Send and Recv requests.
// We can perform Send and Recv on the actual driver.
// As a result, the following code was deleted.
//
// #define NEX_TEST_UDS_RECEIVE_THREAD
// class UdsSocketTransportThreadLauncher;
#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB
class UdsSocketTransportJob;
#endif

class UdsSocketTransportThreadLauncher;

class VirtualNetwork;

struct VirtualMessageIn : public RootObject
{
    VirtualMessageIn():RootObject(), m_oBuffer() {}
    virtual ~VirtualMessageIn() {}

    CHAINABLE(VirtualMessageIn);

    qBuffer m_oBuffer;
};

/* Please see man pages for details 


        
        

        
        
*/
class VirtualSocketDriver: public SocketDriver
{
    class TransportJob;

public:
    class Socket: public SocketDriver::Socket
    {
        friend class VirtualSocketDriver;

    public:
        virtual qBool Open(SocketDriver::TrafficType eTrafficType = SocketDriver::UDP);
        virtual void Close();
        qBool TerminateTransport();

        virtual qBool Bind(qUnsignedInt16 & ui16Port);

        virtual Result RecvFrom(
            qByte* pBuffer,
            qUnsignedInt uiBufferSize,
            SocketDriver::InetAddress* pSourceAddress,
            qUnsignedInt* puiNbBytesRecvd,
            SocketDriver::SocketFlag flags = SocketDriver::NonBlock
        );

        virtual Result SendTo(
            qByte* pBuffer,
            qUnsignedInt uiBufferSize,
            const SocketDriver::InetAddress &refDestinationAddress,
            qUnsignedInt* puiNbBytesSent
        );

        virtual Result Connect(const SocketDriver::InetAddress&)
        {
            return Error;
        }

        virtual Result Recv(qByte*, qUnsignedInt , qUnsignedInt*)
        {
            return Error;
        }
        virtual Result Send(qByte*, qUnsignedInt , qUnsignedInt*)
        {
            return Error;
        }

        virtual qBool SetMulticastAddress(qUnsignedInt32 )
        {
            return false;
        }

        virtual Result Shutdown();

        qBool IsReadyToRead();
        qBool ProcessReceive();
        void DeleteMessageChain();

    private:
        void Lock();
        void Unlock();
        Socket();
        virtual ~Socket();

#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB
        UdsSocketTransportJob* m_pTransportJob;
#endif

        qChain<VirtualMessageIn*> mChainMessageIn;

        mutable CriticalSection mCsReceivedMessage;
    };

    VirtualSocketDriver();
    virtual ~VirtualSocketDriver();

    virtual SocketDriver::Socket* Create();
    virtual void Delete(SocketDriver::Socket* pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);

    /* Please see man pages for details 
    

            
            

            

            
            

    

    
    */
    static qBool RegisterVirtualNetwork(VirtualNetwork* pVirtualNetwork);

    /* Please see man pages for details 
    

            
            

            
            

    
    */
    static qBool UnregisterVirtualNetwork();

    static VirtualNetwork* GetVirtualNetwork();

private:
    static VirtualNetwork* s_pVirtualNetwork;
};

#if NEX_ENABLE_UDS_SOCKET_TRANSPORT_JOB

class UdsSocketTransportJob: public Job
{
public:
    UdsSocketTransportJob(
        const String& strJobDescription
    );

    virtual ~UdsSocketTransportJob();

    void Launch(void* pDummyParam);
    void Finish();

    virtual void Execute();

    qBool IsRunning()
    {
        return mb_Running;
    }

private:
    VirtualSocketDriver::Socket* socket;
    qBool mb_Running;
};

#endif

END_NAMESPACE_NEX

#endif
