﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef _PRUDPMessageInterface_H_
#define _PRUDPMessageInterface_H_

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>

BEGIN_NAMESPACE_NEX

class Packet;
class PacketIn;
class PacketOut;
class Key;
class ByteStream;
class TransportSignatureGenerator;
class SignatureBytes;

/* Please see man pages for details 
 
 
 
 */
class PRUDPMessageInterface
{
public:
    /* Please see man pages for details 
     
     
     
     */


    enum SignatureMethod
    {
        SignatureNone,
        SignatureAddress,
        SignatureData,
        SignatureDataWithHeader,
        SignatureAllPRUDPOld,
        SignatureAllPRUDP,
        SignatureAllPRUDPWithKey,
        SignatureHeaderOnly,
        SignatureRandom,
        SignatureIgnored
    };

    /* Please see man pages for details 
     
     
     
     */
    enum SignatureMatchPattern
    {
        SignatureContinueProcessing,
        SignatureMatch,
        SignatureUnmatch,
        SignatureDrop
    };

    //If static const within a pure virtual class in ARMCC, revise enum because a symbol error will occur upon linking
    enum
    {
        MAX_CALC_SIGNATURE_SIZE = 16, /* Please see man pages for details */
        MIN_HEADER_SIZE = 10, /* Please see man pages for details */
        HEADER_MAGIC_NUMBER1 = 0xEA, /* Please see man pages for details */
        HEADER_MAGIC_NUMBER2 = 0xD0, /* Please see man pages for details */
        RELAY_HEADER_MAGIC_NUMBER = 0xF5, /* Please see man pages for details */
        RELAY_VERSION_BASE_NUMBER = 0xD0 /* Please see man pages for details */
    };


    virtual ~PRUDPMessageInterface() {} ;


    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     */
    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature) = 0;

    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature)= 0;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint) = 0;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *pPacket, SignatureMethod method, const Key *pKey,
            TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &expectedSignature) = 0;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn,qUnsignedInt *pSizeUnpacked = NULL) = 0;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt *pPayloadSize = NULL) = 0;


    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream) = 0;

};
END_NAMESPACE_NEX
#endif // End of _PRUDPMessageInterface_H_
