﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// n2370
//#ifndef _PacketOut_H
#ifndef _PacketOut_H_
#define _PacketOut_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Core/ByteStream.h>
#include <OnlineCore/src/Core/FixedSizePooledObject.h>

BEGIN_NAMESPACE_NEX

class PRUDPEndPoint;
class Timeout;

class PacketOut: public Packet
{
    DEFINE_FIXED_SIZE_POOLED_OBJECT(PacketOut, 64);

public:
    //n2967: We have to manage the state because if there are no packets for the same recipient then it will not be sent via bundling, and so we will re-send it separately.
    enum PacketHistoryBundlingState
    {
        UnbundledAndUnretransmit, //Not bundled or re-sent. Initial state Next state is Bundled or Retransmit
        Bundled, //It was bundled. Next state is OnceBundledOrRetransmited
        Retransmit, //Not bundled, so we re-send. Next state is OnceBundledOrRetransmited
        OnceBundledOrRetransmited //It was bundled or re-sent at least once.
    } ;


    PacketOut(
        PRUDPEndPoint *pEndPoint,
        PacketType byPacketType,
        qUnsignedInt16 uiFlags = FLAG_NONE,
        Buffer *pBuffer = NULL,
        qUnsignedInt32 uiCallID = 0
    );
    
    virtual ~PacketOut();
protected:
    // For PacketBufferPacketOut.
    PacketOut() {} // Does nothing. Used in a derivative class.
    void Initialize(PRUDPEndPoint *pEndPoint,
                    PacketType byPacketType,
                    qUnsignedInt16 uiFlags,
                    Buffer *pBuffer,
                    qUnsignedInt32 uiCallID,
                    Timeout* pTimeout);
public:

    Timeout         *GetTimeout()
    {
        return m_pTimeout;
    }

    PRUDPEndPoint   *GetEndPoint()
    {
        return m_pEndPoint;
    }

    void            IncNbTransmission()
    {
        ++m_ui16NbTransmission;
    }
    qUnsignedInt16  GetNbTransmission() const
    {
        return m_ui16NbTransmission;
    }

    void            SetPackedSize(qUnsignedInt32 size)
    {
        m_packedSize = size;
    }
    qUnsignedInt32  GetPackedSize() const
    {
        return m_packedSize;
    }

public:
    inline qBool    IsBundlable() const
    {
        return (GetType() != TYPE_RAW && GetType() != TYPE_DISCONNECT) &&
               (Stream::GetSettings(GetSourceVirtualPort().GetStreamType()).GetBundling().IsEnabled());
    }

    inline qBool    IsBundlableWith(PacketOut *pPacket)
    {
        return pPacket->IsBundlable() && IsBundlableWithPortTypeCheck(pPacket);
    }

    inline qBool    ShouldTestHistoryBundle() const
    {
        // Attempt to bundle history when sending this packet?
        return RootTransport::IsEnablePacketHistoryBundling()
               && CanHistoryBundle() && !IsAcked();
    }


    inline qBool    ShouldDuplicateForHistoryBundle() const
    {
        // Whether to duplicate if a history bundle is present when this packet is sent.
        return RootTransport::IsEnablePacketHistoryBundling()
               && ShouldDuplicateIfHistoryBundle() && !IsAcked();
    }


    inline qBool    IsHistoryBundlableWith(PacketOut *pPacket)
    {
        // Whether the history can bundled with this packet when sending pPacket.
        return pPacket->ShouldTestHistoryBundle() && IsBundlableWithPortTypeCheck(pPacket);
    }
    void            SetExpress(qBool bExpress)
    {
        m_bExpress = bExpress;
    }
    qBool           IsExpress()
    {
        return m_bExpress;
    }

    qBool           ReadyToSend();
    /* Please see man pages for details 
    
    
    */
    qBool           SetInternalPayload(Buffer * pBuffer);
#ifdef TRACE_ENABLED    // n2370
    virtual void Trace(qUnsignedInt64 uiTraceFlags = D_TRACE_TRANSPORT_PACKET)
    {
        TraceImpl(uiTraceFlags, false);
    }
#endif  // TRACE_ENABLED

    inline void SetPacketHistoryBundlingState(PacketHistoryBundlingState s)
    {
        m_packetHistoryBundlingState = s;
    }

    inline PacketHistoryBundlingState GetPacketHistoryBundlingState()
    {
        return m_packetHistoryBundlingState;
    }

    inline void ResetPacketHistoryForceRetransmitTimeout()
    {
        m_packetHistoryForceRetransmitTimeout = Time::GetTime()
                                                + RootTransport::GetPacketHistoryBundlingForcedRetransmitTimeout();
    }


    inline Time GetPacketHistoryForceRetransmitTimeout()
    {
        return m_packetHistoryForceRetransmitTimeout;
    }

    inline qBool IsAcked() const
    {
        return m_isAcked;
    }

    inline void SetAcked()
    {
        m_isAcked = true;
    }

    inline void IncPacketHistoryRetransmitCount()
    {
        m_packetHistoryRetransmitCount++;
    }

    qUnsignedInt16 GetPacketHistoryRetransmitCount() const
    {
        return m_packetHistoryRetransmitCount;
    }

    inline qUnsignedInt32 GetCallID() const
    {
        return m_callID;
    }
#ifdef NEX_CLIENT_PBPOOL
    /* 
     
            
     */
    qBool IsValid()
    {
        if (FlagSet(FLAG_ACK_WANTED) && GetTimeout()==NULL)
        {
            QTRACE(D_TRACE_MEMORY, _T("PacketOut::IsValid() false"));
            // Timeout to be set is not ready even though ACK wait is required. (PacketBufferTimeout is not ready due to memory depletion.)
            return false;
        }

        // Does not check because null can be possible for GetPayload.

        return true;
    }
#endif // NEX_CLIENT_PBPOOL

private:
    qBool           IsBundlableWithPortTypeCheck(PacketOut *pPacket);

    Timeout             *m_pTimeout;
    PRUDPEndPoint       *m_pEndPoint;
    qUnsignedInt16      m_ui16NbTransmission;
    qUnsignedInt16      m_packetHistoryRetransmitCount; //n2967
    Time                m_oReadyToQueueTime;
    qBool               m_bExpress;
    qBool               m_isAcked; //n2967
    qUnsignedInt32      m_packedSize ; //n2967
    PacketHistoryBundlingState m_packetHistoryBundlingState; //n2967
    Time                m_packetHistoryForceRetransmitTimeout; //n2967
    qUnsignedInt32      m_callID;
};

END_NAMESPACE_NEX

#endif
