/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _QueuingSocket_H_
#define _QueuingSocket_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferSet.h>
#include <OnlineCore/src/Transport/UDP/PacketQueue.h>
#include <OnlineCore/src/Transport/UDP/PacketHistoryChainDefine.h>
#include <OnlineCore/src/Transport/UDP/NetworkEmulator.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/Socket.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionContext.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionNotifier.h>

BEGIN_NAMESPACE_NEX

class PacketIn;
class PacketOut;
class RootTransport;
class ByteStream;
class PRUDPMessageSelector;

class QueuingSocket: public Socket
{
public:
    QueuingSocket(IOCompletionNotifier *pNotifier, qUnsignedInt uiTrafficType, RootTransport *pTransport, qUnsignedInt32 uiMtu, TransportBuffer *pTransportBuffer);
    virtual ~QueuingSocket();

    qBool Bind(InetAddress *pBindAddress, qUnsignedInt16 *ui16Port, qBool bReuse = false);
    qBool Send(Buffer *pBuffer, InetAddress *pAddress);
    qBool Queue(Buffer *pBuffer, InetAddress *pAddress);
    qBool Flush();
    Buffer *CreateBufferFromPacketQueue(PacketQueue *pPacketQueue, PacketHistoryChain *pHistoryPacketQueue, qUnsignedInt32 uiMTU );
    static void RetransmitFromHistoryPacketQueue(PacketQueue *pPacketQueue, PacketHistoryChain *pHistoryPacketChain, qUnsignedInt16 localPort);
    qUnsignedInt32 CompleteSend();
    virtual void Recv(qUnsignedInt32 uiSize);
    virtual qBool IsRecvDataExists() const;

    qUnsignedInt FillPacketQueueFromBuffer(Buffer *pBuffer, const InetAddress *pSourceAddress, PacketQueue *pQueue, TransportStreamManager *pTransportStreamMnager, qBool* pIsNoBuffer);
    Buffer *CompleteBufferRecv();

    qUnsignedInt32 QueueSize() const;
    qBool InputFlooded();
    qBool OutputFlooded();

    const InetAddress &GetSourceAddress() const
    {
        return m_oSourceAddress;
    }
    void InputQueue(Buffer *pBuffer)
    {
        m_oInputEmulator.Queue(pBuffer, m_oSourceAddress);
    }
    void Dequeue(Buffer *pBuffer)
    {
        m_oInputEmulator.Dequeue(pBuffer);
    }
    qBool GetHead( Buffer **ppBuffer, InetAddress *pAddr )
    {
        return m_oInputEmulator.GetHead(ppBuffer,pAddr);
    }
    qUnsignedInt32 GetRefCount()
    {
        return m_uiRefCount;
    }
    void IncreaseRefCount()
    {
        m_uiRefCount++;
    }
    void DecreaseRefCount()
    {
        m_uiRefCount--;
    }

    qBool SendPending()
    {
        return m_bSendPending;
    }
    qBool RecvPending()
    {
        return m_bRecvPending;
    }

    qBool RecvToComplete()
    {
        return m_pRecvCompletionContext->ReadyForCompletion();
    }

    qBool SendToComplete()
    {
        return m_pSendCompletionContext->ReadyForCompletion();
    }

    /* Please see man pages for details 
     
            
            
     */
    void SetUsePacketBufferPool(qBool flag)
    {
        m_usePacketBufferPool = flag;
    }
    qBool GetUsePacketBufferPool() const
    {
        return m_usePacketBufferPool;
    }

#if NEX_ENABLE_SOCKET_TRANSPORT_SWITCH
    virtual void SetEnableTransport(qBool enable);
#endif

#ifdef TEST_ENABLED
    static qBool Test();
    static qBool PacketHistoryBundlingTest();
#endif

private:
    void CreateCompletionContext(IOCompletionContext **ppContext);
    void ClearCompletionContext(IOCompletionContext *pContext);

protected:
    IOCompletionContext *m_pRecvCompletionContext;
    IOCompletionContext *m_pSendCompletionContext;
    IOCompletionNotifier *m_pIOCompletionNotifier;

    qBool m_bSendPending;
    qBool m_bRecvPending;

    InetAddress m_oSourceAddress;
    qUnsignedInt32 m_uiRefCount;
    NetworkEmulator m_oOutputEmulator;
    NetworkEmulator m_oInputEmulator;

    qUnsignedInt m_iTranasportBufferReceiveCount;

    PRUDPMessageSelector* m_pPRUDPMessageSelector;

    PacketQueue m_sendingPacketQueue; //n2967: Allocate temporary region to member, because WaterMark in PacketQueue is thread unsafe.

    TransportBuffer *m_pTransportBuffer;

    qBool m_usePacketBufferPool; // Is the receive packet allocated in the packet buffer pool?

    virtual qBool SendBuffer(Buffer *pBuffer, InetAddress *pAddress);
    PacketIn* ExtractIncomingPacket(TransportStreamManager *pTransportStreamManager,
                                    const InetAddress *pSourceAddress,
                                    Buffer *pBuffer,
                                    qBool* pIsNoBuffer);
};

//n2967:For bundling past history.
void PacketHistoryChainPurge(PacketHistoryChain* chain);
PacketHistoryChain::iterator PacketHistoryChainDequeue(
    PacketHistoryChain *chain, PacketHistoryChain::iterator it);
void PacketHistoryChainQueueFront(PacketHistoryChain *chain, PacketOut *pPacket);


END_NAMESPACE_NEX

#endif
