﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BackEndServices_H_
#define _BackEndServices_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Core/Holder.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <RendezVous/Core/src/Common/ProtocolCallContext.h>
#include <RendezVous/Core/src/Common/DataHolder.h>
#include <RendezVous/Core/src/Common/NotificationEventHandler.h>
#include <RendezVous/Core/src/Common/NintendoProtocolFoundation/NintendoNotificationEventHandler.h>
#include <RendezVous/Core/src/Common/LoginDataDDL.h>
#include <RendezVous/Core/src/Client/SecureConnection/SecureConnectionClient.h>

/* Please see man pages for details */
BEGIN_NAMESPACE_NEX

class RemoteLogDeviceServer;
class SecureStream;
class Credentials;
class AuthenticationClient;
class SecureConnectionClient;
class StreamManager;
class NotificationEventManager;
class NintendoNotificationEventManager;
class RendezVousOperation;
class ProtocolRegistry;


typedef CustomDataHolder<LoginData> LoginDataHolder;

/* Please see man pages for details 
 
 */
class OSDK_API BackEndServices: public RootObject
{
public:

    /* Please see man pages for details */
    virtual ~BackEndServices();

    /*
    \brief Returns whether or not a given Principal is connected to both the
    lobby's Authentication Service and Secure Connection Service.
    \returns true if the given Principal is connected to both the lobby's Authentication Service and
    Secure Connection Service, i.e., the Principal is logged in to the lobby, false if not or an error
    occurred.
    \remarks When a Principal logs in to the lobby using the platform specific back-end services Login()
    method, it automatically connects to both the Authentication Service and the Secure Connection
    Service in order to create a secure transport layer that guarantees the integrity, confidentiality,
    and authenticity of data sent across the network. The protocol used is PRUDPS, Quazal's proprietary,
    reliable, secure UDP transport protocol. After a Principal logs out from the lobby, this method will return false.

    By default each BackEndServices client sends a heart beat signal to the Rendez-Vous servers every 10 seconds.
    If the servers do not receive a signal for a period of one minute, then it is assumed that the client is no longer
    connected. If you suspect that a fault has occurred on the client, you can use HealthClient::PingDaemon. If the
    client is connected, the call will succeed rapidly. To test for a fault, you can issue the call and wait on the
    CallContext for 5 seconds. If it successful within that time, you know that no fault has occurred. If it has failed
    or it still pending, you can assume that a fault has occurred.
    \sa Login, Logout, HealthClient::PingDaemon
    */
    /* Please see man pages for details 
     

     
     
     

     

     
     
     

    
    
    */
    qResult IsConnected() const;

    /*
    \brief Disconnects a Principal from both the lobby's Authentication Service and
    Secure Connection Service.

    \param pContext A pointer to the CallContext for this call.
    \returns true if the Principal was successfully disconnected from both the lobby's Authentication Service
    and Secure Connection Service, false if not or an error occurred.
    \remarks This method should be called after Logout to correctly disconnect the Principal from both the
    Authentication Service and the Secure Connection Service. Refer to Logout for an example of this
    method in use.

    \note This method is now obsolete. To log out from the lobby you simply need to call Logout.
    \sa IsConnected, Logout
    */
    qBool Disconnect(CallContext *pContext);

    /* Please see man pages for details */
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword, const qChar* szAddress,
                 qUnsignedInt16 uiBasePort, Credentials** ppCredentials, String *pstrReturnMsg=NULL)
    {
        return LoginImpl(pContext, NULL, szUserName, szPassword, szAddress, uiBasePort, ppCredentials,
                         pstrReturnMsg, NULL);
    }

    /* Please see man pages for details */
    qBool Login_(CallContext *pContext, const qChar* szUserName, const qChar8* szPassword,
                 const qChar* szAddress, qUnsignedInt16 uiBasePort, const DataHolder &oAnyData,
                 Credentials** ppCredentials, String *pstrReturnMsg)
    {

        return LoginImpl(pContext, NULL, szUserName, szPassword, szAddress, uiBasePort, ppCredentials,
                         pstrReturnMsg, (DataHolder *)&oAnyData);
    }

    /* Please see man pages for details */
    qBool Logout_(Credentials *pCredentials);

    /* Please see man pages for details */
    qBool Logout_(CallContext *pContext, Credentials *pCredentials);

    /* Please see man pages for details */
    qBool LogoutImpl(CallContext *pContext, Credentials *pCredentials);

    /*
    \brief Terminates and cleans up outstanding Login/Logout calls, ensuring that the object is in an appropriate
    state for deletion.

    \param pContext A pointer to the CallContext for this call.
    \remarks To ensure that you can delete the appropriate back end services object even if there are incomplete login or
    logout calls. To prevent problems, you should always call this method before deleting the object.
    An example of the Terminate() method is:
    \code
            ProtocolCallContext oContext;
            pBackEndServices->Terminate(&oContext);
            oContext.Wait();
            delete pBackEndServices;
    \endcode
    */
    /* Please see man pages for details 
     

     
     

     
     

     
     

     
     

     

     
     
     */
    qBool  Terminate(CallContext* pContext);

    /* Please see man pages for details */
    /* Please see man pages for details 
     

     
     

     
     */
    Credentials* GetCredentials()
    {
        return m_pCurrentCredentials;
    }

    /* Please see man pages for details 
     

     
     

     

     
     */
    static void EnableLoginDataCache();

    /* Please see man pages for details 
     

     
     */
    static void DisableLoginDataCache();

    /* Please see man pages for details 
     

     
     
     
     */
    static qBool IsAbleLoginDataCache();

protected:
    /* Please see man pages for details */
    typedef  void (*PreTerminateCallback)(CallContext*, const UserContext*);

    /* Please see man pages for details 
    
    
    
    
    
    */
    void RegisterPreTerminateCallback(PreTerminateCallback pfPreTerminateCallback, const UserContext& oContext);

    /* Please see man pages for details */
    qBool LoginWithData(CallContext *pContext, const qChar8* szPassword, const qChar* szAddress, qUnsignedInt16 uiBasePort,
                        const LoginDataHolder& loginData, const DataHolder& secureData, Credentials** ppCredentials)
    {
        return LoginImplWithData(pContext, NULL, szPassword, szAddress, uiBasePort, loginData, secureData, ppCredentials);
    }

private:

    struct OSDK_API InternalData
    {
        PreTerminateCallback m_Callback;
        UserContext        m_oUserContext;
        qBool              m_bPreTerminateCallbackRegistered;
        InternalData():m_bPreTerminateCallbackRegistered(false) {};
        ~InternalData()
        {
            m_bPreTerminateCallbackRegistered = false;
        }

        void Set(const PreTerminateCallback &cb, const UserContext &c);
    };
    InternalData  m_PreTerminateCB;
    static void  TerminateCB(CallContext* pContext, const UserContext* pUserContext);
    CallContext m_oContext;
    CallContext* m_pUserCallContext;

    qBool  TerminateImpl();

    /* Please see man pages for details 
    */
    void ReleaseCredentials();

public:
    /* Please see man pages for details 
    
    
    */
    virtual void PostLogoutCleanup();

    /* Please see man pages for details */
    StreamManager *GetStreamManager() const;

    /* Please see man pages for details */
    SecureStream *GetSecureStream() const;

    // TODO: API to select no incoming connections, insecure
    // incoming connections or secure incoming connections.

    /*
    \brief Returns a pointer to the AuthenticationClient of the Principal who has logged in.
    \returns A pointer to the AuthenticationClient of the Principal who has logged in.
    \remarks This call is only valid once a Principal has logged in to the lobby via the BackEndServices
    Login method.
    \sa Authentication Service, Login
    */
    AuthenticationClient* GetAuthenticationClient() const;

    /*
    \brief Returns a pointer to the SecureConnectionClient of the Principal who has logged in.
    \returns A pointer to the SecureConnectionClient of the Principal who has logged in.
    \remarks This call is only valid once a Principal has logged in to the lobby via the BackEndServices
    Login method.
    \sa Secure Connection Service, Login
    */
    SecureConnectionClient* GetSecureConnectionClient() const;

    /* Please see man pages for details */
    void SetConnectionInfo(qUnsignedInt16 uiPort, Stream::ID idStream, qUnsignedInt32 uiConnectionFlags)
    {
        m_uiPort = uiPort;
        m_idStream = idStream;
        m_uiConnectionFlags=uiConnectionFlags;
    }

    //{secret}
    qUnsignedInt16 GetLocalPort() const
    {
        return m_uiPort;
    }

    //{secret}
    Stream::ID GetStreamID() const
    {
        return m_idStream;
    }

    //{secret}
    qUnsignedInt32 GetConnectionFlags() const
    {
        return m_uiConnectionFlags;
    }

    /*
    \brief The Process will accept (log) remote service traces.
    \remarks In order to remotely debug a service you can log the service traces via a remote Process,
    such as a client Process. In order to do so, in your /UserConfiguration.py/ script you must first configure which
    traces will be output by the back-end services via the TraceLog.SetFlag method of the quazal module. If in your
    /UserConfiguration.py/ script log are enabled, then when AcceptRemoteTraces is called on a remote process, such as a client,
    all traces and service events that were configured to be logged and that result from the remote Process's actions, will
    be logged on that remote Process. In order to stop a remote Process logging service traces, call IgnoreRemoteTraces.

    This method should be called immediately after the BackEndServices object is created.
    The LogDevice where the traces will be logged to must be specified on the remote Process. Refer to the
    Data Logging section for further details.
    \sa IgnoreRemoteTraces, Data Logging, SandboxConfiguration
    */
    void AcceptRemoteTraces();

    /*
    \brief The Process will ignore all traces sent by a remote service.
    \remarks If a back-end service is configured to remotely log traces and the remote Process calls
    IgnoreRemoteTraces, any traces that the Process receives will be disregarded. In order for a remote
    Process to log service traces AcceptRemoteTraces must be called.
    \sa AcceptRemoteTraces
    */
    void IgnoreRemoteTraces();

#ifdef LOG_ENABLED
    /* Please see man pages for details */
    RemoteLogDeviceServer* GetRemoteLogDeviceServer()
    {
        return m_pRemoteLogDeviceServer;
    }
#endif

    /*
    \brief Registers a specific NotificationEventHandler to the back-end services.

    \param pNotificationEventHandler A pointer to the NotificationEventHandler that will be registered to the
    back-end services.

    \returns true if the specified NotificationEventHandler was successfully registered to the back-end
    services, false if not or an error occurred.

    \remarks This method should be called after Login is called and before the required
    ServiceClient objects are created.  Refer to the Integration page for further details as to the
    correct order of calls. A NotificationEventHandler determines what should happen when a Process
    receives a given NotificationEvent. If no handler is registered then the Process will not
    receive any NotificationEvents. Once a NotificationEventHandler is registered, the Process
    associated to this BackEndServices object will receive all built-in NotificationEvents as well
    as custom NotificationEvents (as long as custom events are not implemented otherwise) until
    UnregisterNotificationEventHandler is called. Several NotificationEventHandlers may be
    registered to the back-end services at any given time.

    \sa UnregisterNotificationEventHandler, NotificationEventHandler, Integration
    */
    /* Please see man pages for details 
     

     
     

     
     
     
     */
    qBool RegisterNotificationEventHandler(NotificationEventHandler* pNotificationEventHandler);
    qBool RegisterNintendoNotificationEventHandler(NintendoNotificationEventHandler* pNintendoNotificationEventHandler);

    /*
    \brief Unregisters a specific NotificationEventHandler from the back-end services.

    \param pNotificationEventHandler A pointer to the NotificationEventHandler that will be unregistered
                            from the back-end services.

    \returns true if the specified NotificationEventHandler was successfully unregistered from the
          back-end services, false if not or an error occurred.

    \remarks This method should be called after Logout is called. Refer to the Integration page for
    further details as to the correct order of calls. When there is no NotificationEventHandler
    registered to the BackEndServices, a Process will not receive any NotificationEvents.

    \sa RegisterNotificationEventHandler, NotificationEventHandler, Integration
    */
    /* Please see man pages for details 
     

     
     
     
    */
    qBool UnregisterNotificationEventHandler(NotificationEventHandler* pNotificationEventHandler);
    qBool UnregisterNintendoNotificationEventHandler(NintendoNotificationEventHandler* pNintendoNotificationEventHandler);

    /* Please see man pages for details */
    StreamManager* CreateStreamManager() const;
    /* Please see man pages for details */
    void InitStreamManagerFactory(StreamManager* (*pfStreamManagerFactory)());

    /*
    \brief Sets the access key of the sandbox to which this BackEndServices client wishes to connect.

    \param strSandboxAccessKey The access key of the sandbox to which this BackEndServices client wishes to connect.
    \remarks Quazal assigns a unique access key to each sandbox on the Rendez-Vous servers. In order
    to connect to a given sandbox, the correct access key for that sandbox must be passed via this method
    before Login is called.

    Refer to the BackEndServices class overview for an example of this method in use.
    */
    static void SetSandboxAccessKey(const String& strSandboxAccessKey);

    /*
    \brief Sets the timeout for the Login call.

    \param tiTimeout The maximum time in milliseconds that the system will wait. The default timeout is 30,000 milliseconds (30 s).
    \remarks This method sets how long a client will wait for the Login call to succeed. It must be called before the
    call to Login is made.
    \sa Login
    */
    void SetLoginTimeout(const TimeInterval& tiTimeout)
    {
        m_tiLoginTimeout = tiTimeout;
    }

    /*
    \brief Enable or Disable advertising of Public Url returned at login to Network.

    \param[in] enable The flag to enable or disable advertising of URL to Network
    \remarks This method is only used by the JobLogin. It must be called before the call to Login is made.
    \sa Login
    */
    void SetAdvertisePublicUrlFlag(qBool enable)
    {
        m_advertisePublicURL = enable;
    }
    qBool GetAdvertisePublicUrlFlag() const
    {
        return m_advertisePublicURL;
    }

    /* Please see man pages for details */
    void SetPublicUrl(const StationURL& publicUrl)
    {
        m_urlPublic = publicUrl;
    }

    /*
    \brief Returns the Public Url returned at login from RendezVous.
    \remarks Public Url is only valid after a successful login is made.
    \sa Login
    */
    StationURL GetPublicUrl() const
    {
        return m_urlPublic;
    }

    /*
    \brief Returns a pointer to an OperationManager.
    \returns A pointer to an OperationManager.

    \remarks This method returns a pointer to the OperationManager
    that manages all RendezVousOperations. The main purpose of
    the OperationManager is to manage the callbacks called at the
    beginning and end of the various RendezVous System Operations.
    Before using it, be sure to have at least one BackEndServices
    instance created.

    \sa OperationManager
    */
    //static OperationManager* GetOperationManager();

    /* Please see man pages for details */
    qBool LoginJobIsInProgress() const;

    /* Please see man pages for details */
    qBool LogoutJobIsInProgress() const;

    /* Please see man pages for details */
    qBool TerminateJobIsInProgress() const;

    /*
    \brief Returns a user-friendly error string given an error code returned by qResult,

    \param strErrorCause    The user-friendly error string. Not all codes are translated yet
    \param uiReturnCode     Provides the value returned by qResult::GetReturnCode.
    \remarks Returns a user-friendly error string when given a return code from qResult
    */
    static String FormatQErrorCodeString(const String& strErrorCause, qUnsignedInt32 uiReturnCode);

    /* Please see man pages for details */
    static void SetProtocolVersion(qUnsignedInt32 uiProtocolVersion);

    /* Please see man pages for details */
    static void SetCustomData(const DataHolder &hCustomData)
    {
        m_hCustomData = hCustomData;
    }
    /* Please see man pages for details */
    static const DataHolder& GetCustomData()
    {
        return m_hCustomData;
    }

    /*
    \brief This method compares the parameter that is passed against the current password.
    \param szPassword: Accepts a string that is compared against the current password. Returns true if
    the strings match, otherwise it returns false.
    */
    qBool ValidatePassword(const qChar8* szPassword);

    /* Please see man pages for details */
    TimeInterval    GetLoginTimeout() const
    {
        return m_tiLoginTimeout;
    }

    /* Please see man pages for details */
    void SetEffectiveLoginTimeout(TimeInterval tiTimeout);

    /* Please see man pages for details */
    qBool EffectiveTimeoutIsSet() const;

    /* Please see man pages for details */
    void SetWellKnownPortNumber(qUnsignedInt16 uiPort)
    {
        m_uiPort = uiPort;
    };

    /*
    \brief This method enables traces and specifies the trace flags for
    NotificationsEvents that are received from the server.

    \param[in] uiTraceFlags The trace flags to use for the NotificationsEvents traces.
    \note By default the trace flag is TRACE_NEVER.
    \sa NotificationEventHandler
    */
    void SetNotificationEventTraceFlags(qUnsignedInt64 uiTraceFlags);

    /*
    \brief This method returns the NotificationsEvents trace flags.
    \returns The NotificationsEvents trace flags
    \sa NotificationEventHandler
    */
    //n1589:qUnsignedInt64
    qUnsignedInt64 GetNotificationEventTraceFlags();
    qUnsignedInt64 GetNintendoNotificationEventTraceFlags();

    /*
    \brief This method returns the global ProtocolRegistry which can be used to register client protocols.
                 The Registry is used to parse Responses on a specific client protocol.
    \returns A pointer to the global ProtocolRegistry
    \sa ProtocolRegistry
    */
    ProtocolRegistry* GetProtocolRegistry() const;

    /* Please see man pages for details 
    
    */
    virtual void RegisterServerProtocols();
    /* Please see man pages for details */
    virtual void UnregisterServerProtocols();
    /* Please see man pages for details */
    void UpdateRemoteLogDeviceServer();

private:
    /* Please see man pages for details */
    qBool LoginImpl(CallContext *pContext, qResult *pqResult, const qChar* szUserName, const qChar8* szPassword,
                    const qChar* szAddress, qUnsignedInt16 uiBasePort, Credentials** ppCredentials,
                    String *pstrReturnMsg, DataHolder *pAnyData);
    /* Please see man pages for details */
    qBool LoginImplWithData(CallContext *pContext, qResult *pqResult, const qChar8* szPassword, const qChar* szAddress, qUnsignedInt16 uiBasePort,
                            const LoginDataHolder& loginData, const DataHolder& secureData, Credentials** ppCredentials);

private:
    /* Please see man pages for details */
    BackEndServices(const BackEndServices&);
    BackEndServices& operator=(const BackEndServices&);

protected:

    /* Please see man pages for details */
    BackEndServices();

    /* Please see man pages for details 
    */
    void OnLoginStart(CallContext& context);

    /* Please see man pages for details 
    */
    void OnLogoutStart(CallContext& context);

    /* Please see man pages for details */
    qUnsignedInt16 GetWellKnownPortNumber() const;

    /* Please see man pages for details */
    Job* GetJobLogin() const
    {
        return m_pJobLogin;
    }

private:

    void RegisterOperationOnCallContext(CallContext& context, RendezVousOperation& op);

    static String GetResultFacilityCore(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityTransport(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityRendezVous(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityPythonCore(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilityNews(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilityGameSession(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityZZZAuthentication(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityPrivilege(const qUnsignedInt32 uiErrorCode);
    //   static String GetResultFacilityTracking(const qUnsignedInt32 uiErrorCode);
    static String GetResultFacilityFriends(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilityTitleStorage(const qUnsignedInt32 uiErrorCode);
    // static String GetResultFacilitySocialNetworks(const qUnsignedInt32 uiErrorCode);
    //  static String GetResultFacilitySubAccountManagement(const qUnsignedInt32 uiErrorCode);
    //static String GetResultFacilityZZZFriends(const qUnsignedInt32 uiErrorCode);

    friend class JobBackEndServicesTerminate;   //JobBackEndServicesTerminate calls CancelLoginJob
    void CancelLoginJob();

    TimeInterval    m_tiLoginTimeout;
    Credentials*    m_pCurrentCredentials;

    qUnsignedInt16  m_uiPort;
    Stream::ID      m_idStream;
    qUnsignedInt32  m_uiConnectionFlags;

#ifdef LOG_ENABLED
    RemoteLogDeviceServer*      m_pRemoteLogDeviceServer;
#endif
    NotificationEventManager*   m_pNotificationEventManager;
    NintendoNotificationEventManager*   m_pNintendoNotificationEventManager;

    qBool           m_bAcceptRemoteTraces;

    StreamManager* (*m_pfStreamManagerFactory)();

    static DataHolder m_hCustomData;

    Job*    m_pJobLogin;
    Job*    m_pJobLogout;
    Job*    m_pJobTerminate;

    //  See bug:4123
    class OSDK_API EffectiveLoginTimeout
    {
    public:
        EffectiveLoginTimeout();

        void Set(TimeInterval tiTimeout);
        TimeInterval Get() const;

        qBool IsSet() const;

    private:
        TimeInterval    m_tiTimeout;
    };
    EffectiveLoginTimeout   m_oEffectiveLoginTimeout;

    //static PseudoGlobalVariable<OperationManager> s_operationManager;
    RendezVousOperation* m_loginOperation;
    RendezVousOperation* m_logoutOperation;

    qBool m_advertisePublicURL;
    StationURL m_urlPublic;


    static qBool s_IsAbleLoginDataCache;
};

/* Please see man pages for details */
END_NAMESPACE_NEX

#endif
