﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SecureConnectionClient_H
#define _SecureConnectionClient_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/SecureConnectionProtocolDDL.h>
#include <RendezVous/Core/src/Common/ConnectionDataDDL.h>
#include <RendezVous/Core/src/Common/DataHolder.h>

BEGIN_NAMESPACE_NEX

class StreamManager;

/* Please see man pages for details 
    
    
    

    
    
    
    
*/
class OSDK_API SecureConnectionClient: public ServiceClient
{
public:
    /* Please see man pages for details */
    SecureConnectionClient();
    /* Please see man pages for details */
    virtual ~SecureConnectionClient();

    /* Please see man pages for details */
    qBool RegisterURLs(ProtocolCallContext *pContext, qResult* prResult, StationURL* purlPublic);

    /* Please see man pages for details */
    qBool RegisterURLsEx(ProtocolCallContext *pContext, qResult* prResult, StationURL* purlPublic,
                         const DataHolder& hCustomMessage);

    /* Please see man pages for details */
    qBool RequestConnectionData(ProtocolCallContext *pContext,
                                qUnsignedInt32 cidTarget, qUnsignedInt32 pidTarget,
                                qList<ConnectionData>* plstConnectionsData);

    /* Please see man pages for details 
        


        
        
        

        
        

        

        
        
        
        
        
        
        
    */
    qBool RequestURLs(ProtocolCallContext *pContext,
                      qUnsignedInt32 cidTarget, qUnsignedInt32 pidTarget,
                      qList<StationURL>* plstURLs);

    /* Please see man pages for details 
        

        
        
        

        
        
        

        
        
        
        
        
        
        
    */
    qBool RequestURLs(ProtocolCallContext *pContext,qUnsignedInt32 pidTarget,
                      qList<StationURL>* plstURLs);

    qBool TestConnectivity(ProtocolCallContext *pContext);

    qBool UpdateURLs(ProtocolCallContext *pContext);


    /*
     * :private
     * Updates the URL information of local stations that match the address and port, including the information on the server.
     *
     * If the URL of a local station that matches the address and port of targetUrl is successfully updated using Network::ReplaceLocalURL, the URLs of hosts that can be retrieved from the server with the GatheringURLs function are also updated.
     * URLs that do not have a port number cannot be updated. The server adds the RVCID and PID at the time of update.
     *
     * \param[in, out] pContext  Specifies a pointer to ProtocolCallContext for this call.
     * \param[in] targetUrl  Specifies the URL to be replaced.
     * \param[in] url  Specifies the URL after replacement.
     * \retval true Specifies the call succeeded. (Does not specify successful replacement.)
     * \retval false  Specifies the call failed, or the replacement of the local station URL failed. (URL with the same address did not exist.)
     
*/
    qBool ReplaceURL(ProtocolCallContext *pContext, const StationURL& targetUrl, const StationURL& url);

    /*
     * :private
     * Sends the binary report to the server.
     *
     * Does not check whether it has been sent successfully.
     *
     * \param[in] reportId  The type of report to send.
     * \param[in] reportData  The address for the binary data of the report to send.
     * \param[in] reportDataSize  The size of the binary data of the report to send.
     */
    void SendReport(qUnsignedInt32 reportId, const void* reportData, qUnsignedInt32 reportDataSize);


protected:
    /* Please see man pages for details */
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    void PrepareURLs(qList<StationURL>& lstURLs);

private:
    OSDK_DISABLE_EXPORT_WARNING(SecureConnectionProtocolClient m_oProtocol);
};

END_NAMESPACE_NEX

#endif
