/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SecureEndPoint_H_
#define _SecureEndPoint_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>

BEGIN_NAMESPACE_NEX

class OSDK_API SecureEndPoint: public EndPoint
{
public:
    SecureEndPoint(ConnectionOrientedStream *pTransport, const StationURL *pStationURL);
    virtual ~SecureEndPoint();

    EndPoint *GetAssociatedEndPoint();
    void SetAssociatedEndPoint(EndPoint *pEndPoint);

    virtual qUnsignedInt32  GetRTT() const;
    virtual qUnsignedInt32  GetRTTAverage() const;

    virtual qResult         RegisterEventHandler(EndPointEventHandler *pEndPointEventHandler);

    virtual qBool           IsNotConnected();
    virtual qBool           IsConnecting();
    virtual qBool           IsDisconnecting();
    virtual qBool           IsFaulty();
    virtual qBool           IsConnected();

    virtual qBool           PeerIsConnected();
    virtual qBool           PeerIsDisconnected();

    virtual void            SetPeerConnected();
    virtual void            SetPeerDisconnected();

    virtual ConnectionState GetConnectionState() const;
    virtual qBool           SetConnectionState(ConnectionState eState);

    virtual void            SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout);
    virtual void            SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime);
    virtual qUnsignedInt32  GetKeepAliveTimeout();
    virtual qUnsignedInt32  GetMaxSilenceTime();
    virtual qBool IsAnyPacketProcessed();
    virtual qUnsignedInt32 GetSignaledFaultError() const;

    /* Please see man pages for details */
    void    CompleteClose();
    void    CompleteDisconnect();



    /*
     
     
     
     */
    virtual void ResetEndPointInfo()
    {
        SYSTEMCHECK(false);
    };

    /*
     
     
     
     
     
     
     */
    virtual qBool GetEndPointInfo(EndPointInfo* info) const
    {
        DUMMY_PARAM(info);
        SYSTEMCHECK(false);
        return false;
    }


    /*
     
     
     
     
     
     
     */
    virtual qUnsignedInt32 GetReliableBufferPacketCount() const
    {
        SYSTEMCHECK(false);
        return 0;
    }

    virtual void GetReliableBufferPacketCounts(nn::nex::qVector<qUnsignedInt32>&) const
    {
        SYSTEMCHECK(false);
    }

    virtual qByte GetMaxSubStreamID() const
    {
        return 0;
    }

#if NEX_ENABLE_USE_SECURE_KEY
    virtual void SetEncryptionKey(const Key &oKey);
    virtual Key *GetEncryptionKey();
#endif

protected:

    //  TODO:CHECK - dead code
    virtual qResult         _Connect(Buffer *pInBuffer, Buffer *pOutBuffer,
                                     const SecureEndPoint::Callback pCallback, const UserContext &oContext,
                                     qUnsignedInt32 uiTimeout);

    //  TODO:CHECK - dead code
    virtual qResult         _Disconnect(const EndPoint::Callback pCallback, const UserContext &oContext,
                                        qUnsignedInt32 uiTimeout);

    //  TODO:CHECK - dead code
    virtual qResult         _Send(Buffer *pBuffer, qUnsignedInt32 uiFlags, qBool bPacketOutInternalBuffer=false, qByte substream=0, qUnsignedInt32 uiCallID=0);

    static void             DisconnectCallback(EndPoint* pEndPoint, qResult rResult,
            const UserContext* pContext);

    virtual qUnsignedInt64 GetSupportedFunctionsFlags() const;

private:

    void                    SignalFaultEvent(qUnsignedInt32 uiError, qBool bProcessNow);

    EndPoint*                   m_pAssociatedEndPoint;

    SecureEndPoint::Callback    m_pOwnDisconnectCallback;
    UserContext                 m_oOwnDisconnectUserContext;
};


END_NAMESPACE_NEX

#endif
