﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BuiltInGroups_H_
#define _BuiltInGroups_H_

BEGIN_NAMESPACE_NEX

/**
\brief Namespace for BuiltInGroups enum.
*/
#ifdef SWIG
class BuiltInGroups
{
public:
#else
namespace BuiltInGroups
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the groups, or categories, to which a given Principal may belong.
    \remarks A Principal may belong to more than one group. You cannot modify the built-in Group s however,
    you may add an almost unlimited number of user-defined groups.

    By default, Rendez-Vous grants or denies certain Capabilities to certain groups. For example,
    only Administrators may configure a sandbox and Guests cannot change their password. For
    full details of the default access rights, log in to the Rendez-Vous evaluation sandbox
    via the Management Console and have a look at the individual Capabilities listed in the
    Security section.

    Configuration of access rights via Capabilities and Group s is performed via the
    Back-end Management Service. Refer to this service for details as to how to implement
    custom access-rights.

    \note In order to access these types you must include the RendezVous.h file.
    \sa Principal, PrincipalID
    */
    enum BuiltInGroups
    {

        /**
        The ID of the built-in All group. All Principal s always belong to this Group.
        */
        All = 1,
        /**
        The ID of the built-in Services group. All Services belong to this Group.
        */
        Services = 2,
        /**
        The ID of the built-in Administrators group. Administrators are usually given unrestricted
        abilities to modify the lobby or database.
        */
        Administrators = 4,
        /**
        The ID of the built-in Monitoring group. This group is mainly used internally.
        */
        Monitoring = 8,
        /**
        The ID of the built-in Guests group. Guests are usually given limited lobby Capabilities,
        such as not being able to participate in ranked game sessions or create a friends list.
        */
        Guests = 16,
        /**
        The ID of the built-in Players group.
        */
        Players = 32,
        /**
        The ID of the built-in Siege group.
        */
        Siege = 64,
        /**
        The ID of the built-in Background Players group.
        */
        BackgroundPlayers = 128,
        /**
        The ID of the first user-defined group, which by default is equal to 256. As a Principal
        may belong to more than one group, IDs are defined as bits and must therefore, increment
        by a factor of two.
        */
        FirstUserGroup = 256
    };
#ifdef SWIG
};
#else
}
#endif
END_NAMESPACE_NEX

#endif
