﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _LogDeviceID_H_
#define _LogDeviceID_H_

BEGIN_NAMESPACE_NEX

/**
\brief Namespace for enum LogDeviceID.
*/
#ifdef SWIG
class LogDeviceID
{
public:
#else
namespace LogDeviceID
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the unique IDs assigned to each Rendez-Vous log device.
    \remarks For further information regarding the different types of log devices refer to Log Devices.
    \note In order to access these types you must include the LogDeviceID.h file.
    */
    enum LogDeviceID
    {

        /**
        A null (nonexistent) log device.
        */
        Null                                = 0,
        /**
        Logs to a standard console output.
        */
        ConsoleOutput                       = 1,
        /**
        Logs to the debug output of an attached debugger.
        */
        DebugOutput                         = 2,
        /**
        Logs to a file.
        */
        FileOutput                          = 3,
        /**
        Logs to a remote log device.
        */
        RemoteLogDevice                     = 4,
        /**
        Logs to a set of rotating files.
        */
        RotatingLogDevice                   = 6,
        /** \internal */
        First                               = Null,
        /** \internal */
        Last                                = RotatingLogDevice
    };
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif //#ifndef _LogDeviceID_H_
