﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _KerberosEncryption_H
#define _KerberosEncryption_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Core/Encryption/RC4/RC4Encryption.h>
#include <OnlineCore/src/Core/Checksum/HMAC/HMACChecksum.h>

#ifdef DEBUG_VERSION
#include <OnlineCore/src/Core/Key.h>
#endif


BEGIN_NAMESPACE_NEX

//n2690
#if NEX_ENABLE_KERBEROS_KEY_LENGTH_MAX
#define KERBEROS_KEY_LENGTH 32
#else
#define KERBEROS_KEY_LENGTH 16
#endif

class Buffer;
class Key;
class KeyDerivation;

class OSDK_API KerberosEncryption: public RootObject
{
public:
    KerberosEncryption();
    ~KerberosEncryption();

    qBool Encrypt(const Buffer& oInBuffer, Buffer *pOutBuffer);
    qBool Decrypt(const Buffer& oInBuffer, Buffer *pOutBuffer);

    qBool Encrypt(const Buffer& oInBuffer, Buffer *pOutBuffer, const Key& oKey);
    qBool Decrypt(const Buffer& oInBuffer, Buffer *pOutBuffer, const Key& oKey);

    static qBool CreateKey(CallContext* pCallContext, PrincipalID id, const qChar8* szPassword, Key* pkeyResult);
    static Key CreateKey(PrincipalID id, const qChar8* szPassword);

    static qBool InitializeKey(CallContext* pCallContext, const qChar8* szPassword, Key* pkeyResult);
    static Key InitializeKey(const qChar8* szPassword);

    static qBool FinalizeKey(CallContext* pCallContext, PrincipalID id, const Key & oPartialKey, Key* pkeyResult);
    static Key FinalizeKey(PrincipalID id, const Key & oPartialKey);

    static void SetKeyDerivation(KeyDerivation *pKeyDerivation);

    void SetKey(const Key& oKey);
    const Key& GetKey() const { return m_oKey; }

    static KeyCache* s_pKeyCache;
    static KeyDerivation* s_pKeyDerivation;
    static void EnablePasswordCache();
    static void DisablePasswordCache();

    /*
     * Create a new "key" that is the concatenated, MD5 hash of "additionalKey" and the original key.
     */
    Key GenerateConcatenateKey(const Key& additionalKey) const;

    /*
     * Create a new "key" that is the concatenated, MD5 hash of "key1" and "key2."
     */
    static Key GenerateConcatenateKey(const Key& key1, const Key& key2);

private:
    RC4Encryption m_oLocalEncryption;
    HMACChecksum m_oLocalChecksum;
    Key m_oKey;
};

END_NAMESPACE_NEX

#endif

