﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NotificationEventHandler_H
#define _NotificationEventHandler_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <RendezVous/Core/src/Common/NotificationEvent.h>

BEGIN_NAMESPACE_NEX

/*
\ingroup group_rdv_core_common
\brief This class processes all NotificationEvents sent by any given service.

\remarks This class manages the events that occur when a Process receives a given
NotificationEvent. When a Process receives a NotificationEvent, the event is sent to
the registered NotificationEventHandler of the given Process. Before a NotificationEventHandler
can be used *it must* be registered to the back-end services via *RendezVous::RegisterNotificationEventHandler*.

By default, all received NotificationEvents are ignored. If you want to customize how Notification events
are processed, then you need to sub-class this class and implement the ProcessNotificationEvent method.

\code

class MyNotifHandler: public NotificationEventHandler
{
public:
    void ProcessNotificationEvent(const NotificationEvent& event);
};

void MyNotifHandler::ProcessNotificationEvent(const NotificationEvent& event)
{
    String outputMessage;
    qUnsignedInt32 eventType = event.GetType();
    if(eventType == NotificationEvents::GameSessionEvent)
    {
        outputMessage.Format(_T("Notification received EventType = %d"), eventType);
        QLOG(EventLog::Info, outputMessage);
        qUnsignedInt32 subType = event.GetSubType();
        switch(subType)
        {
            case GameSessionEvents::MessageReceived: //Sub-type
                outputMessage.Format(_T("Notification message=> (%s)"), event.GetStringParam());
                QLOG(EventLog::Info, outputMessage);
                break;

            default:
                outputMessage.Format(_T("Unhandled SubType = %d"), subType);
                QLOG(EventLog::Error, outputMessage);
                break;
        }
    }
    else
    {
        outputMessage.Format(_T("Unhandled EventType = %d"), eventType);
        QLOG(EventLog::Error, outputMessage);
    }
}
\endcode

\sa NotificationEvent, NotificationEvents
*/
/* Please see man pages for details 
 

 
 
 

 
 */
class OSDK_API NotificationEventHandler : public RootObject
{
public:
    NotificationEventHandler() {}
    virtual ~NotificationEventHandler() {};

    /*
    \brief Is called by \productname each time the specified Notification event occurs.
    \param oEvent The NotificationEvent object to be processed.

    \remarks This method will be called by \productname whenever the specified Notification event
    is received by the Process to which this NotificationEventHandler is registered. By default, all
    received NotificationEvents are ignored. If you want to specify what should happen when a given
    NotificationEvent is received, then you need to sub-class this class and implement this method.
    */
    /* Please see man pages for details 
     

     
    */
    virtual void ProcessNotificationEvent(const NotificationEvent& oEvent) = 0;
};

END_NAMESPACE_NEX

#endif
