﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ProtocolCallContext_H_
#define _ProtocolCallContext_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/SpinTest.h>

BEGIN_NAMESPACE_NEX

class Credentials;

/*
\ingroup group_rdv_core_common
\brief This class defines the context of a call to a lobby service.
\remarks This class is used to define the call context of calls made to the lobby service. The basic
pattern you should follow when using a ProtocolCallContext with a \productname call is as follows,
where TEST() should be replaced by your own code that tests that the specified expressions are true.

\code
        ProtocolCallContext oContext;
        // ALWAYS initialize out parameters!
        qBool bCallResult = false;
        Parameter oSomeInputParameter = oSomeImportantValue;
        Parameter oSomeOutputParameter = oSomeInitialValue;
        qBool bCallIssued = GetSomeClient()->CallSomeRMC(&oContext, &bCallResult, oSomeInputParameter, &oSomeOutputParameter);
        // 1. Check that the RMC was issued successfully. The qBool return value
        // will be false if the RMC failed to be issued from the client.
        TEST(bCallIssued == true);
        // 2. Check that Wait returns true if a blocking call is made. This will return
        // false if the RMC did not get a return message (for example, timeout)
        TEST(oContext.Wait());
        // 3. Check that GetState==CallSuccess. It will be false if the RMC execution failed
        // on the server (for example, an exception was thrown).
        if (oContext.GetState==CallContext::CallSuccess) {
            // 4a. GetOutcome normally returns true in this case. If not, you should check the
            // Error Codes for details.
            TEST(QSUCCEEDED(oContext.GetOutcome().GetReturnCode()));
            // 4b. This will be false iff the remote method returns false.
            TEST(bCallResult == true);
        } else {
            // 5a. GetOutcome returns false in this case. You should check the Error Codes for details.
            TEST(QFAILED(oContext.GetOutcome().GetReturnCode()));
            // 5b. The remote method's return value was likely not set since the call failed.
            TEST(bCallResult == false);
        }
\endcode

Refer to AccountManagementClient::UpdateCustomData for an example of a ProtocolCallContext in use.
*/

/* Please see man pages for details 
 

 
 

 
 */
class OSDK_API ProtocolCallContext : public CallContext
{
public:
    ProtocolCallContext(Credentials *pCredentials=NULL);
    virtual ~ProtocolCallContext();

    /* Please see man pages for details 
    
    */
    void AddReturnValuePtr(void* pReturnValue);
    /* Please see man pages for details 
    
    */
    void* GetReturnValuePtr(qUnsignedInt iIndex);
    /* Please see man pages for details 
    
    */
    void ClearReturnValuePtr();
    /* Please see man pages for details 
    
    */
    void PatchReturnCodePtr(qBool *pbResult)
    {
        m_pbResult = pbResult;
    }
    /* Please see man pages for details 
    
    */
    void PatchReturnCodePtr(qResult *pqResult)
    {
        m_pqResult = pqResult;
    }

    /* Please see man pages for details 
    
    */
    void SetCredentials(Credentials *pCredentials);
    /* Please see man pages for details 
    
    */
    Credentials* GetCredentials() const
    {
        return(m_pCredentials);
    }

private:
    virtual void BeginTransition(State eNewState, qResult rOutcome, qBool bSetOutcome);

private:
    OSDK_DISABLE_EXPORT_WARNING(qVector<void*> m_vReturnValuePtr);
    qBool *         m_pbResult;
    qResult *       m_pqResult;
    Credentials *   m_pCredentials;
};

END_NAMESPACE_NEX

#endif
