﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Ticket_H
#define _Ticket_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <OnlineCore/src/Core/Key.h>

BEGIN_NAMESPACE_NEX

class KerberosEncryption;

class OSDK_API Ticket: public RefCountedObject
{
public:
#if NEX_ENABLE_USE_TICKET_PERIOD
    Ticket(PrincipalID idOwner, PrincipalID idTarget, const Key& oKey, DateTime dtCreatedAt);
#else
    Ticket(PrincipalID idOwner, PrincipalID idTarget, const Key& oKey);
#endif
    Ticket(const Buffer& bufEncryptedTicket, PrincipalID idTarget);
#if NEX_ENABLE_USE_TICKET_PERIOD
    Ticket(const Buffer& bufEncryptedTicket, PrincipalID idOwner, PrincipalID idTarget, const Key& oKey, DateTime dtCreatedAt);
#else
    Ticket(const Buffer& bufEncryptedTicket, PrincipalID idOwner, PrincipalID idTarget, const Key& oKey, DateTime dtCreatedAt);
#endif
    virtual ~Ticket();

    PrincipalID GetOwnerID() const
    {
        return m_idOwner;
    }
    PrincipalID GetTargetID() const
    {
        return m_idTarget;
    }
#if NEX_ENABLE_USE_TICKET_PERIOD
    DateTime GetCreatedAt() const
    {
        return m_dtCreatedAt;
    }
#endif
    const Buffer& GetBuffer() const
    {
        return m_bufEncryptedTicket;
    }
    const Key& GetKey() const
    {
        return m_key;
    }

    void Encrypt(KerberosEncryption* pEncryption, const Key& oEncryptionKeyOfTarget);
    void Decrypt(KerberosEncryption* pEncryption, const Key& initializeVector);

    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;

    qBool IsValid() const;

private:
    qBool m_bValid;
    // Real ticket (encrypted and signed)
    Buffer m_bufEncryptedTicket;

    // Ticket meaning
    PrincipalID m_idOwner;
    PrincipalID m_idTarget;
    Key m_key;
#if NEX_ENABLE_USE_TICKET_PERIOD
    DateTime m_dtCreatedAt;
#endif
    // TODO - Should have timestamp/expiration info
};

END_NAMESPACE_NEX

#endif

