﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumeration must be
// specify so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

#ifndef _JugemAuthenticationErrors_H_
#define _JugemAuthenticationErrors_H_

#ifndef SWIG
#include <OnlineCore/src/Platform/Core/Result.h>
#endif

BEGIN_NAMESPACE_NEX

//group=Error Code Enumerations
/* Please see man pages for details 
 
 */
#ifdef SWIG
class AuthenticationErrorCode
{
public:
#else
namespace AuthenticationErrorCode
{
#endif

    //group= Type Definitions

    /*
    Description: Enumerates the error codes for the StorageBase facility that a call may return.
    */
    /* Please see man pages for details 
     
    */
    enum ErrorCode
    {
        /*
         NAS returns error code
         */
        NASAuthenticateError = 1,    //
        /*
         can't parse NAS return content
         */
        TokenParseError = 2,         //
        /*
         Http Connection Error
         */
        HttpConnectionError = 3,     //
        /*
         Http DNS Error
         */
        HttpDNSError = 4,            //
        /*
         Http Get Proxy Setting Error
         */
        HttpGetProxySetting = 5,     //
        /*
         RendezVous : Token Expired
         */
        TokenExpired = 6,            //
        /*
         RendezVous : Validation fail
         */
        ValidationFailed = 7,        //
        /*
         RendezVous : token has invalid param
         */
        InvalidParam = 8,            //
        /*
         RendezVous : token pid is not user pid
         */
        PrincipalIdUnmatched = 9,    //
        /*
         RendezVous : move count unmatch
         */
        MoveCountUnmatch = 10,       //
        /*
         RendezVous : The servers are now under maintenance.
        */
        UnderMaintenance = 11,       //
        /*
         RendezVous : This version of client is not supported.
        */
        UnsupportedVersion = 12,     //
        /*
         RendezVous : NGS version is old for this client. Please update NGS.
        */
        ServerVersionIsOld = 13,     //
        /*
         Unknown Error
         */
        Unknown = 14,                 //
        /*
         RendezVous : NGS version is new for this client. Please update NEX.
         */
        ClientVersionIsOld = 15       //
    };
    //note: When adding new results, it is necessary to change `NEX_QRESULT_TO_STR` in `OnlineCore/src/Platform/Core/Result.cpp`.
    //See operations done when adding error codes to the wiki.
    //If the errors the application should handle have increased, add them to the list of errors the application should handle in the`overall.rst` of the `common manual.`

#ifndef SWIG
    // For LobbyApi compatibility
    inline qResult ConvertResultTransportToAuthentication(const qResult& result)
    {
        switch(result.GetReturnCode())
        {
        case QERROR(Transport, DnsError):
            return QRESULT_ERROR(Authentication, HttpDNSError);
        case QERROR(Transport, InvalidUrl):
            return QRESULT_ERROR(Authentication, HttpGetProxySetting);
        case QERROR(Transport, Unknown):
            return QRESULT_ERROR(Authentication, HttpConnectionError);
        default:
            return result;
        }
    }
#endif
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
