﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are 
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DATA_STORE_CHANGE_META_COMPARE_PARAM_H
#define _DATA_STORE_CHANGE_META_COMPARE_PARAM_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreChangeMetaCompareParamDDL.h"
#include "DataStoreConstants.h"
#include "DataStoreMetaInfo.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
class DataStoreChangeMetaCompareParam: public DDLCLASS(DataStoreChangeMetaCompareParam) {
public:
    /* Please see man pages for details 
        

        

            
    */
    DataStoreChangeMetaCompareParam()
    {
        Reset();
    }

    /* Please see man pages for details 
        

        

            

        
        
    */
    DataStoreChangeMetaCompareParam(qUnsignedInt32 comparisonFlag, const DataStoreMetaInfo& metaInfo)
    {
        Set(comparisonFlag, metaInfo);
    }

    /* Please see man pages for details 
        
    */
    virtual ~DataStoreChangeMetaCompareParam(){}

    /* Please see man pages for details 
        

            
            

        
        
    */
    void Set(qUnsignedInt32 comparisonFlag, const DataStoreMetaInfo& metaInfo)
    {
        Reset();
        SetComparisonFlag(comparisonFlag);
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_NAME)
        {
            SetName(metaInfo.GetName());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_ACCESS_PERMISSION)
        {
            SetAccessPermission(metaInfo.GetAccessPermission());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_UPDATE_PERMISSION)
        {
            SetUpdatePermission(metaInfo.GetUpdatePermission());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_PERIOD)
        {
            SetPeriod(metaInfo.GetPeriod());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_METABINARY)
        {
            qVector<qByte> bin = metaInfo.GetMetaBinary();
            if (!bin.empty())
            {
                SetMetaBinary(&bin[0], bin.size());
            }
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_TAGS)
        {
            SetTags(metaInfo.GetTags());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_DATA_TYPE)
        {
            SetDataType(metaInfo.GetDataType());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_REFERRED_CNT)
        {
            SetReferredCount(metaInfo.GetReferredCount());
        }
        if (comparisonFlag & DataStoreConstants::COMPARISON_FLAG_STATUS)
        {
            SetDataStatus(metaInfo.GetDataStatus());
        }
    }
    
    /* Please see man pages for details 
        

            
            

        
    */
    void SetComparisonFlag(qUnsignedInt32 comparisonFlag) { this->comparisonFlag = comparisonFlag; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetComparisonFlag() const { return this->comparisonFlag; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetName(const String& name)
    {
#ifdef LOG_ENABLED
        if (name.GetLength() > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, _T("DataStoreChangeMetaCompareParam::SetName(): name length is over the limit. ") << _T("name=") << name << _T(", length=") << name.GetLength());
        }
#endif
        this->name = name;
    }
    
    /* Please see man pages for details 
        

        
    */
    const String& GetName() const { return this->name; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetAccessPermission(const DataStorePermission& permission) { this->permission = permission; }
    
    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetAccessPermission() const { return this->permission; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission) { this->delPermission = updatePermission; }
    
    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetUpdatePermission() const { return this->delPermission; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetPeriod(qUnsignedInt16 period) { this->period = period; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetPeriod() const { return this->period; }
    
    /* Please see man pages for details 
        

            
            
            

        
    */
    void SetTags(const qVector<String>& tags)
    {
#ifdef LOG_ENABLED
        if (tags.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, _T("DataStoreChangeMetaCompareParam::SetTags(): slot size is over the limit. size=") << tags.size());
        }

        qVector<String>::const_iterator it = tags.begin();
        for (; it != tags.end(); ++it)
        {
            if (it->GetLength() > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, _T("DataStoreChangeMetaCompareParam::SetTags(): tag length is over the limit. ") << _T("tag=") << *it << _T(", length=") << it->GetLength());
            }
        }
#endif
        this->tags = tags;
    }
    
    /* Please see man pages for details 
        

        
    */
    const qVector<String>& GetTags() const { return this->tags; }

    /* Please see man pages for details 
        

            
            

        
        
    */
    void SetMetaBinary(const void* p, qUnsignedInt32 size) {
        this->metaBinary.clear();
        if (p != NULL) {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /* Please see man pages for details 
        

        

        
    */
    qVector<qByte> GetMetaBinary() const {
        // Conversion from qBuffer to qVector<qByte>
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end()); 
        return vb;
    }

    /* Please see man pages for details 
        

        

        

        
    */
    void GetMetaBinary(qVector<qByte>& metaBinary) const {
        // Conversion from qBuffer to qVector<qByte>
        metaBinary.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /* Please see man pages for details 
        

        
    */
    void SetDataType(qUnsignedInt16 dataType) { this->dataType = dataType; }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetDataType() const { return this->dataType; }

    /* Please see man pages for details 
        

        
    */
    void SetDataStatus(DataStoreConstants::DataStatus status) { this->status = static_cast<qByte>(status); }

    /* Please see man pages for details 
        

        
    */
    DataStoreConstants::DataStatus GetDataStatus() const { return static_cast<DataStoreConstants::DataStatus>(this->status); }

    /* Please see man pages for details 
        

        
    */
    void SetReferredCount(qUnsignedInt32 referredCount) { this->referredCnt = referredCount; }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetReferredCount() const { return this->referredCnt; }

    /* Please see man pages for details 
        
    */
    void Reset()
    {
        SetComparisonFlag(DataStoreConstants::COMPARISON_FLAG_NONE);
        SetName(_T(""));
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetDataType(0);
        SetDataStatus(DataStoreConstants::DATA_STATUS_NONE);
        SetReferredCount(0);
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << _T("DataStoreChangeMetaCompareParam: (")
           << _T(", comparisonFlag") << comparisonFlag
           << _T(", name") << name
           << _T(", accessPermission=") << permission
           << _T(", updatePermission=") << delPermission
           << _T(", period=") << period
           << _T(", dataType=") << dataType
           << _T(", referredCnt=") << referredCnt
           << _T(", status=") << status;

        ss << _T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << _T(", ");
        }
        ss << _T(")");

        ss << _T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
           ss << hex << *it;
        }

        ss << _T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

#endif
