﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are 
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DATA_STORE_CHANGE_META_PARAM_V1_H
#define _DATA_STORE_CHANGE_META_PARAM_V1_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreChangeMetaParamV1DDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
class DataStoreChangeMetaParamV1: public DDLCLASS(DataStoreChangeMetaParamV1) {
public:
    /* Please see man pages for details 
        
    */
    DataStoreChangeMetaParamV1()
    {
        Reset();
    }
    
    /* Please see man pages for details 
        
    */
    virtual ~DataStoreChangeMetaParamV1(){}
    
    /* Please see man pages for details 
        

            

        
    */
    void SetDataId(qUnsignedInt64 dataId) { this->dataId = dataId; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt64 GetDataId() const { return this->dataId; }
    
    /* Please see man pages for details 
        

            
            

        
    */
    void SetModificationFlag(qUnsignedInt32 modificationFlag) { this->modifiesFlag = modificationFlag; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetModificationFlag() const { return this->modifiesFlag; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetName(const String& name)
    {
#ifdef LOG_ENABLED
        if (name.GetLength() > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, _T("DataStoreChangeMetaParamV1::SetName(): name length is over the limit. ") << _T("name=") << name << _T(", length=") << name.GetLength());
        }
#endif
        this->name = name;
    }
    
    /* Please see man pages for details 
        

        
    */
    const String& GetName() const { return this->name; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetAccessPermission(const DataStorePermission& permission) { this->permission = permission; }
    
    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetAccessPermission() const { return this->permission; }
    
    /* Please see man pages for details 
        

            
            

        
    */
    void SetDeletePermission(const DataStorePermission& delPermission)
    {
        QLOG(EventLog::Warning, _T("DataStoreChangeMetaParamV1::SetDeletePermission() is deprecated. Use DataStoreChangeMetaParamV1::SetUpdatePermission()"));
        this->delPermission = delPermission;
    }
    
    /* Please see man pages for details 
        

            

       
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, _T("DataStoreChangeMetaParamV1::GetDeletePermission() is deprecated. Use DataStoreChangeMetaParamV1::GetUpdatePermission()"));
        return this->delPermission;
    }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission) { this->delPermission = updatePermission; }
    
    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetUpdatePermission() const { return this->delPermission; }
    
    /* Please see man pages for details 
        

            

        
    */
    void SetPeriod(qUnsignedInt16 period) { this->period = period; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetPeriod() const { return this->period; }
    
    /* Please see man pages for details 
        

            
            
            

        
    */
    void SetTags(const qVector<String>& tags)
    {
#ifdef LOG_ENABLED
        if (tags.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, _T("DataStoreChangeMetaParamV1::SetTags(): slot size is over the limit. size=") << tags.size());
        }

        qVector<String>::const_iterator it = tags.begin();
        for (; it != tags.end(); ++it)
        {
            if (it->GetLength() > DataStoreConstants::MAX_TAG_LENGTH)
            {
                QLOG(EventLog::Warning, _T("DataStoreChangeMetaParamV1::SetTags(): tag length is over the limit. ") << _T("tag=") << *it << _T(", length=") << it->GetLength());
            }
        }
#endif
        this->tags = tags;
    }
    
    /* Please see man pages for details 
        

        
    */
    const qVector<String>& GetTags() const { return this->tags; }

    /* Please see man pages for details 
        

            
            

        
        
    */
    void SetMetaBinary(const void* p, qUnsignedInt32 size) {
        this->metaBinary.clear();
        if (p != NULL) {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /* Please see man pages for details 
        

        

        
    */
    qVector<qByte> GetMetaBinary() const {
        // Conversion from qBuffer to qVector<qByte>
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end()); 
        return vb;
    }

    /* Please see man pages for details 
        

        

        

        
    */
    void GetMetaBinary(qVector<qByte>& metaBinary) const {
        // Conversion from qBuffer to qVector<qByte>
        metaBinary.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /* Please see man pages for details 
        

            
            
            
            
            

        
    */
    void SetUpdatePassword(qUnsignedInt64 updatePassword) { this->updatePassword = updatePassword; }
    
    /* Please see man pages for details 
        

        
    */
    qUnsignedInt64 GetUpdatePassword() const { return this->updatePassword; }

    /* Please see man pages for details 
        
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetModificationFlag(DataStoreConstants::MODIFICATION_FLAG_NONE);
        SetName(_T(""));
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetUpdatePassword(DataStoreConstants::INVALID_PASSWORD);
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << _T("DataStoreChangeMetaParamV1: (")
           << _T("dataId=") << dataId
           << _T(", modificationFlag") << modifiesFlag
           << _T(", name") << name
           << _T(", accessPermission=") << permission
           << _T(", updatePermission=") << delPermission
           << _T(", period=") << period
           << _T(", updatePassword=") << updatePassword;

        ss << _T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << _T(", ");
        }
        ss << _T(")");

        ss << _T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
           ss << hex << *it;
        }

        ss << _T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

#endif
