﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DATA_STORE_META_INFO_H
#define _DATA_STORE_META_INFO_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreMetaInfoDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
class DataStoreMetaInfo: public DDLCLASS(DataStoreMetaInfo)
{
public:
    /* Please see man pages for details 
        
    */
    DataStoreMetaInfo()
    {
        // When the user generates this class, IsValid function will return false.
        dataId = DataStoreConstants::INVALID_DATAID;
    }

    /* Please see man pages for details 
        
    */
    virtual ~DataStoreMetaInfo() {}

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetOwnerId() const
    {
        return this->ownerId;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetSize() const
    {
        return this->size;
    }

    /* Please see man pages for details 
        

        
    */
    String GetName() const
    {
        return this->name;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetDataType() const
    {
        return this->dataType;
    }

    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetAccessPermission() const
    {
        return this->permission;
    }

    /* Please see man pages for details 
        

            

       
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, _T("DataStoreMetaInfo::GetDeletePermission() is deprecated. Use DataStoreMetaInfo::GetUpdatePermission()"));
        return this->delPermission;
    }

    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetUpdatePermission() const
    {
        return this->delPermission;
    }

    /* Please see man pages for details 
        

        
    */
    const DateTime& GetCreatedTime() const
    {
        return this->createdTime;
    }

    /* Please see man pages for details 
        

        
    */
    const DateTime& GetUpdatedTime() const
    {
        return this->updatedTime;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetPeriod() const
    {
        return this->period;
    }

    /* Please see man pages for details 
        

        
    */
    DataStoreConstants::DataStatus GetDataStatus() const
    {
        return static_cast<DataStoreConstants::DataStatus>(this->status);
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetReferredCount() const
    {
        return this->referredCnt;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetReferDataId() const
    {
        return this->referDataId;
    }

    /* Please see man pages for details 
        

        
    */
    DataStoreConstants::DataFlag GetDataFlag() const
    {
        return static_cast<DataStoreConstants::DataFlag>(this->flag);
    }

    /* Please see man pages for details 
        

        
    */
    const DateTime& GetReferredTime() const
    {
        return this->referredTime;
    }

    /* Please see man pages for details 
        

        
    */
    const DateTime& GetExpireTime() const
    {
        return this->expireTime;
    }

    /* Please see man pages for details 
        

        
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /* Please see man pages for details 
        

        
    */
    qMap<qInt8, DataStoreRatingInfo> GetRating() const
    {
        qMap<qInt8, DataStoreRatingInfo> ratingMap;

        qVector<DataStoreRatingInfoWithSlot>::const_iterator it;
        for (it = this->ratings.begin(); it != this->ratings.end(); ++it)
        {
            ratingMap.insert(qMap<qInt8, DataStoreRatingInfo>::value_type(it->slot, it->rating));
        }
        return ratingMap;
    }

    /* Please see man pages for details 
        

        

        
    */
    void GetRating(qMap<qInt8, DataStoreRatingInfo>& ratingMap) const
    {
        ratingMap.clear();
        qVector<DataStoreRatingInfoWithSlot>::const_iterator it;
        for (it = this->ratings.begin(); it != this->ratings.end(); ++it)
        {
            ratingMap.insert(qMap<qInt8, DataStoreRatingInfo>::value_type(it->slot, it->rating));
        }
    }

    /* Please see man pages for details 
        

        

        
    */
    qVector<qByte> GetMetaBinary() const
    {
        // Conversion from qBuffer to qVector<qByte>
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end());
        return vb;
    }

    /* Please see man pages for details 
        

        

        

        
    */
    void GetMetaBinary(qVector<qByte>& metaBinary) const
    {
        // Conversion from qBuffer to qVector<qByte>
        metaBinary.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /* Please see man pages for details 
        

        

        
    */
    qBool IsValid() const
    {
        return dataId != DataStoreConstants::INVALID_DATAID;
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << _T("DataStoreMetaInfo: (")
           << _T("dataId=") << dataId
           << _T(", ownerId=") << ownerId
           << _T(", size=") << size
           << _T(", name=") << name
           << _T(", dataType=") << dataType
           << _T(", accessPermission=") << permission
           << _T(", updatePermission=") << delPermission
           << _T(", createdTime=") << createdTime
           << _T(", updatedTime=") << updatedTime
           << _T(", period=") << period
           << _T(", status=") << status
           << _T(", referredCnt=") << referredCnt
           << _T(", referDataId=") << referDataId
           << _T(", dataFlag=") << hex << flag << dec
           << _T(", referredTime=") << referredTime
           << _T(", expireTime=") << expireTime;

        ss << _T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << _T(", ");
        }
        ss << _T(")");

        ss << _T(", ratings=(");
        for (qVector<DataStoreRatingInfoWithSlot>::const_iterator it = ratings.begin(); it != ratings.end(); ++it)
        {
            ss << _T("slot=") << it->slot << _T(", ratingInfo=") << it->rating << _T(", ");
        }
        ss << _T(")");

        ss << _T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
            ss << hex << *it;
        }

        ss << _T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

#endif
