﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DataStorePermission_H
#define _DataStorePermission_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePermissionDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
class DataStorePermission: public DDLCLASS(DataStorePermission)
{

public:
    /* Please see man pages for details 
        

        
        
    */
    explicit DataStorePermission(DataStoreConstants::Permission permission = DataStoreConstants::PERMISSION_PRIVATE, const qVector<qUnsignedInt32>& recipientIds = qVector<qUnsignedInt32>())
    {
        SetPermission(permission);
        SetRecipientIds(recipientIds);
        // Test comparisons will no longer be possible if the order changes, but this is not an issue other than in testing, so do not sort
        // recipientIds.sort();
    }

    /* Please see man pages for details 
        
    */
    virtual ~DataStorePermission() {}

    /* Please see man pages for details 
        

        
    */
    void SetPermission(DataStoreConstants::Permission permission)
    {
        this->permission = static_cast<qByte>(permission);
    }

    /* Please see man pages for details 
        

        
    */
    DataStoreConstants::Permission GetPermission() const
    {
        return static_cast<DataStoreConstants::Permission>(this->permission);
    }

    /* Please see man pages for details 
        

            
            
            
            

        
    */
    void SetRecipientIds(const qVector<qUnsignedInt32>& recipientIds)
    {
        this->recipientIds = recipientIds;
    }

    /* Please see man pages for details 
        

        

        
        
        
    */
    const qVector<qUnsignedInt32>& GetRecipientIds() const
    {
        return this->recipientIds;
    }

    /* Please see man pages for details 
        
    */
    void Reset()
    {
        SetPermission(DataStoreConstants::PERMISSION_PRIVATE);
        SetRecipientIds(qVector<qUnsignedInt32>());
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /* Please see man pages for details 
        
        
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStorePermission& oPermission);
};

inline StringStream& operator<<(StringStream& oStr, const DataStorePermission& oPermission)
{
    qVector<qUnsignedInt32>::const_iterator itr = oPermission.recipientIds.begin();

    oStr << _T("DataStorePermission: (permission=") << oPermission.permission
         << _T(", targetIds=(");

    for (int i = 0; itr != oPermission.recipientIds.end(); i++, itr++)
    {
        if (i)
        {
            oStr << _T(", ");
        }
        oStr << *itr;
    }

    oStr << _T("))");

    return oStr;
}

END_NAMESPACE_NEX

#endif
