﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DATA_STORE_PREPARE_POST_PARAM_H
#define _DATA_STORE_PREPARE_POST_PARAM_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePreparePostParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    

    

*/
class DataStorePreparePostParam: public DDLCLASS(DataStorePreparePostParam)
{
public:
    /* Please see man pages for details 
        
    */
    DataStorePreparePostParam()
    {
        Reset();
    }

    /* Please see man pages for details 
        
    */
    virtual ~DataStorePreparePostParam() {}

    /* Please see man pages for details 
        

            
            
            

        
    */
    void SetSize(qUnsignedInt32 size)
    {
        this->size = size;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetSize() const
    {
        return this->size;
    }

    /* Please see man pages for details 
        

            
            

        
    */
    void SetName(const String& name)
    {
#ifdef LOG_ENABLED
        if (name.GetLength() > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, _T("DataStorePostParam::SetName(): name length is over the limit. ") << _T("name=") << name << _T(", length=") << name.GetLength());
        }
#endif
        this->name = name;
    }

    /* Please see man pages for details 
        

        
    */
    String GetName() const
    {
        return this->name;
    }

    /* Please see man pages for details 
        

            
            

        
    */
    void SetDataType(const qUnsignedInt16 dataType)
    {
        this->dataType = dataType;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetDataType() const
    {
        return this->dataType;
    }

    /* Please see man pages for details 
        

            
            

        
    */
    void SetAccessPermission(const DataStorePermission& permission)
    {
        this->permission = permission;
    }

    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetAccessPermission() const
    {
        return this->permission;
    }

    /* Please see man pages for details 
        

            
            

        
    */
    void SetDeletePermission(const DataStorePermission& delPermission)
    {
        QLOG(EventLog::Warning, _T("DataStorePostParam::SetDeletePermission() is deprecated. Use DataStorePostParam::SetUpdatePermission()"));
        this->delPermission = delPermission;
    }

    /* Please see man pages for details 
        

            

       
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, _T("DataStorePostParam::GetDeletePermission() is deprecated. Use DataStorePostParam::GetUpdatePermission()"));
        return this->delPermission;
    }

    /* Please see man pages for details 
        

            

        
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission)
    {
        this->delPermission = updatePermission;
    }

    /* Please see man pages for details 
        

        
    */
    const DataStorePermission& GetUpdatePermission() const
    {
        return this->delPermission;
    }

    /* Please see man pages for details 
        

            
            

        
    */
    void SetDataFlag(qUnsignedInt32 flag)
    {
        this->flag = flag;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetDataFlag() const
    {
        return this->flag;
    }

    /* Please see man pages for details 
        

            
            
            
            
            
            

        
    */
    void SetPeriod(qUnsignedInt16 period)
    {
        this->period = period;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt16 GetPeriod() const
    {
        return this->period;
    }

    /* Please see man pages for details 
        
        

            
            

        
    */
    void SetReferDataId(qUnsignedInt32 referDataId)
    {
        this->referDataId = referDataId;
    }

    /* Please see man pages for details 
        
        

        
    */
    qUnsignedInt32 GetReferDataId() const
    {
        return this->referDataId;
    }

    /* Please see man pages for details 
        

            
            
            


        
    */
    void SetTags(const qVector<String>& tags)
    {
#ifdef LOG_ENABLED
        if (tags.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, _T("DataStorePostParam::SetTags(): slot size is over the limit. size=") << tags.size());
        }
#endif
        this->tags = tags;
    }

    /* Please see man pages for details 
        

        
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /* Please see man pages for details 
        

            
            
            

        
    */
    void SetRatingSetting(const qMap<qInt8, DataStoreRatingInitParam>& ratingInitParam)
    {
        this->ratingInitParams.clear();
        qMap<qInt8, DataStoreRatingInitParam>::const_iterator it = ratingInitParam.begin();
        for (; it != ratingInitParam.end(); ++it)
        {
            DataStoreRatingInitParamWithSlot paramSlot;
            paramSlot.slot = it->first;
            paramSlot.param = it->second;
            this->ratingInitParams.push_back(paramSlot);
        }
    }

    /* Please see man pages for details 
     
     */
    void ClearRatingSetting()
    {
        this->ratingInitParams.clear();
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    qBool AddRatingSetting(qInt8 slot, const DataStoreRatingInitParam& ratingInitParam)
    {
        qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = this->ratingInitParams.begin();
        for (; it != this->ratingInitParams.end(); ++it)
        {
            if(it->slot == slot)
            {
                return false;
            }
        }

        DataStoreRatingInitParamWithSlot paramSlot;
        paramSlot.slot = slot;
        paramSlot.param = ratingInitParam;
        this->ratingInitParams.push_back(paramSlot);

        return true;
    }




    /* Please see man pages for details 
        

        
    */
    void GetRatingSetting(qMap<qInt8, DataStoreRatingInitParam>& ratingInitParam) const
    {
        ratingInitParam.clear();
        qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = this->ratingInitParams.begin();
        for (; it != this->ratingInitParams.end(); ++it)
        {
            ratingInitParam.insert(std::make_pair(it->slot, it->param));
        }
    }

    /* Please see man pages for details 
        

            
            

        
        
    */
    void SetMetaBinary(const void* p, qUnsignedInt32 size)
    {
        this->metaBinary.clear();
        if (p != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size);
        }
    }

    /* Please see man pages for details 
        

        

        
    */
    qVector<qByte> GetMetaBinary() const
    {
        // Conversion from qBuffer to qVector<qByte>
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end());
        return vb;
    }

    /* Please see man pages for details 
        

        

        

        
    */
    void GetMetaBinary(qVector<qByte>& metaBinary) const
    {
        // Conversion from qBuffer to qVector<qByte>
        metaBinary.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /* Please see man pages for details 
        

        

            
            

    */
    void SetPersistenceInitParam(const DataStorePersistenceInitParam& persistenceInitParam)
    {
        this->persistenceInitParam = persistenceInitParam;
    }

    /* Please see man pages for details 
        

        
    */
    const DataStorePersistenceInitParam& GetPersistenceInitParam() const
    {
        return this->persistenceInitParam;
    }

    /* Please see man pages for details 
        
    */
    void Reset()
    {
        SetSize(0);
        SetName(_T(""));
        SetDataType(0);
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetDataFlag(DataStoreConstants::DATA_FLAG_NONE);
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetReferDataId(DataStoreConstants::INVALID_DATAID);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetPersistenceInitParam(DataStorePersistenceInitParam());
        SetRatingSetting(qMap<qInt8, DataStoreRatingInitParam>());
        this->extraData.clear();
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << _T("DataStorePreparePostParam: (")
           << _T("size=") << size
           << _T(", name=") << name
           << _T(", dataType") << dataType
           << _T(", accessPermission=") << permission
           << _T(", updatePermission=") << delPermission
           << _T(", dataFlag=") << hex << flag << dec
           << _T(", period=") << period
           << _T(", referDataId=") << referDataId
           << _T(", persistenceInitParam=") << persistenceInitParam;

        ss << _T(", ratingInitParams=(");
        for (qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = ratingInitParams.begin(); it != ratingInitParams.end(); ++it)
        {
            ss << _T("slot=") << it->slot << _T(", ratingInitParam=") << it->param << _T(", ");
        }
        ss << _T(")");

        ss << _T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << _T(", ");
        }
        ss << _T(")");

        ss << _T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
            ss << hex << *it;
        }

        ss << _T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }

};

END_NAMESPACE_NEX

#endif
