﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/



#ifndef _DATA_STORE_RATING_LOCK_INIT_PARAM_H
#define _DATA_STORE_RATING_LOCK_INIT_PARAM_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
class DataStoreRatingLockInitParam: public RootObject
{
public:
    /* Please see man pages for details 
        
    */
    DataStoreRatingLockInitParam()
    {
        Reset();
    }

    /* Please see man pages for details 
        
    */
    virtual ~DataStoreRatingLockInitParam() {}

    /* Please see man pages for details 
        
    */
    void Reset()
    {
        lockType = DataStoreConstants::RATING_LOCK_NONE;
        periodDuration = 0;
        periodHour = 0;
    }

    /* Please see man pages for details 
        

        
    */
    void SetIntervalLock(qInt16 intervalSec)
    {
        SYSTEMCHECK(intervalSec >= 0);
        lockType = DataStoreConstants::RATING_LOCK_INTERVAL;
        periodDuration = intervalSec;
        periodHour = 0;
    }

    /* Please see man pages for details 
        

            
            

        
        
                   
                   
    */
    void SetPeriodicLock(DataStoreConstants::RatingLockPeriod period, qInt8 hour)
    {
        SYSTEMCHECK(period < 0);
        SYSTEMCHECK(hour >= -23 && hour <= 23);
        lockType = DataStoreConstants::RATING_LOCK_PERIOD;
        periodDuration = static_cast<qInt16>(period);
        periodHour = hour;
    }

    /* Please see man pages for details 
        

            
            
            
            
            
            

        
        
                   
    */
    void SetDaysAfterLock(qInt16 days, qInt8 hour)
    {
        SYSTEMCHECK(days >= 0);
        SYSTEMCHECK(hour >= -23 && hour <= 23);
        lockType = DataStoreConstants::RATING_LOCK_PERIOD;
        periodDuration = days;
        periodHour = hour;
    }

    /* Please see man pages for details 
        

    */
    void SetPermanentLock()
    {
        lockType = DataStoreConstants::RATING_LOCK_PERMANENT;
        periodDuration = 0;
        periodHour = 0;
    }

    // secret
    DataStoreConstants::RatingLockType GetLockType() const
    {
        return lockType;
    }

    // secret
    qInt8 GetPeriodHour() const
    {
        return periodHour;
    }

    // secret
    qInt16 GetPeriodDuration() const
    {
        return periodDuration;
    }
    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    inline friend StringStream& operator<<(StringStream& oStr, const DataStoreRatingLockInitParam& oRatingLockInitParam);

    qBool operator==(const DataStoreRatingLockInitParam& rhs) const
    {
        return lockType == rhs.lockType
               && periodHour == rhs.periodHour
               && periodDuration == rhs.periodDuration;
    }
    qBool operator!=(const DataStoreRatingLockInitParam& rhs) const
    {
        return !(*this == rhs);
    }
private:
    DataStoreConstants::RatingLockType lockType;
    qInt8 periodHour;
    qInt16 periodDuration;
};

inline StringStream& operator<<(StringStream& oStr, const DataStoreRatingLockInitParam& oRatingLockInitParam)
{
    oStr <<_T("DataStoreRatingLockInitParam: (")
         << _T("lockType=") << oRatingLockInitParam.lockType
         << _T(", periodHour=") << oRatingLockInitParam.periodHour
         << _T(", periodDuration=") << oRatingLockInitParam.periodDuration
         << _T(")");

    return oStr;
}

END_NAMESPACE_NEX

#endif
