﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SERVICE_HTTP_CLIENT_H_
#define SERVICE_HTTP_CLIENT_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

class Credentials;

class ServiceHttpClient : public RootObject
{
public:

    /* Please see man pages for details 
      
    */
    enum PriorityType
    {
        PRIORITY_TYPE_UNKNOWN    = 0,    //
        PRIORITY_TYPE_FOREGROUND = 1,    //
        PRIORITY_TYPE_BACKGROUND = 2     //
    };

    ServiceHttpClient();

    virtual ~ServiceHttpClient();

    // Always returns NULL. Provided for compatibility with `ServiceClient`.
    Credentials* GetCredentials() const
    {
        return NULL;
    }
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    /* Please see man pages for details 
        

            
            
            
            

        
        
        

        
        
    */
    virtual qBool SetSandboxContext(qUnsignedInt32 gameServerId, const qChar* pAccessKey, qUnsignedInt8 accountSlotNo);
#else
    /* Please see man pages for details 
        

            
            
            
            

        
        

        
        
    */
    virtual qBool SetSandboxContext(qUnsignedInt32 gameServerId, const qChar* pAccessKey);
#endif

    /* Please see man pages for details 
        
        

            

        
        
    */
    void SetUserContext(qUnsignedInt32 principalId, const String& password)
    {
        m_PrincipalId = principalId;
        m_Password = password;
    }

    /* Please see man pages for details 
        

       
       
       

        
    */
    void SetHppHttpThreadPriority(qUnsignedInt32 priority)
    {
        m_HppHttpThreadPriority = priority;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetHppHttpThreadPriority() const
    {
        return m_HppHttpThreadPriority;
    }

    /* Please see man pages for details 
        
        

       
       

        
    */
    void SetHppUrl(const String& hppUrl)
    {
        m_HppUrl = hppUrl;
        QLOG(EventLog::Info, _T("HPP URL has been changed: ") << m_HppUrl);
    }

    /* Please see man pages for details 
        

        
    */
    String GetHppUrl() const
    {
        return m_HppUrl;
    }

    /* Please see man pages for details 
        
        

            
            

        
    */
    void SetUseHttps(qBool useHttps)
    {
        m_UseHttps = useHttps;
    }

    /* Please see man pages for details 
        

        
    */
    qBool GetUseHttps() const
    {
        return m_UseHttps;
    }

    /* Please see man pages for details 
        

            

        
    */
    void SetHppTransferTimeout(TimeInterval timeoutMilliSecond)
    {
        m_TimeoutMilliSecond = timeoutMilliSecond;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetHppTransferTimeout() const
    {
        return m_TimeoutMilliSecond;
    }

    /* Please see man pages for details 
        
        

            

        
    */
    void SetSkipAuthentication(qBool skipAuthentication)
    {
        (void)skipAuthentication;
    }

    /* Please see man pages for details 
      

      
      

      
    */
    void SetPriorityType(ServiceHttpClient::PriorityType priorityType) { m_PriorityType = priorityType; }

    /* Please see man pages for details 
      

      
    */
    ServiceHttpClient::PriorityType GetPriorityType() const { return m_PriorityType; }

    /// Default value for the priority of the HTTP communication thread used to communicate with HTTP to PRUDP Proxy.
    static const qUnsignedInt32 DEFAULT_HTTP_THREAD_PRIORITY = 16;

    /// The default timeout value used for HTTP to PRUDP Proxy communication. (in milliseconds)
    static const qUnsignedInt32 DEFAULT_HPP_TRANSFER_TIMEOUT = 60 * 1000;
protected:
    String MakeHppUrl() const;

    qUnsignedInt32 m_GameId;
    String m_AccessKey;
    PrincipalID m_PrincipalId;
    String m_Password;
    qUnsignedInt32 m_HppHttpThreadPriority;
    String m_HppUrl;
    qBool m_UseHttps;
    TimeInterval m_TimeoutMilliSecond;
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    qUnsignedInt8 m_AccountSlotNo;
    String m_NfsEnv;            // The Cafe specification (only for Cafe) is to get a string, rather than an enum.
#else
    qUnsignedInt32 m_NfsType;   // Do not want the NFS type to be public so substitute qUnsignedInt32
    qUnsignedInt8 m_NfsNumber;
#endif
    ServiceHttpClient::PriorityType m_PriorityType;
private:
    qBool InitializeNfsLibrary() const;
    void FinalizeNfsLibrary() const;
    qBool GetPrincipalId(qUnsignedInt32& outPrincipalId) const;
    qBool GetPassword(String& outPassword) const;
#if defined(CAFE_TARGET) || defined(CAFEIOP_TARGET)
    qBool GetNfsSettings(String& nfsEnv) const;
#else
    qBool GetNfsSettings(qUnsignedInt32& nfsType, qUnsignedInt8& nfsNumber) const;
#endif
    qBool SetSandboxContextImpl(qUnsignedInt32 gameId, const qChar* pAccessKey);
};

END_NAMESPACE_NEX

#endif /* SERVICE_HTTP_CLIENT_H_ */
