﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __MatchmakeConstants_h__
#define __MatchmakeConstants_h__

BEGIN_NAMESPACE_NEX

/// The number of attributes that can be specified in a matchmaking session
static const qUnsignedInt32 NUM_MATCHMAKE_SESSION_ATTRIBUTES = 6;

/// Index of base attribute value when selecting matchmaking session most closely matching conditions.
static const qUnsignedInt32 MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX = 1;

/// Index of base attribute value when selecting matchmaking session with range broadening.
static const qUnsignedInt32 MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX = MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX;

//Invalid persistent gathering code.
static const qUnsignedInt64 INVALID_PERSISTENT_GATHERING_CODE = 0;

//Upper limit of persistent gathering registrations per person.
static const qUnsignedInt32 PERSISTENT_GATHERING_CREATION_MAX = 4;

//Upper limit of persistent gathering participations per person.
static const qUnsignedInt32 PERSISTENT_GATHERING_PARTICIPATION_MAX = 16;

//Maximum number of chat participants in a persistent gathering. The maximum number of participants must be set at or below this number.
static const qUnsignedInt64 PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX = 32;

#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
/// Invalid community code
static const qUnsignedInt64 INVALID_COMMUNITY_CODE = INVALID_PERSISTENT_GATHERING_CODE;

/// Upper limit for community registrations for a single person
static const qUnsignedInt32 COMMUNITY_CREATION_MAX = PERSISTENT_GATHERING_CREATION_MAX;

/// Upper limit for community participation for a single person
static const qUnsignedInt32 COMMUNITY_PARTICIPATION_MAX = PERSISTENT_GATHERING_PARTICIPATION_MAX;

//Maximum number of chat participants in a community. The maximum number of participants must be set at or below this number.
static const qUnsignedInt64 COMMUNITY_CHAT_PARTICIPANTS_MAX = PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX;
#endif

/// Maximum value of the number of attribute value elements when searching matchmaking sessions for multiple attribute values.
static const qUnsignedInt32 SETATTIBUTE_VECTOR_SIZE_MAX = 100;

/// Maximum game progress score.
static const qUnsignedInt8 MAX_PROGRESS_SCORE = 100;

/// Minimum update interval (in seconds) for game progress score.
static const qUnsignedInt32 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME = 30;

// Maximum user password length for a matchmaking session.
static const qUnsignedInt32 MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH = 32;

// Matchmaking session system password length.
static const qUnsignedInt32 MATCHMAKE_SESSION_SYSTEM_PASSWORD_LENGTH = 16;


extern const qChar* MATCHMAKE_PARAM_KEY_SETTING_INDEX;
extern const qChar* MATCHMAKE_PARAM_KEY_NINTENDO_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_OVERRIDE_IP_ADDRESS;
extern const qChar* MATCHMAKE_PARAM_KEY_USE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_RATING_VALUE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISCONNECTION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_VIOLATION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_LATITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_LONGITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISTANCE;
extern const qChar* MATCHMAKE_PARAM_KEY_TOTAL_SCORE;
extern const qChar* MATCHMAKE_PARAM_KEY_GEO_IP_RESULT;
extern const qChar* MATCHMAKE_PARAM_KEY_UPDATE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_SERVER_RETURNED;
extern const qChar* MATCHMAKE_PARAM_KEY_LAST_GID_FOR_PARTICIPATION_CHECK;

/*!
 @brief    The matchmaking type enumerators.

 Currently, you can specify MATCHMAKE_SYSTEM_TYPE_ANYBODY, MATCHMAKE_SYSTEM_TYPE_FRIEND, and MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING (MATCHMAKE_SYSTEM_TYPE_COMMUNITY).
 */
typedef enum
{
    MATCHMAKE_SYSTEM_TYPE_INVALID,         //!< Disabled.
    MATCHMAKE_SYSTEM_TYPE_ANYBODY,         //!< Anybody can participate.
    MATCHMAKE_SYSTEM_TYPE_FRIEND,          //!< Only the owner and friends can join.
    MATCHMAKE_SYSTEM_TYPE_FRIEND_INVITE,   //!< Only invited users who are friends with the owner can join (not usable).
    MATCHMAKE_SYSTEM_TYPE_ANYBODY_INVITE,  //!< Only invited users can participate (not usable).
    // Shift from Community -> PersistentGathering. Same as community match.
    MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //!< Matches persistent gatherings. Exists for compatibility.
    // If CTR (includes generating documents) || If not CTR, when not generating documents.
#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_COMMUNITY = MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //!< Matches communities. (Communities will be eliminated, so we recommend using persistent gatherings.)
#endif
    // If Cafe (includes generating documents) || If not Cafe, when not generating documents.
#if defined(CAFE_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY,   //!< Matches Miiverse communities. Exists for compatibility.
#endif
    MATCHMAKE_SYSTEM_TYPE_END              //!< End of enumerator (not usable).
}
MatchmakeSystemType;

/*!
 @brief    Methods for selecting which among the discovered matchmaking sessions to join.

       When you use the auto matchmaking feature and more than one matchmaking session that matches the search criteria is discovered, the game server decides which session to join based on the method specified by the <tt>MatchmakeSessionSearchCriteria::SetSelectionMethod</tt> function. The default is RANDOM.
       Use the <tt>MatchmakeSessionSearchCriteria::SetSelectionMethod</tt> function to specify a <tt>MatchmakeSelectionMethod</tt> value.
       If more than 50 matchmaking sessions match the search criteria, the selection method specified by the <tt>MatchmakeSessionSearchCriteria::SetSelectionMethod</tt> function is applied to any of 50 of the sessions.

       Specify MATCHMAKE_SELECTION_METHOD_RANDOM to randomly select a session.

       Specify MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR to select the matchmaking session whose attribute is nearest in value to the attribute of the <tt>MatchmakeSession</tt> object passed to the argument of the <tt>MatchmakeExtensionClient::AutoMatchmake</tt> function.
       The value used is the MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR index attribute value.
       If more than one matchmaking session has the same index attribute value, the matchmake session is selected in the order of their gathering IDs.
       If over 50 matchmaking sessions match the search criteria, then the matchmaking session with the nearest attribute value will not necessarily be one that is selected.

       Specify MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE to use range broadening to select the matchmaking session.
       A range here refers to a single interval of values among what are any number of intervals, created by dividing a series of integers using some arbitrary threshold value. 
       The default is based on the Elo rating system, where the integers from 0 to 10000 are divided into 12 intervals, each of which is a range. The ranges are 0-1100, 1101-1240, 1241-1400, 1401-1520, 1521-1620, 1621-1720, 1721-1815, 1816-1925, 1926-2040, 2041-2180, 2181-2300 and 2301-10000.
       When range broadening is chosen as the selection method, the matchmaking session is selected randomly from among those that match the search criteria and whose attribute values fall within the same range as the attribute of the <tt>MatchmakeSession</tt> object passed to the argument of the <tt>MatchmakeExtensionClient::AutoMatchmake</tt> function. 
       The value used is the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value.
       If no matchmaking session in the same range can be found, the range is broadened to some optional extent to encompass values in nearby ranges. Once the range has been broadened this way, the matchmaking session that matches the search criteria and whose attribute is nearest in value to the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value is selected.
       The default is to broaden the range to encompass the two range intervals above and below the current range.
       For example, if the range was set to 1401-1520 and no matchmaking session could be found, then the range will be broadened to 1101-1720.
       As an example of how to use this feature, consider a game where players are ranked according to their rating scores. Players with nearby rankings can be matched by using the rating score for the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value and setting ranges of rankings.

       Specify MATCHMAKE_SELECTION_METHOD_PROGRESS_SCORE to select the matchmaking session that matches the search criteria and has the highest game-progress score, as set by the <tt>MatchmakeSession::SetProgressScore</tt> function or the <tt>MatchmakeExtensionClient::UpdateProgressScore</tt> function. 
       This feature is useful for matchmaking sessions that accept new participants even after a battle has begun and lets them join in when the next battle begins. 
       In this case, new participants can save time for waiting for the next battle by joining the session where the battle is ending soon, instead of the session where a battle has just begun.
       To realize this feature, you set the game progress in the session to 0 when a battle begins and periodically advance the progress using the <tt>MatchmakeExtensionClient::UpdateProgressScore</tt> function as the battle progresses.
       Making frequent calls to <tt>MatchmakeExtensionClient::UpdateProgressScore</tt> puts a load on the game server. 
       So update the game progress at intervals of UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME seconds at the shortest.

       Specify MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE to select the matchmaking session using a combination of range broadening and the game progress.
       The matchmaking session with the highest game progress score is selected from among those that match the search criteria and whose attribute values fall within the same range as the attribute of the <tt>MatchmakeSession</tt> object passed to the argument of the <tt>MatchmakeExtensionClient::AutoMatchmake</tt> function. 
       The value used is the MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX index attribute value.
       If no matchmaking session in the same range can be found, then the range is broadened to encompass values in nearby ranges, after which the matchmaking session with the highest game progress score is selected from among those matchmaking sessions that match the search criteria in this new broadened range.

       Specify MATCHMAKE_SELECTION_METHOD_SCORE_BASED to select the optimal matchmaking session using a score based on various parameters. 
       Define score-conversion setting using a score for each parameter value, and then select the matchmaking session that matches the search criteria and has the highest total score when its parameters are compared with those of the local station.
       The following parameters can be used. <br />

       @li Difference in rating value<br>
       @li Difference in drop rate<br>
       @li Difference in violation rate<br>
       @li Distance in terms of Internet route<br>
       @li Country match<br>
       @li Country match when there is no distance information<br>
       @li Temporal continuance of matchmaking session<br>
       @li Progress of matchmaking session<br>

       <br />
       You can make use of a number of kinds of score-conversion settings, switching between indexes of settings.
       For example, you can switch between indexes based on the game mode to change how scores are assigned.
       If you specify multiple MatchmakeSessionSearchCriteria , you can use a strict score-conversion setting for the first set of criteria, and a looser score-conversion setting for the second set of criteria, using these to search for the appropriate matchmaking session. 
       To use this function, you need to use the overload of the <tt>MatchmakeSessionSearchCriteria::SetSelectionMethod</tt> function that takes MatchmakeParam for an argument. In addition, the MatchmakeParam object specified by this function must also be specified for the MatchmakeSession object used by the <tt>MatchmakeSession::SetMatchmakeParam</tt> function.
       The local station's rating value, drop rate, violation rate and country information must be set by the application for MatchmakeParam.
       If the application calculates the rating value, drop rate, violation rate, consider using MatchmakeReferee.



































*/
typedef enum
{
    MATCHMAKE_SELECTION_METHOD_RANDOM = 0,        //!< Makes a random selection.
    MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR,  //!< Selects the matchmaking session with the closest attribute value.
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE,     //!< Selects using range broadening.
    MATCHMAKE_SELECTION_METHOD_PROGRESS_SCORE,    //!< Selects the matchmaking session with the highest game progress score.
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE,  //!< Selects using a combination of range broadening and game progress score.
    MATCHMAKE_SELECTION_METHOD_SCORE_BASED        //!< Selects the optimal session after scoring various parameters. 
}
MatchmakeSelectionMethod;

/*!
 @brief    The result of getting the position information.
*/
typedef enum
{
    MATCHMAKE_GEO_IP_RESULT_INVALID = 0,    //!< Not used. Can be used as an invalid value.
    MATCHMAKE_GEO_IP_RESULT_FOUND,          //!< Found the position information.
    MATCHMAKE_GEO_IP_RESULT_NOT_FOUND,      //!< Could not find the position information.
    MATCHMAKE_GEO_IP_RESULT_UNUSED          //!< This setting does not use position information.
}
MATCHMAKE_GEO_IP_RESULT;


// CreateMatchmakeSessionOption, JoinMatchmakeSessionOption, and AutoMatchmakeOption look at the values defined in MatchmakeOption and use the same values for items with the same meaning.
/*!
 :private
 @brief    Matchmaking session-related option bit flags.
*/
enum MatchmakeOption
{
    MATCHMAKE_OPTION_NONE                           = 0x00000000, //!< No options specified.
    MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = 0x00000001, //!< Takes the gathering ID specified to check for previous participation and saves it to the matchmaking session's MatchmakeParam. Not saved for a persistent gatherings.
    MATCHMAKE_OPTION_DONT_CARE_MY_BLACKLIST         = 0x00000002 //!< Set this to "true" to force the local station to join the session even if there is a participant who is on the local station's blacklist.
};

/*!
 :private
 @brief    Option bit flags when creating a matchmaking session.
*/
enum CreateMatchmakeSessionOption
{
    CREATE_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< No options specified.
    CREATE_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //!< Takes the gathering ID specified to check for previous participation and saves it to the matchmaking session's MatchmakeParam. Not saved for a persistent gatherings.
};

/*!
 :private
 @brief    Option bit flags when joining a matchmaking session.
*/
enum JoinMatchmakeSessionOption
{
    JOIN_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< No options specified.
    JOIN_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK, //!< Takes the gathering ID specified to check for previous participation and saves it to the matchmaking session's MatchmakeParam. Not saved for a persistent gatherings.
    JOIN_MATCHMAKE_SESSION_OPTION_DONT_CARE_MY_BLACKLIST         = MATCHMAKE_OPTION_DONT_CARE_MY_BLACKLIST //!< Set this to "true" to force the local station to join the session even if there is a participant who is on the local station's blacklist.
};

/*!
 :private
 @brief    Option bit flags when automatic matchmaking.
*/
enum AutoMatchmakeOption
{
    AUTO_MATCHMAKE_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //!< No options specified.
    AUTO_MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //!< Takes the gathering ID specified to check for previous participation and saves it to MatchmakeParam. 
};

/*!
 @brief    Flags related to the behavior when joining.
*/
enum JoinMatchmakeSessionBehavior
{
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_JOIN_MYSELF,                      //!< The local station also joins. If already joined, the result is QERROR(RendezVous, AlreadyParticipatedGathering).
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_IM_ALREADY_JOINED                 //!< The local station does not join because already participating. If not already joined, the result is QERROR(RendezVous, NotParticipatedGathering). This can be used to get others to join a matchmaking session the local station has already joined.
};

/*!
 @brief    Bit flags indicating which items to update.
*/
enum MatchmakeSessionModificationFlag
{
    MATCHMAKE_SESSION_MODIFICATION_FLAG_NONE                        = 0x00000000,   //!< Do not update any items.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_ATTRIBUTES                  = 0x00000001,   //!< Attribute.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPEN_PARTICIPATION          = 0x00000002,   //!< Setting for closing the session.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_APPLICATION_BUFFER          = 0x00000004,   //!< Application-defined data.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_PROGRESS_SCORE              = 0x00000008,   //!< Progress.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPTION0                     = 0x00000010,   //   Option0 (not used).
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM             = 0x00000020,   //!< Matchmaking parameters. Replaces all parameters. Cannot be used at the same time as MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE    = 0x00000040,   //!< Matchmaking parameters. Updates existing parameters and adds new parameters. Cannot be used at the same time as MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_STARTED_TIME                = 0x00000080,   //!< The time the session started.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_USER_PASSWORD               = 0x00000100,   //!< User password.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_GAME_MODE                   = 0x00000200,   //!< Game mode.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_DESCRIPTION                 = 0x00000400,   //!< Description string.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MIN_PARTICIPANTS            = 0x00000800,   //!< Minimum number of participants
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MAX_PARTICIPANTS            = 0x00001000,   //!< Maximum number of participants.
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_SYSTEM_TYPE       = 0x00002000    //!< Matchmaking system type.
};

namespace BuiltInParticipationPolicies
{
static const qUnsignedInt32 FriendOnlyParticipationPolicy = FirstUserPolicy - 2;        // Only friends can join
static const qUnsignedInt32 CommunityParticipationPolicy = FirstUserPolicy - 3;         // Community
static const qUnsignedInt32 MatchmakeSessionParticipationPolicy = FirstUserPolicy - 4;  // Matchmake Session
static const qUnsignedInt32 AnybodyParticipationPolicy          = FirstUserPolicy - 5;  // A matchmake session that anyone can join
}

/*!
 @brief    Option set when it is a matchmaking session that anyone can join.
 */
namespace AnybodyParticipationPolicyArgument
{
/*!
 @brief    Option set when it is a matchmaking session that anyone can join.
 */
typedef enum
{
    WithoutClose          = 0, //!< Do not change the recruit status of the matchmaking session even if the GatheringFlags::MigrateOwner flag is set and the owner changes automatically.
    CloseOnOwnerMigration = 1, //!< Automatically close the matchmaking session when the owner has changed automatically because of the GatheringFlags::MigrateOwner flag.
    End
}
AnybodyParticipationPolicyArgument;
}

/*!
 @name Options specified when matchmaking with friends,
 */
namespace FriendOnlyParticipationPolicyArgument
{
/*!
 @name Options specified when matchmaking with friends,
 */
typedef enum
{
    CloseOnOwnerMigration, //!< Automatically close the matchmaking session when the owner has changed automatically because of the GatheringFlags::MigrateOwner flag.
    WithoutClose,          //!< Do not change the recruit status of the matchmaking session even if the GatheringFlags::MigrateOwner flag is set and the owner changes automatically.
    End
}
FriendOnlyParticipationPolicyArgument;
}

END_NAMESPACE_NEX

#endif  // __MatchmakeConstants_h__
