﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NATTraversalRelayClient_H_
#define _NATTraversalRelayClient_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>

#include <RendezVous/Core/src/Client/Credentials.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>

#include <OnlineCore/src/Transport/Interface/NATRelayInterface.h>
#include <RendezVous/Services/NATTraversal/src/Common/NATTraversalRelayProtocol.h>


BEGIN_NAMESPACE_NEX


#ifdef SN_TARGET_NON_SUPPORT_A
#pragma control %push diag
#pragma diag_suppress=999
#endif
/*
 * Implements NATTraversalRelayProtocol for a client.
 *
 * Because class NATTraversalRelayProtocol: public DDLPROTOCOL(NATTraversalProtocolServer), Server is DDL.
 * 
 */

class NATTraversalRelayProtocolImpl: public NATTraversalRelayProtocol
{
public:
    NATTraversalRelayProtocolImpl() : NATTraversalRelayProtocol() {}
    virtual ~NATTraversalRelayProtocolImpl() {}
private:
    virtual void InitiateProbe(const DDLTYPE(stationurl)::CPPType& urlStationToProbe);
};


// Stopped inheritance of NATTraversalRelayProtocol by NATTraversalRelayClient, and moved that done by NATTraversalClient here. Also, NATRelayInterface in now in a form to register. RequestProbe is processed by NATTraversalRelayProtocol.
// 
// 
// 


class NATTraversalRelayClient: public ServiceClient
{
public:
    NATTraversalRelayClient();
    virtual ~NATTraversalRelayClient();

    void Init();

    void CallRequestProbeInitiation(ProtocolCallContext *pCallContext, const qList<StationURL> &lstTarget);
    void CallRequestProbeInitiationExt(ProtocolCallContext *pContext, const qList<StationURL> &urlProbeInitiationList, const StationURL& urlStationToProbe);
    void CallReportNATTraversalResult(ProtocolCallContext *pContext, const qUnsignedInt32 &pid, const qBool &result, const qUnsignedInt32 &rtt);
    void CallReportNATProperties(ProtocolCallContext *pContext, const qUnsignedInt32& natmapping, const qUnsignedInt32& natfiltering, const qUnsignedInt32& rtt);
    qBool CallGetRelaySignatureKey(ProtocolCallContext *pContext, qInt32 *relayType, DateTime* currentUTCTime,
            String* address, qUnsignedInt16* port, qInt32* relayAddressType, qUnsignedInt32* gameServerID);
    void CallReportNATTraversalResultDetail(ProtocolCallContext *pContext,  const qUnsignedInt32 &cid, const qBool &result,
            const NATTraversalResult &detail, qUnsignedInt32& rtt);

    void RegisterRelay(NATRelayInterface* pRelay)
    {
        m_pRelayInterface = pRelay;
        m_pRelayInterface->RegisterRelayClient(this);
    }

    void UnregisterRelay()
    {
        if (m_pRelayInterface)
        {
            m_pRelayInterface->UnregisterRelayClient();
            m_pRelayInterface = NULL;
        }
    }

    NATRelayInterface *GetInterface()
    {
        return m_pRelayInterface;
    }

    NATTraversalRelayProtocol *GetProtocol()
    {
        return m_pRelayProtocol;
    }

    virtual qBool IsConnected() const;

    virtual NATTraversalRelayProtocol* CreateNATTraversalRelayProtocol();

protected:
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);
private:
    virtual void ConnectionStateHasChanged();

private:
    NATRelayInterface*          m_pRelayInterface;
    NATTraversalRelayProtocol*  m_pRelayProtocol;
};

#ifdef SN_TARGET_NON_SUPPORT_A
#pragma control %pop diag
#endif

END_NAMESPACE_NEX

#endif
